package flatbuffersdata

import (
	"encoding/json"

	"newrelic"
	"newrelic/collector"
)

var (
	SampleAppInfo = newrelic.AppInfo{
		License:           collector.LicenseKey("0123456789abcdefghijklmnopqrstuvwxyz1234"),
		Appname:           "Application",
		AgentLanguage:     "php",
		AgentVersion:      "0.0.1",
		Settings:          map[string]interface{}{"newrelic.analytics_events.capture_attributes": "1", "newrelic.analytics_events.enabled": "1", "newrelic.appname": "one;two;three", "newrelic.attributes.enabled": "1", "newrelic.attributes.exclude": "no value", "newrelic.attributes.include": "no value", "newrelic.browser_monitoring.attributes.enabled": "0", "newrelic.browser_monitoring.attributes.exclude": "no value", "newrelic.browser_monitoring.attributes.include": "no value", "newrelic.browser_monitoring.auto_instrument": "1", "newrelic.browser_monitoring.capture_attributes": "0", "newrelic.browser_monitoring.debug": false, "newrelic.browser_monitoring.loader": "rum", "newrelic.capture_params": "0", "newrelic.cross_application_tracer.enabled": "1", "newrelic.custom_insights_events.enabled": "1", "newrelic.daemon.collector_host": "collector.newrelic.com", "newrelic.daemon.location": "/php/php_agent/releases/linux/daemon/newrelic-daemon", "newrelic.daemon.logfile": "/opt/nr/logs/newrelic-daemon.log", "newrelic.daemon.loglevel": "verbosedebug", "newrelic.daemon.port": "/tmp/.newrelic.sock", "newrelic.daemon.special.integration": "1", "newrelic.enabled": "1", "newrelic.error_collector.attributes.enabled": "1", "newrelic.error_collector.attributes.exclude": "no value", "newrelic.error_collector.attributes.include": "no value", "newrelic.error_collector.capture_attributes": "1", "newrelic.error_collector.enabled": "1", "newrelic.error_collector.prioritize_api_errors": "0", "newrelic.error_collector.record_database_errors": "1", "newrelic.feature_flag": "no value", "newrelic.framework": "no value", "newrelic.framework.drupal.modules": "1", "newrelic.high_security": "0", "newrelic.ignored_params": "no value", "newrelic.labels": "no value", "newrelic.license": "123456789", "newrelic.logfile": "/opt/nr/logs/php_agent.log", "newrelic.loglevel": "verbosedebug", "newrelic.special.expensive_node_min": "0", "newrelic.special.max_nesting_level": "-1", "newrelic.synthetics.enabled": "1", "newrelic.transaction_events.attributes.enabled": "1", "newrelic.transaction_events.attributes.exclude": "no value", "newrelic.transaction_events.attributes.include": "no value", "newrelic.transaction_events.enabled": "1", "newrelic.transaction_tracer.attributes.enabled": "1", "newrelic.transaction_tracer.attributes.exclude": "no value", "newrelic.transaction_tracer.attributes.include": "no value", "newrelic.transaction_tracer.capture_attributes": "1", "newrelic.transaction_tracer.custom": "no value", "newrelic.transaction_tracer.detail": "1", "newrelic.transaction_tracer.enabled": "1", "newrelic.transaction_tracer.explain_enabled": "1", "newrelic.transaction_tracer.explain_threshold": "1", "newrelic.transaction_tracer.record_sql": "obfuscated", "newrelic.transaction_tracer.slow_sql": "1", "newrelic.transaction_tracer.stack_trace_threshold": "1", "newrelic.transaction_tracer.threshold": "1", "newrelic.webtransaction.name.files": "no value", "newrelic.webtransaction.name.functions": "no value", "newrelic.webtransaction.name.remove_trailing_path": "0"},
		Environment:       newrelic.JSONString(`[]`),
		HighSecurity:      false,
		Labels:            newrelic.JSONString(`[]`),
		RedirectCollector: "collector.newrelic.com",
	}
	SampleTxn = Txn{
		Name:             "WebTransaction/Action/heyo",
		SamplingPriority: 0.80000,
		Metrics:          SampleMetrics,
		Errors:           []*newrelic.Error{SampleError},
		Trace:            SampleTrace,
		SlowSQLs:         SampleSlowSQLs,
		AnalyticEvent:    SampleAnalyticEvent,
		CustomEvents:     SampleCustomEvents,
		ErrorEvents:      SampleErrorEvents,
		SpanEvents:       SampleSpanEvents,
	}
	SampleAnalyticEvent = json.RawMessage(`[{"type":"Transaction","name":"OtherTransaction\/php\/php\/fake\/heyo.php","timestamp":1421910689.96993,"duration":2.06631},{"zip":"zap"},{}]`)
	SampleCustomEvents  = []json.RawMessage{
		json.RawMessage(`[{"x":1}]`),
		json.RawMessage(`[{"x":2}]`),
		json.RawMessage(`[{"x":3}]`),
	}
	SampleErrorEvents = []json.RawMessage{
		json.RawMessage(`[{"type": "TransactionError",` +
			`"timestamp": 1445290225.1948,"error.class": "HeyoException",` +
			`"error.message": "Uncaught exception 'HeyoException' with message 'foo!' in /Users/earnold/workspace/php_integration_tests/integration/errors/heyo.php:6",` +
			`"transactionName": "OtherTransaction/php/heyo.php","duration": 0.00101,"nr.transactionGuid": "390b8adab3b435c2"}]`),
	}
	SampleSpanEvents = []json.RawMessage{
		json.RawMessage(`[{"Span1":1}]`),
		json.RawMessage(`[{"Span2":2}]`),
		json.RawMessage(`[{"Span3":3}]`),
	}
	SampleError = &newrelic.Error{
		Priority: 100,
		Data:     newrelic.JSONString(`[1378167,"mytxn","myMessage","myClass",{"stack_trace":["zip","zap"]}]`),
	}
	SampleTrace = &newrelic.TxnTrace{
		MetricName:           "WebTransaction/Action/heyo",
		RequestURI:           "alpha/beta/gamma",
		UnixTimestampMillis:  123456.123456,
		DurationMillis:       2001,
		Data:                 newrelic.JSONString("[[0,{},{},[0,2001,\"ROOT\",{},[[0,2001,\"`0\",{},[[0,2001,\"`1\",{},[]]]]]],{\"agentAttributes\":{\"SERVER_NAME\":\"\"},\"intrinsics\":{\"cpu_time\":0.00200,\"cpu_user_time\":0.00200,\"cpu_sys_time\":0.00000}}],[\"OtherTransaction\\/php\\/php\\/fake\\/heyo.php\",\"Custom\\/alpha\"]]"),
		GUID:                 "abcdef0123456789",
		ForcePersist:         false,
		SyntheticsResourceID: "1234",
	}
	SampleSlowSQLs = []*newrelic.SlowSQL{
		&newrelic.SlowSQL{
			MetricName:  "Database/my_table/insert",
			ID:          newrelic.SQLId(1),
			Count:       20,
			TotalMicros: 1000,
			MinMicros:   25,
			MaxMicros:   75,
			Params:      newrelic.JSONString(`{"backtrace":["zip", "zap"]}`),
			Query:       "select * from heyo",
			TxnName:     "WebTransaction/Action/zap",
			TxnURL:      "zip/zap/zop",
		},
		&newrelic.SlowSQL{
			MetricName:  "Database/big_table/delete",
			ID:          newrelic.SQLId(2),
			Count:       10,
			TotalMicros: 2000,
			MinMicros:   150,
			MaxMicros:   250,
			Params:      newrelic.JSONString(`{"backtrace":["bink", "bonk"]}`),
			Query:       "select * from alpha",
			TxnName:     "WebTransaction/Action/bink",
			TxnURL:      "alpha/beta/gamma",
		},
		&newrelic.SlowSQL{
			MetricName:  "Database/other_table/insert",
			ID:          newrelic.SQLId(3),
			Count:       1,
			TotalMicros: 15,
			MinMicros:   15,
			MaxMicros:   15,
			Params:      newrelic.JSONString(`{"backtrace":["green", "yellow"]}`),
			Query:       "insert ? into other_table",
			TxnName:     "WebTransaction/Action/other",
			TxnURL:      "green/yellow/other",
		},
	}
	SampleMetrics = []metric{
		metric{Name: "apdex satisfied", Data: [6]float64{2, 0, 0, 8, 9, 0}, Scoped: true},
		metric{Name: "one", Data: [6]float64{2, 4, 2, 2, 2, 8}, Scoped: true},
		metric{Name: "three", Data: [6]float64{1, 4, 2, 4, 4, 6}, Scoped: true},
		metric{Name: "Apdex", Data: [6]float64{332, 26, 3, 0.5, 0.5, 0}, Scoped: true},
		metric{Name: "CPU/User Time", Data: [6]float64{361, 2.9386199999999874, 0, 0, 0.06499, 0.14663000000000007}, Scoped: true},
		metric{Name: "CPU/User/Utilization", Data: [6]float64{361, 62.965800000000016, 0, 0, 1.28535, 54.57723999999999}, Scoped: true},
		metric{Name: "Datastore/MySQL/all", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}, Scoped: true},
		metric{Name: "Datastore/MySQL/allWeb", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}, Scoped: true},
		metric{Name: "Datastore/all", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}, Scoped: true},
		metric{Name: "Datastore/allWeb", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}, Scoped: true},
		metric{Name: "Datastore/operation/MySQL/insert", Data: [6]float64{53000, 23.411430000000006, 23.411430000000006, 4e-05, 0.22667, 0.6342200000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City100/insert", Data: [6]float64{31, 0.8240199999999998, 0.8240199999999998, 8e-05, 0.20662, 0.13516}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1000/insert", Data: [6]float64{31, 0.0036000000000000025, 0.0036000000000000025, 8e-05, 0.00024, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1001/insert", Data: [6]float64{31, 0.0077300000000000025, 0.0077300000000000025, 8e-05, 0.00419, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1002/insert", Data: [6]float64{31, 0.029580000000000002, 0.029580000000000002, 8e-05, 0.02557, 0.00065}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1003/insert", Data: [6]float64{31, 0.045639999999999986, 0.045639999999999986, 8e-05, 0.02688, 0.00093}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1004/insert", Data: [6]float64{31, 0.01601, 0.01601, 8e-05, 0.01236, 0.00015}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1005/insert", Data: [6]float64{31, 0.0037800000000000025, 0.0037800000000000025, 8e-05, 0.00032, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1006/insert", Data: [6]float64{31, 0.0038200000000000013, 0.0038200000000000013, 8e-05, 0.0003, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1007/insert", Data: [6]float64{31, 0.01021, 0.01021, 8e-05, 0.00634, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1008/insert", Data: [6]float64{31, 0.03393999999999999, 0.03393999999999999, 7e-05, 0.02871, 0.00082}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1009/insert", Data: [6]float64{31, 0.003740000000000002, 0.003740000000000002, 8e-05, 0.0003, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City101/insert", Data: [6]float64{31, 0.01243, 0.01243, 8e-05, 0.00422, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1010/insert", Data: [6]float64{31, 0.0066900000000000015, 0.0066900000000000015, 8e-05, 0.00279, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1011/insert", Data: [6]float64{31, 0.004310000000000003, 0.004310000000000003, 8e-05, 0.00041, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1012/insert", Data: [6]float64{31, 0.021939999999999998, 0.021939999999999998, 8e-05, 0.01815, 0.00033}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1013/insert", Data: [6]float64{31, 0.01472, 0.01472, 8e-05, 0.01084, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1014/insert", Data: [6]float64{31, 0.01627, 0.01627, 8e-05, 0.0123, 0.00015}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1015/insert", Data: [6]float64{31, 0.003930000000000001, 0.003930000000000001, 8e-05, 0.00036, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1016/insert", Data: [6]float64{31, 0.0060500000000000016, 0.0060500000000000016, 8e-05, 0.00213, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1017/insert", Data: [6]float64{31, 0.026320000000000003, 0.026320000000000003, 7e-05, 0.02105, 0.00044}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1018/insert", Data: [6]float64{31, 0.03601999999999999, 0.03601999999999999, 7e-05, 0.02726, 0.00075}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1019/insert", Data: [6]float64{31, 0.018070000000000003, 0.018070000000000003, 8e-05, 0.01439, 0.00021}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City102/insert", Data: [6]float64{31, 0.01621, 0.01621, 7e-05, 0.00282, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1020/insert", Data: [6]float64{31, 0.0071400000000000005, 0.0071400000000000005, 8e-05, 0.00162, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1021/insert", Data: [6]float64{31, 0.011030000000000002, 0.011030000000000002, 8e-05, 0.00717, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1022/insert", Data: [6]float64{31, 0.02203, 0.02203, 7e-05, 0.01506, 0.00024}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1023/insert", Data: [6]float64{31, 0.016520000000000003, 0.016520000000000003, 7e-05, 0.00876, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1024/insert", Data: [6]float64{31, 0.0042600000000000025, 0.0042600000000000025, 7e-05, 0.00049, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1025/insert", Data: [6]float64{31, 0.006030000000000001, 0.006030000000000001, 7e-05, 0.00228, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1026/insert", Data: [6]float64{31, 0.004180000000000001, 0.004180000000000001, 8e-05, 0.00038, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1027/insert", Data: [6]float64{31, 0.05949999999999998, 0.05949999999999998, 7e-05, 0.03733, 0.00157}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1028/insert", Data: [6]float64{31, 0.008480000000000003, 0.008480000000000003, 8e-05, 0.00289, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1029/insert", Data: [6]float64{31, 0.003920000000000001, 0.003920000000000001, 8e-05, 0.00032, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City103/insert", Data: [6]float64{31, 0.02198, 0.02198, 7e-05, 0.00749, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1030/insert", Data: [6]float64{31, 0.004950000000000003, 0.004950000000000003, 8e-05, 0.00082, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1031/insert", Data: [6]float64{31, 0.004530000000000002, 0.004530000000000002, 8e-05, 0.00077, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1032/insert", Data: [6]float64{31, 0.012320000000000001, 0.012320000000000001, 8e-05, 0.00821, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1033/insert", Data: [6]float64{31, 0.008360000000000003, 0.008360000000000003, 7e-05, 0.0044, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1034/insert", Data: [6]float64{31, 0.003880000000000002, 0.003880000000000002, 8e-05, 0.00035, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1035/insert", Data: [6]float64{31, 0.057259999999999964, 0.057259999999999964, 7e-05, 0.01974, 0.0009699999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1036/insert", Data: [6]float64{31, 0.004550000000000001, 0.004550000000000001, 7e-05, 0.00065, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1037/insert", Data: [6]float64{31, 0.013670000000000002, 0.013670000000000002, 7e-05, 0.00582, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1038/insert", Data: [6]float64{31, 0.021030000000000004, 0.021030000000000004, 8e-05, 0.0141, 0.0002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1039/insert", Data: [6]float64{31, 0.009160000000000001, 0.009160000000000001, 7e-05, 0.00482, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City104/insert", Data: [6]float64{31, 0.017650000000000006, 0.017650000000000006, 7e-05, 0.00563, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1040/insert", Data: [6]float64{31, 0.06812999999999997, 0.06812999999999997, 8e-05, 0.04324, 0.00231}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1041/insert", Data: [6]float64{31, 0.006720000000000002, 0.006720000000000002, 7e-05, 0.00329, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1042/insert", Data: [6]float64{31, 0.028050000000000002, 0.028050000000000002, 8e-05, 0.02368, 0.00056}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1043/insert", Data: [6]float64{31, 0.0060500000000000016, 0.0060500000000000016, 7e-05, 0.00234, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1044/insert", Data: [6]float64{31, 0.003940000000000002, 0.003940000000000002, 8e-05, 0.00031, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1045/insert", Data: [6]float64{31, 0.004240000000000002, 0.004240000000000002, 8e-05, 0.00035, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1046/insert", Data: [6]float64{31, 0.0042200000000000015, 0.0042200000000000015, 8e-05, 0.00046, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1047/insert", Data: [6]float64{31, 0.004210000000000003, 0.004210000000000003, 7e-05, 0.00046, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1048/insert", Data: [6]float64{31, 0.024540000000000003, 0.024540000000000003, 8e-05, 0.0206, 0.00042}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1049/insert", Data: [6]float64{31, 0.020670000000000004, 0.020670000000000004, 7e-05, 0.01454, 0.00022}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City105/insert", Data: [6]float64{31, 0.01565, 0.01565, 7e-05, 0.0056, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1050/insert", Data: [6]float64{31, 0.008220000000000002, 0.008220000000000002, 8e-05, 0.00462, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1051/insert", Data: [6]float64{31, 0.031020000000000002, 0.031020000000000002, 8e-05, 0.01083, 0.00027}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1052/insert", Data: [6]float64{31, 0.02222, 0.02222, 8e-05, 0.01579, 0.00025}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1053/insert", Data: [6]float64{31, 0.03722999999999999, 0.03722999999999999, 8e-05, 0.01795, 0.00054}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1054/insert", Data: [6]float64{31, 0.0234, 0.0234, 7e-05, 0.0181, 0.00033}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1055/insert", Data: [6]float64{31, 0.015080000000000001, 0.015080000000000001, 7e-05, 0.01072, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1056/insert", Data: [6]float64{31, 0.0041100000000000025, 0.0041100000000000025, 7e-05, 0.00036, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1057/insert", Data: [6]float64{31, 0.01841, 0.01841, 7e-05, 0.01095, 0.00013000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1058/insert", Data: [6]float64{31, 0.011840000000000002, 0.011840000000000002, 7e-05, 0.00403, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1059/insert", Data: [6]float64{31, 0.004110000000000002, 0.004110000000000002, 7e-05, 0.00026, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City106/insert", Data: [6]float64{31, 0.01449, 0.01449, 7e-05, 0.0053, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1060/insert", Data: [6]float64{31, 0.01948, 0.01948, 8e-05, 0.01028, 0.00014000000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1061/insert", Data: [6]float64{31, 0.015200000000000003, 0.015200000000000003, 8e-05, 0.00396, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1062/insert", Data: [6]float64{31, 0.00974, 0.00974, 8e-05, 0.00524, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1063/insert", Data: [6]float64{31, 0.0040900000000000025, 0.0040900000000000025, 7e-05, 0.00033, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1064/insert", Data: [6]float64{31, 0.006470000000000003, 0.006470000000000003, 7e-05, 0.00194, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1065/insert", Data: [6]float64{31, 0.006200000000000002, 0.006200000000000002, 7e-05, 0.00214, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1066/insert", Data: [6]float64{31, 0.004190000000000002, 0.004190000000000002, 7e-05, 0.00035, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1067/insert", Data: [6]float64{31, 0.01814, 0.01814, 7e-05, 0.01405, 0.0002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1068/insert", Data: [6]float64{31, 0.01565, 0.01565, 7e-05, 0.01139, 0.00013}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1069/insert", Data: [6]float64{31, 0.0057500000000000025, 0.0057500000000000025, 7e-05, 0.00191, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City107/insert", Data: [6]float64{31, 0.01267, 0.01267, 7e-05, 0.00386, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1070/insert", Data: [6]float64{31, 0.008950000000000003, 0.008950000000000003, 7e-05, 0.00462, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1071/insert", Data: [6]float64{31, 0.003880000000000001, 0.003880000000000001, 7e-05, 0.00043, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1072/insert", Data: [6]float64{31, 0.004090000000000002, 0.004090000000000002, 7e-05, 0.0003, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1073/insert", Data: [6]float64{31, 0.02066, 0.02066, 7e-05, 0.01499, 0.00022}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1074/insert", Data: [6]float64{31, 0.038029999999999994, 0.038029999999999994, 7e-05, 0.02756, 0.0008}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1075/insert", Data: [6]float64{31, 0.018249999999999995, 0.018249999999999995, 7e-05, 0.00967, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1076/insert", Data: [6]float64{31, 0.003860000000000001, 0.003860000000000001, 7e-05, 0.00038, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1077/insert", Data: [6]float64{31, 0.003570000000000002, 0.003570000000000002, 7e-05, 0.00025, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1078/insert", Data: [6]float64{31, 0.008020000000000001, 0.008020000000000001, 7e-05, 0.00362, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1079/insert", Data: [6]float64{31, 0.008150000000000003, 0.008150000000000003, 7e-05, 0.00448, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City108/insert", Data: [6]float64{31, 0.010989999999999998, 0.010989999999999998, 7e-05, 0.00215, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1080/insert", Data: [6]float64{31, 0.004220000000000001, 0.004220000000000001, 7e-05, 0.00054, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1081/insert", Data: [6]float64{31, 0.03114, 0.03114, 7e-05, 0.02802, 0.00078}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1082/insert", Data: [6]float64{31, 0.034849999999999985, 0.034849999999999985, 8e-05, 0.01574, 0.00039000000000000005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1083/insert", Data: [6]float64{31, 0.006560000000000002, 0.006560000000000002, 7e-05, 0.00313, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1084/insert", Data: [6]float64{31, 0.016800000000000002, 0.016800000000000002, 8e-05, 0.01069, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1085/insert", Data: [6]float64{31, 0.03347999999999999, 0.03347999999999999, 7e-05, 0.02705, 0.00074}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1086/insert", Data: [6]float64{31, 0.007150000000000002, 0.007150000000000002, 7e-05, 0.00303, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1087/insert", Data: [6]float64{31, 0.003560000000000002, 0.003560000000000002, 8e-05, 0.00026, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1088/insert", Data: [6]float64{31, 0.004210000000000001, 0.004210000000000001, 8e-05, 0.00057, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1089/insert", Data: [6]float64{31, 0.007280000000000001, 0.007280000000000001, 7e-05, 0.00263, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City109/insert", Data: [6]float64{31, 0.010090000000000002, 0.010090000000000002, 7e-05, 0.00167, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1090/insert", Data: [6]float64{31, 0.004620000000000003, 0.004620000000000003, 7e-05, 0.00047, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1091/insert", Data: [6]float64{31, 0.011540000000000007, 0.011540000000000007, 7e-05, 0.00513, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1092/insert", Data: [6]float64{31, 0.03387999999999997, 0.03387999999999997, 8e-05, 0.02608, 0.0006900000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1093/insert", Data: [6]float64{31, 0.017220000000000003, 0.017220000000000003, 7e-05, 0.01324, 0.00018}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City1094/insert", Data: [6]float64{31, 0.06561999999999998, 0.06561999999999998, 8e-05, 0.02835, 0.00158}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City110/insert", Data: [6]float64{31, 0.012260000000000004, 0.012260000000000004, 7e-05, 0.00199, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City111/insert", Data: [6]float64{31, 0.02963, 0.02963, 7e-05, 0.0206, 0.00042}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City112/insert", Data: [6]float64{31, 0.009950000000000002, 0.009950000000000002, 7e-05, 0.00159, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City113/insert", Data: [6]float64{31, 0.010130000000000002, 0.010130000000000002, 7e-05, 0.00177, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City114/insert", Data: [6]float64{31, 0.009220000000000002, 0.009220000000000002, 7e-05, 0.00174, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City115/insert", Data: [6]float64{31, 0.008430000000000002, 0.008430000000000002, 7e-05, 0.00106, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City116/insert", Data: [6]float64{31, 0.011060000000000002, 0.011060000000000002, 7e-05, 0.00229, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City117/insert", Data: [6]float64{31, 0.011550000000000003, 0.011550000000000003, 7e-05, 0.00215, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City118/insert", Data: [6]float64{31, 0.009450000000000002, 0.009450000000000002, 7e-05, 0.00118, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City119/insert", Data: [6]float64{31, 0.008730000000000002, 0.008730000000000002, 7e-05, 0.00118, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City120/insert", Data: [6]float64{31, 0.018600000000000005, 0.018600000000000005, 7e-05, 0.0102, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City121/insert", Data: [6]float64{31, 0.012660000000000003, 0.012660000000000003, 7e-05, 0.0047, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City122/insert", Data: [6]float64{31, 0.01788, 0.01788, 7e-05, 0.00367, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City123/insert", Data: [6]float64{31, 0.025110000000000004, 0.025110000000000004, 7e-05, 0.01601, 0.00027}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City124/insert", Data: [6]float64{31, 0.01037, 0.01037, 7e-05, 0.0018, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City125/insert", Data: [6]float64{31, 0.007809999999999999, 0.007809999999999999, 7e-05, 0.00104, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City126/insert", Data: [6]float64{31, 0.009050000000000002, 0.009050000000000002, 7e-05, 0.001, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City127/insert", Data: [6]float64{31, 0.018989999999999996, 0.018989999999999996, 7e-05, 0.01073, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City128/insert", Data: [6]float64{31, 0.008300000000000004, 0.008300000000000004, 7e-05, 0.00143, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City129/insert", Data: [6]float64{31, 0.008920000000000004, 0.008920000000000004, 7e-05, 0.00153, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City130/insert", Data: [6]float64{31, 0.009450000000000002, 0.009450000000000002, 7e-05, 0.00165, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City131/insert", Data: [6]float64{31, 0.0151, 0.0151, 7e-05, 0.00755, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City132/insert", Data: [6]float64{31, 0.0071200000000000005, 0.0071200000000000005, 7e-05, 0.00098, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City133/insert", Data: [6]float64{31, 0.02159, 0.02159, 7e-05, 0.01466, 0.00021}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City134/insert", Data: [6]float64{31, 0.00693, 0.00693, 7e-05, 0.00097, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City135/insert", Data: [6]float64{31, 0.01277, 0.01277, 7e-05, 0.00573, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City136/insert", Data: [6]float64{31, 0.008560000000000002, 0.008560000000000002, 7e-05, 0.00112, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City137/insert", Data: [6]float64{31, 0.009430000000000001, 0.009430000000000001, 7e-05, 0.00298, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City138/insert", Data: [6]float64{31, 0.01058, 0.01058, 7e-05, 0.00319, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City139/insert", Data: [6]float64{31, 0.009880000000000002, 0.009880000000000002, 7e-05, 0.00274, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City140/insert", Data: [6]float64{31, 0.015870000000000002, 0.015870000000000002, 7e-05, 0.00774, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City141/insert", Data: [6]float64{31, 0.03871, 0.03871, 7e-05, 0.02521, 0.0006600000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City142/insert", Data: [6]float64{31, 0.00794, 0.00794, 7e-05, 0.00101, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City143/insert", Data: [6]float64{31, 0.01088, 0.01088, 7e-05, 0.00334, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City144/insert", Data: [6]float64{31, 0.007349999999999998, 0.007349999999999998, 7e-05, 0.00106, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City145/insert", Data: [6]float64{31, 0.008159999999999999, 0.008159999999999999, 7e-05, 0.0012, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City146/insert", Data: [6]float64{31, 0.029589999999999998, 0.029589999999999998, 7e-05, 0.01876, 0.00035}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City147/insert", Data: [6]float64{31, 0.01404, 0.01404, 7e-05, 0.00647, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City148/insert", Data: [6]float64{31, 0.02865, 0.02865, 7e-05, 0.0103, 0.00017}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City149/insert", Data: [6]float64{31, 0.03610000000000001, 0.03610000000000001, 7e-05, 0.02732, 0.00075}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City150/insert", Data: [6]float64{31, 0.007470000000000001, 0.007470000000000001, 7e-05, 0.00096, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City151/insert", Data: [6]float64{31, 0.013230000000000006, 0.013230000000000006, 7e-05, 0.00458, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City152/insert", Data: [6]float64{31, 0.011340000000000003, 0.011340000000000003, 7e-05, 0.00296, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City153/insert", Data: [6]float64{31, 0.03273, 0.03273, 7e-05, 0.01266, 0.00023}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City154/insert", Data: [6]float64{31, 0.009830000000000002, 0.009830000000000002, 7e-05, 0.0027, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City155/insert", Data: [6]float64{31, 0.008780000000000001, 0.008780000000000001, 7e-05, 0.00114, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City156/insert", Data: [6]float64{31, 0.03438999999999999, 0.03438999999999999, 7e-05, 0.02226, 0.00051}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City157/insert", Data: [6]float64{31, 0.008300000000000002, 0.008300000000000002, 7e-05, 0.001, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City158/insert", Data: [6]float64{31, 0.008190000000000003, 0.008190000000000003, 7e-05, 0.00112, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City159/insert", Data: [6]float64{31, 0.00831, 0.00831, 7e-05, 0.00112, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City160/insert", Data: [6]float64{31, 0.027860000000000003, 0.027860000000000003, 7e-05, 0.01951, 0.00038}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City161/insert", Data: [6]float64{31, 0.017960000000000004, 0.017960000000000004, 7e-05, 0.01012, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City162/insert", Data: [6]float64{31, 0.007709999999999999, 0.007709999999999999, 7e-05, 0.00114, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City163/insert", Data: [6]float64{31, 0.009510000000000005, 0.009510000000000005, 7e-05, 0.00109, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City164/insert", Data: [6]float64{31, 0.010100000000000005, 0.010100000000000005, 7e-05, 0.00141, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City165/insert", Data: [6]float64{31, 0.00834, 0.00834, 7e-05, 0.00097, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City166/insert", Data: [6]float64{31, 0.014629999999999999, 0.014629999999999999, 7e-05, 0.00365, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City167/insert", Data: [6]float64{31, 0.019500000000000003, 0.019500000000000003, 7e-05, 0.01049, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City168/insert", Data: [6]float64{31, 0.00741, 0.00741, 7e-05, 0.0009, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City169/insert", Data: [6]float64{31, 0.02917, 0.02917, 7e-05, 0.01622, 0.00027}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City170/insert", Data: [6]float64{31, 0.030470000000000004, 0.030470000000000004, 7e-05, 0.01261, 0.00022}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City171/insert", Data: [6]float64{31, 0.007549999999999999, 0.007549999999999999, 7e-05, 0.00113, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City172/insert", Data: [6]float64{31, 0.008220000000000002, 0.008220000000000002, 7e-05, 0.00116, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City173/insert", Data: [6]float64{31, 0.010980000000000002, 0.010980000000000002, 7e-05, 0.00255, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City174/insert", Data: [6]float64{31, 0.01675, 0.01675, 7e-05, 0.00699, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City175/insert", Data: [6]float64{31, 0.014010000000000002, 0.014010000000000002, 7e-05, 0.00312, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City176/insert", Data: [6]float64{31, 0.011140000000000006, 0.011140000000000006, 7e-05, 0.00305, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City177/insert", Data: [6]float64{31, 0.016810000000000002, 0.016810000000000002, 7e-05, 0.00768, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City178/insert", Data: [6]float64{31, 0.007389999999999999, 0.007389999999999999, 7e-05, 0.00115, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City179/insert", Data: [6]float64{31, 0.014900000000000004, 0.014900000000000004, 7e-05, 0.00474, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City180/insert", Data: [6]float64{31, 0.013340000000000001, 0.013340000000000001, 7e-05, 0.00477, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City181/insert", Data: [6]float64{31, 0.010290000000000004, 0.010290000000000004, 7e-05, 0.00175, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City182/insert", Data: [6]float64{31, 0.008009999999999998, 0.008009999999999998, 7e-05, 0.00118, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City183/insert", Data: [6]float64{31, 0.00873, 0.00873, 7e-05, 0.00126, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City184/insert", Data: [6]float64{31, 0.009210000000000003, 0.009210000000000003, 7e-05, 0.00172, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City185/insert", Data: [6]float64{31, 0.00883, 0.00883, 7e-05, 0.0012, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City186/insert", Data: [6]float64{31, 0.01839, 0.01839, 7e-05, 0.00929, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City187/insert", Data: [6]float64{31, 0.009420000000000001, 0.009420000000000001, 7e-05, 0.00134, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City188/insert", Data: [6]float64{31, 0.049100000000000005, 0.049100000000000005, 7e-05, 0.02553, 0.00081}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City189/insert", Data: [6]float64{31, 0.018189999999999998, 0.018189999999999998, 7e-05, 0.00516, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City190/insert", Data: [6]float64{31, 0.010490000000000001, 0.010490000000000001, 7e-05, 0.00239, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City191/insert", Data: [6]float64{31, 0.011730000000000003, 0.011730000000000003, 7e-05, 0.00222, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City192/insert", Data: [6]float64{31, 0.008400000000000001, 0.008400000000000001, 7e-05, 0.00136, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City193/insert", Data: [6]float64{31, 0.022780000000000005, 0.022780000000000005, 7e-05, 0.01209, 0.00015999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City194/insert", Data: [6]float64{31, 0.020200000000000003, 0.020200000000000003, 7e-05, 0.01261, 0.00016}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City195/insert", Data: [6]float64{31, 0.011749999999999998, 0.011749999999999998, 7e-05, 0.00218, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City196/insert", Data: [6]float64{31, 0.008079999999999999, 0.008079999999999999, 7e-05, 0.00139, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City197/insert", Data: [6]float64{31, 0.016300000000000005, 0.016300000000000005, 7e-05, 0.00811, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City198/insert", Data: [6]float64{31, 0.010110000000000003, 0.010110000000000003, 7e-05, 0.00192, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City199/insert", Data: [6]float64{31, 0.008100000000000001, 0.008100000000000001, 7e-05, 0.00112, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City200/insert", Data: [6]float64{31, 0.007749999999999998, 0.007749999999999998, 7e-05, 0.0012, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City201/insert", Data: [6]float64{31, 0.01495, 0.01495, 7e-05, 0.00524, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City202/insert", Data: [6]float64{31, 0.009440000000000002, 0.009440000000000002, 7e-05, 0.00188, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City203/insert", Data: [6]float64{31, 0.007829999999999998, 0.007829999999999998, 7e-05, 0.00105, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City204/insert", Data: [6]float64{31, 0.02313, 0.02313, 7e-05, 0.00833, 0.00010999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City205/insert", Data: [6]float64{31, 0.03678999999999999, 0.03678999999999999, 8e-05, 0.02367, 0.0006}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City206/insert", Data: [6]float64{31, 0.011290000000000001, 0.011290000000000001, 7e-05, 0.00143, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City207/insert", Data: [6]float64{31, 0.027540000000000005, 0.027540000000000005, 7e-05, 0.01078, 0.00016}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City208/insert", Data: [6]float64{31, 0.010100000000000001, 0.010100000000000001, 7e-05, 0.00366, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City209/insert", Data: [6]float64{31, 0.0129, 0.0129, 7e-05, 0.00619, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City210/insert", Data: [6]float64{31, 0.009030000000000002, 0.009030000000000002, 7e-05, 0.00208, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City211/insert", Data: [6]float64{31, 0.02022, 0.02022, 7e-05, 0.00759, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City212/insert", Data: [6]float64{31, 0.03062, 0.03062, 7e-05, 0.01474, 0.00028000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City213/insert", Data: [6]float64{31, 0.016700000000000003, 0.016700000000000003, 7e-05, 0.00611, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City214/insert", Data: [6]float64{31, 0.029880000000000004, 0.029880000000000004, 7e-05, 0.0146, 0.00025}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City215/insert", Data: [6]float64{31, 0.0070399999999999985, 0.0070399999999999985, 7e-05, 0.00079, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City216/insert", Data: [6]float64{31, 0.00721, 0.00721, 7e-05, 0.00091, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City217/insert", Data: [6]float64{31, 0.06302, 0.06302, 7e-05, 0.02977, 0.0015899999999999998}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City218/insert", Data: [6]float64{31, 0.00902, 0.00902, 7e-05, 0.00202, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City219/insert", Data: [6]float64{31, 0.01595, 0.01595, 7e-05, 0.00607, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City220/insert", Data: [6]float64{31, 0.01633, 0.01633, 7e-05, 0.006, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City221/insert", Data: [6]float64{31, 0.01559, 0.01559, 7e-05, 0.00507, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City222/insert", Data: [6]float64{31, 0.025089999999999998, 0.025089999999999998, 7e-05, 0.01654, 0.00028000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City223/insert", Data: [6]float64{31, 0.01391, 0.01391, 7e-05, 0.00496, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City224/insert", Data: [6]float64{31, 0.00767, 0.00767, 7e-05, 0.00125, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City225/insert", Data: [6]float64{31, 0.026690000000000002, 0.026690000000000002, 7e-05, 0.01222, 0.00016999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City226/insert", Data: [6]float64{31, 0.013000000000000001, 0.013000000000000001, 7e-05, 0.00598, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City227/insert", Data: [6]float64{31, 0.014800000000000004, 0.014800000000000004, 7e-05, 0.00748, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City228/insert", Data: [6]float64{31, 0.012070000000000003, 0.012070000000000003, 7e-05, 0.00348, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City229/insert", Data: [6]float64{31, 0.01446, 0.01446, 7e-05, 0.0077, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City230/insert", Data: [6]float64{31, 0.007260000000000001, 0.007260000000000001, 7e-05, 0.00126, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City231/insert", Data: [6]float64{31, 0.010270000000000003, 0.010270000000000003, 7e-05, 0.00291, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City232/insert", Data: [6]float64{31, 0.03822999999999999, 0.03822999999999999, 7e-05, 0.0131, 0.00027}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City233/insert", Data: [6]float64{31, 0.05641000000000001, 0.05641000000000001, 7e-05, 0.02542, 0.00091}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City234/insert", Data: [6]float64{31, 0.016120000000000006, 0.016120000000000006, 7e-05, 0.00568, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City235/insert", Data: [6]float64{31, 0.018850000000000002, 0.018850000000000002, 7e-05, 0.01195, 0.00014}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City236/insert", Data: [6]float64{31, 0.011400000000000004, 0.011400000000000004, 7e-05, 0.00419, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City237/insert", Data: [6]float64{31, 0.015470000000000005, 0.015470000000000005, 7e-05, 0.00535, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City238/insert", Data: [6]float64{31, 0.011910000000000002, 0.011910000000000002, 7e-05, 0.00462, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City239/insert", Data: [6]float64{31, 0.008280000000000001, 0.008280000000000001, 7e-05, 0.00203, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City240/insert", Data: [6]float64{31, 0.013030000000000002, 0.013030000000000002, 7e-05, 0.00642, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City241/insert", Data: [6]float64{31, 0.01922, 0.01922, 7e-05, 0.00862, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City242/insert", Data: [6]float64{31, 0.019830000000000004, 0.019830000000000004, 7e-05, 0.00871, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City243/insert", Data: [6]float64{31, 0.010330000000000004, 0.010330000000000004, 7e-05, 0.00331, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City244/insert", Data: [6]float64{31, 0.01285, 0.01285, 7e-05, 0.0066, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City245/insert", Data: [6]float64{31, 0.018540000000000004, 0.018540000000000004, 7e-05, 0.01001, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City246/insert", Data: [6]float64{31, 0.012520000000000003, 0.012520000000000003, 7e-05, 0.00326, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City247/insert", Data: [6]float64{31, 0.01553, 0.01553, 7e-05, 0.00779, 7.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City248/insert", Data: [6]float64{31, 0.012100000000000001, 0.012100000000000001, 7e-05, 0.00305, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City249/insert", Data: [6]float64{31, 0.014920000000000001, 0.014920000000000001, 7e-05, 0.00394, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City250/insert", Data: [6]float64{31, 0.020370000000000003, 0.020370000000000003, 7e-05, 0.01362, 0.00019}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City251/insert", Data: [6]float64{31, 0.01205, 0.01205, 7e-05, 0.00634, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City252/insert", Data: [6]float64{31, 0.014250000000000004, 0.014250000000000004, 7e-05, 0.00648, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City253/insert", Data: [6]float64{31, 0.0066999999999999985, 0.0066999999999999985, 7e-05, 0.0011, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City254/insert", Data: [6]float64{31, 0.008389999999999996, 0.008389999999999996, 7e-05, 0.00183, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City255/insert", Data: [6]float64{31, 0.045970000000000004, 0.045970000000000004, 7e-05, 0.03152, 0.00102}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City256/insert", Data: [6]float64{31, 0.00842, 0.00842, 7e-05, 0.00106, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City257/insert", Data: [6]float64{31, 0.016260000000000004, 0.016260000000000004, 7e-05, 0.00722, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City258/insert", Data: [6]float64{31, 0.01666, 0.01666, 7e-05, 0.0065, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City259/insert", Data: [6]float64{31, 0.01003, 0.01003, 7e-05, 0.00135, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City260/insert", Data: [6]float64{31, 0.015020000000000002, 0.015020000000000002, 7e-05, 0.00432, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City261/insert", Data: [6]float64{31, 0.021290000000000003, 0.021290000000000003, 7e-05, 0.00981, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City262/insert", Data: [6]float64{31, 0.03193999999999999, 0.03193999999999999, 7e-05, 0.01951, 0.00039000000000000005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City263/insert", Data: [6]float64{31, 0.009289999999999998, 0.009289999999999998, 7e-05, 0.00114, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City264/insert", Data: [6]float64{31, 0.00847, 0.00847, 7e-05, 0.00114, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City265/insert", Data: [6]float64{31, 0.013260000000000001, 0.013260000000000001, 7e-05, 0.00496, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City266/insert", Data: [6]float64{31, 0.014320000000000005, 0.014320000000000005, 7e-05, 0.0059, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City267/insert", Data: [6]float64{31, 0.017740000000000002, 0.017740000000000002, 7e-05, 0.00995, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City268/insert", Data: [6]float64{31, 0.009570000000000002, 0.009570000000000002, 7e-05, 0.00129, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City269/insert", Data: [6]float64{31, 0.023300000000000005, 0.023300000000000005, 7e-05, 0.01071, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City270/insert", Data: [6]float64{31, 0.009450000000000002, 0.009450000000000002, 7e-05, 0.00115, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City271/insert", Data: [6]float64{31, 0.011030000000000002, 0.011030000000000002, 7e-05, 0.00165, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City272/insert", Data: [6]float64{31, 0.019690000000000003, 0.019690000000000003, 7e-05, 0.00848, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City273/insert", Data: [6]float64{31, 0.00957, 0.00957, 7e-05, 0.00159, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City274/insert", Data: [6]float64{31, 0.03062, 0.03062, 7e-05, 0.01655, 0.00029}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City275/insert", Data: [6]float64{31, 0.019370000000000002, 0.019370000000000002, 7e-05, 0.00568, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City276/insert", Data: [6]float64{31, 0.009880000000000003, 0.009880000000000003, 7e-05, 0.00122, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City277/insert", Data: [6]float64{31, 0.029029999999999997, 0.029029999999999997, 7e-05, 0.01697, 0.00029}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City278/insert", Data: [6]float64{31, 0.05147000000000001, 0.05147000000000001, 7e-05, 0.03355, 0.00116}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City279/insert", Data: [6]float64{31, 0.01823, 0.01823, 7e-05, 0.01047, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City280/insert", Data: [6]float64{31, 0.02148, 0.02148, 7e-05, 0.00463, 5.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City281/insert", Data: [6]float64{31, 0.01794, 0.01794, 7e-05, 0.00324, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City282/insert", Data: [6]float64{31, 0.035149999999999994, 0.035149999999999994, 7e-05, 0.02539, 0.00064}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City283/insert", Data: [6]float64{31, 0.009790000000000005, 0.009790000000000005, 7e-05, 0.00229, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City284/insert", Data: [6]float64{31, 0.013920000000000002, 0.013920000000000002, 7e-05, 0.00261, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City285/insert", Data: [6]float64{31, 0.00866, 0.00866, 7e-05, 0.00125, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City286/insert", Data: [6]float64{31, 0.028640000000000002, 0.028640000000000002, 7e-05, 0.01268, 0.00019}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City287/insert", Data: [6]float64{31, 0.01781, 0.01781, 7e-05, 0.00626, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City288/insert", Data: [6]float64{31, 0.013090000000000001, 0.013090000000000001, 7e-05, 0.00263, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City289/insert", Data: [6]float64{31, 0.010740000000000003, 0.010740000000000003, 7e-05, 0.00201, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City290/insert", Data: [6]float64{31, 0.02665, 0.02665, 7e-05, 0.01012, 0.00013000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City291/insert", Data: [6]float64{31, 0.01648, 0.01648, 7e-05, 0.00415, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City292/insert", Data: [6]float64{31, 0.008150000000000003, 0.008150000000000003, 7e-05, 0.00098, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City293/insert", Data: [6]float64{31, 0.010520000000000002, 0.010520000000000002, 7e-05, 0.00168, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City294/insert", Data: [6]float64{31, 0.010620000000000001, 0.010620000000000001, 7e-05, 0.00264, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City295/insert", Data: [6]float64{31, 0.01679, 0.01679, 7e-05, 0.00751, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City296/insert", Data: [6]float64{31, 0.05086999999999999, 0.05086999999999999, 7e-05, 0.03414, 0.00122}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City297/insert", Data: [6]float64{31, 0.22410999999999995, 0.22410999999999995, 7e-05, 0.21356, 0.04561}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City298/insert", Data: [6]float64{31, 0.01573, 0.01573, 7e-05, 0.00289, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City299/insert", Data: [6]float64{31, 0.016959999999999996, 0.016959999999999996, 7e-05, 0.00596, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City300/insert", Data: [6]float64{31, 0.019330000000000003, 0.019330000000000003, 7e-05, 0.00995, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City301/insert", Data: [6]float64{31, 0.009999999999999998, 0.009999999999999998, 7e-05, 0.00172, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City302/insert", Data: [6]float64{31, 0.014710000000000003, 0.014710000000000003, 7e-05, 0.00746, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City303/insert", Data: [6]float64{31, 0.020050000000000002, 0.020050000000000002, 7e-05, 0.01086, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City304/insert", Data: [6]float64{31, 0.01862, 0.01862, 7e-05, 0.00463, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City305/insert", Data: [6]float64{31, 0.011789999999999998, 0.011789999999999998, 7e-05, 0.00212, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City306/insert", Data: [6]float64{31, 0.016360000000000003, 0.016360000000000003, 7e-05, 0.00411, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City307/insert", Data: [6]float64{31, 0.020040000000000006, 0.020040000000000006, 7e-05, 0.0073, 7.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City308/insert", Data: [6]float64{31, 0.023690000000000003, 0.023690000000000003, 7e-05, 0.00783, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City309/insert", Data: [6]float64{31, 0.010940000000000004, 0.010940000000000004, 7e-05, 0.00177, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City310/insert", Data: [6]float64{31, 0.021460000000000003, 0.021460000000000003, 7e-05, 0.00415, 6.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City311/insert", Data: [6]float64{31, 0.010560000000000002, 0.010560000000000002, 7e-05, 0.00117, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City312/insert", Data: [6]float64{31, 0.028279999999999996, 0.028279999999999996, 7e-05, 0.01951, 0.00038}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City313/insert", Data: [6]float64{31, 0.0159, 0.0159, 7e-05, 0.00344, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City314/insert", Data: [6]float64{31, 0.021679999999999998, 0.021679999999999998, 7e-05, 0.00493, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City315/insert", Data: [6]float64{31, 0.017360000000000004, 0.017360000000000004, 7e-05, 0.0029, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City316/insert", Data: [6]float64{31, 0.014260000000000002, 0.014260000000000002, 7e-05, 0.00493, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City317/insert", Data: [6]float64{31, 0.060980000000000006, 0.060980000000000006, 7e-05, 0.02032, 0.00083}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City318/insert", Data: [6]float64{31, 0.02583, 0.02583, 7e-05, 0.01124, 0.00015999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City319/insert", Data: [6]float64{31, 0.02702, 0.02702, 7e-05, 0.00941, 0.00013000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City320/insert", Data: [6]float64{31, 0.01776, 0.01776, 7e-05, 0.00433, 5.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City321/insert", Data: [6]float64{31, 0.018560000000000004, 0.018560000000000004, 7e-05, 0.00287, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City322/insert", Data: [6]float64{31, 0.015410000000000005, 0.015410000000000005, 7e-05, 0.00407, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City323/insert", Data: [6]float64{31, 0.014910000000000001, 0.014910000000000001, 7e-05, 0.00558, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City324/insert", Data: [6]float64{31, 0.016550000000000002, 0.016550000000000002, 7e-05, 0.00699, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City325/insert", Data: [6]float64{31, 0.00944, 0.00944, 7e-05, 0.00234, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City326/insert", Data: [6]float64{31, 0.028180000000000004, 0.028180000000000004, 7e-05, 0.0152, 0.00026000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City327/insert", Data: [6]float64{31, 0.010390000000000005, 0.010390000000000005, 7e-05, 0.00415, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City328/insert", Data: [6]float64{31, 0.02093, 0.02093, 7e-05, 0.01237, 0.00015999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City329/insert", Data: [6]float64{31, 0.008139999999999998, 0.008139999999999998, 7e-05, 0.00159, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City330/insert", Data: [6]float64{31, 0.015300000000000005, 0.015300000000000005, 7e-05, 0.00437, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City331/insert", Data: [6]float64{31, 0.02082, 0.02082, 7e-05, 0.00666, 7.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City332/insert", Data: [6]float64{31, 0.01007, 0.01007, 7e-05, 0.00273, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City333/insert", Data: [6]float64{31, 0.017980000000000006, 0.017980000000000006, 7e-05, 0.00824, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City334/insert", Data: [6]float64{31, 0.015210000000000003, 0.015210000000000003, 7e-05, 0.00559, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City335/insert", Data: [6]float64{31, 0.01835, 0.01835, 7e-05, 0.00459, 5.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City336/insert", Data: [6]float64{31, 0.019670000000000003, 0.019670000000000003, 7e-05, 0.00779, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City337/insert", Data: [6]float64{31, 0.013210000000000003, 0.013210000000000003, 7e-05, 0.00621, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City338/insert", Data: [6]float64{31, 0.0068200000000000005, 0.0068200000000000005, 7e-05, 0.00069, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City339/insert", Data: [6]float64{31, 0.013279999999999998, 0.013279999999999998, 7e-05, 0.00359, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City340/insert", Data: [6]float64{31, 0.00822, 0.00822, 7e-05, 0.00158, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City341/insert", Data: [6]float64{31, 0.00734, 0.00734, 7e-05, 0.0012, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City342/insert", Data: [6]float64{31, 0.012150000000000001, 0.012150000000000001, 7e-05, 0.00332, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City343/insert", Data: [6]float64{31, 0.01155, 0.01155, 7e-05, 0.00295, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City344/insert", Data: [6]float64{31, 0.02581, 0.02581, 7e-05, 0.0067, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City345/insert", Data: [6]float64{31, 0.012080000000000002, 0.012080000000000002, 7e-05, 0.00273, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City346/insert", Data: [6]float64{31, 0.006810000000000001, 0.006810000000000001, 7e-05, 0.00065, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City347/insert", Data: [6]float64{31, 0.012360000000000003, 0.012360000000000003, 7e-05, 0.00419, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City348/insert", Data: [6]float64{31, 0.011840000000000002, 0.011840000000000002, 7e-05, 0.00549, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City349/insert", Data: [6]float64{31, 0.04577, 0.04577, 7e-05, 0.03398, 0.00117}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City350/insert", Data: [6]float64{31, 0.016530000000000003, 0.016530000000000003, 7e-05, 0.00666, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City351/insert", Data: [6]float64{31, 0.015350000000000004, 0.015350000000000004, 7e-05, 0.00479, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City352/insert", Data: [6]float64{31, 0.013859999999999999, 0.013859999999999999, 7e-05, 0.00278, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City353/insert", Data: [6]float64{31, 0.00987, 0.00987, 7e-05, 0.00414, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City354/insert", Data: [6]float64{31, 0.0214, 0.0214, 7e-05, 0.00771, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City355/insert", Data: [6]float64{31, 0.03605, 0.03605, 7e-05, 0.01311, 0.00027000000000000006}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City356/insert", Data: [6]float64{31, 0.01933, 0.01933, 7e-05, 0.00684, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City357/insert", Data: [6]float64{31, 0.04825, 0.04825, 7e-05, 0.02964, 0.0009200000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City358/insert", Data: [6]float64{31, 0.011070000000000002, 0.011070000000000002, 7e-05, 0.00289, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City359/insert", Data: [6]float64{31, 0.0211, 0.0211, 7e-05, 0.01115, 0.00013000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City360/insert", Data: [6]float64{31, 0.01143, 0.01143, 7e-05, 0.00246, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City361/insert", Data: [6]float64{31, 0.0204, 0.0204, 7e-05, 0.01138, 0.00013}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City362/insert", Data: [6]float64{31, 0.010039999999999999, 0.010039999999999999, 7e-05, 0.00198, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City363/insert", Data: [6]float64{31, 0.011260000000000003, 0.011260000000000003, 7e-05, 0.00367, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City364/insert", Data: [6]float64{31, 0.03483000000000001, 0.03483000000000001, 7e-05, 0.01325, 0.00029000000000000006}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City365/insert", Data: [6]float64{31, 0.02536999999999999, 0.02536999999999999, 7e-05, 0.01637, 0.00028000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City366/insert", Data: [6]float64{31, 0.00847, 0.00847, 7e-05, 0.00129, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City367/insert", Data: [6]float64{31, 0.01531, 0.01531, 7e-05, 0.00548, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City368/insert", Data: [6]float64{31, 0.010140000000000001, 0.010140000000000001, 7e-05, 0.00268, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City369/insert", Data: [6]float64{31, 0.013990000000000002, 0.013990000000000002, 7e-05, 0.00339, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City370/insert", Data: [6]float64{31, 0.012680000000000002, 0.012680000000000002, 7e-05, 0.00406, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City371/insert", Data: [6]float64{31, 0.00921, 0.00921, 7e-05, 0.00167, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City372/insert", Data: [6]float64{31, 0.0086, 0.0086, 7e-05, 0.00196, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City373/insert", Data: [6]float64{31, 0.012490000000000001, 0.012490000000000001, 7e-05, 0.00471, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City374/insert", Data: [6]float64{31, 0.015630000000000005, 0.015630000000000005, 8e-05, 0.00715, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City375/insert", Data: [6]float64{31, 0.01593, 0.01593, 7e-05, 0.00457, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City376/insert", Data: [6]float64{31, 0.018250000000000002, 0.018250000000000002, 7e-05, 0.0069, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City377/insert", Data: [6]float64{31, 0.010930000000000002, 0.010930000000000002, 7e-05, 0.00181, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City378/insert", Data: [6]float64{31, 0.01109, 0.01109, 7e-05, 0.00199, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City379/insert", Data: [6]float64{31, 0.021790000000000004, 0.021790000000000004, 7e-05, 0.00736, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City380/insert", Data: [6]float64{31, 0.008650000000000001, 0.008650000000000001, 7e-05, 0.00129, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City381/insert", Data: [6]float64{31, 0.00724, 0.00724, 7e-05, 0.00064, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City382/insert", Data: [6]float64{31, 0.013890000000000001, 0.013890000000000001, 7e-05, 0.00386, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City383/insert", Data: [6]float64{31, 0.03162, 0.03162, 7e-05, 0.01801, 0.00033000000000000005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City384/insert", Data: [6]float64{31, 0.03063, 0.03063, 7e-05, 0.01241, 0.00017999999999999998}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City385/insert", Data: [6]float64{31, 0.009620000000000002, 0.009620000000000002, 7e-05, 0.00138, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City386/insert", Data: [6]float64{31, 0.016710000000000006, 0.016710000000000006, 7e-05, 0.00482, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City387/insert", Data: [6]float64{31, 0.01533, 0.01533, 7e-05, 0.00675, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City388/insert", Data: [6]float64{31, 0.01233, 0.01233, 7e-05, 0.00277, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City389/insert", Data: [6]float64{31, 0.016369999999999996, 0.016369999999999996, 7e-05, 0.00516, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City390/insert", Data: [6]float64{31, 0.018639999999999997, 0.018639999999999997, 7e-05, 0.00445, 5.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City391/insert", Data: [6]float64{31, 0.010730000000000002, 0.010730000000000002, 7e-05, 0.00256, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City392/insert", Data: [6]float64{31, 0.009220000000000002, 0.009220000000000002, 7e-05, 0.00122, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City393/insert", Data: [6]float64{31, 0.01056, 0.01056, 7e-05, 0.0031, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City394/insert", Data: [6]float64{31, 0.02643, 0.02643, 7e-05, 0.01651, 0.00028000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City395/insert", Data: [6]float64{31, 0.0168, 0.0168, 7e-05, 0.00419, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City396/insert", Data: [6]float64{31, 0.014700000000000001, 0.014700000000000001, 7e-05, 0.0046, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City397/insert", Data: [6]float64{31, 0.01475, 0.01475, 7e-05, 0.00494, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City398/insert", Data: [6]float64{31, 0.01006, 0.01006, 7e-05, 0.00249, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City399/insert", Data: [6]float64{31, 0.015990000000000004, 0.015990000000000004, 7e-05, 0.0082, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City400/insert", Data: [6]float64{31, 0.015960000000000002, 0.015960000000000002, 7e-05, 0.00536, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City401/insert", Data: [6]float64{31, 0.0369, 0.0369, 7e-05, 0.01977, 0.00042}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City402/insert", Data: [6]float64{31, 0.013, 0.013, 7e-05, 0.00305, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City403/insert", Data: [6]float64{31, 0.0093, 0.0093, 7e-05, 0.00196, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City404/insert", Data: [6]float64{31, 0.013669999999999998, 0.013669999999999998, 7e-05, 0.00288, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City405/insert", Data: [6]float64{31, 0.020429999999999997, 0.020429999999999997, 7e-05, 0.0078, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City406/insert", Data: [6]float64{31, 0.019799999999999998, 0.019799999999999998, 7e-05, 0.01223, 0.00015}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City407/insert", Data: [6]float64{31, 0.00736, 0.00736, 7e-05, 0.00062, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City408/insert", Data: [6]float64{31, 0.013759999999999996, 0.013759999999999996, 7e-05, 0.00621, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City409/insert", Data: [6]float64{31, 0.01366, 0.01366, 7e-05, 0.00382, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City410/insert", Data: [6]float64{31, 0.010879999999999999, 0.010879999999999999, 7e-05, 0.00303, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City411/insert", Data: [6]float64{31, 0.011660000000000004, 0.011660000000000004, 7e-05, 0.00373, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City412/insert", Data: [6]float64{31, 0.013069999999999998, 0.013069999999999998, 7e-05, 0.00351, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City413/insert", Data: [6]float64{31, 0.01564, 0.01564, 7e-05, 0.00626, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City414/insert", Data: [6]float64{31, 0.008980000000000002, 0.008980000000000002, 7e-05, 0.00098, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City415/insert", Data: [6]float64{31, 0.016309999999999995, 0.016309999999999995, 8e-05, 0.00541, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City416/insert", Data: [6]float64{31, 0.023499999999999997, 0.023499999999999997, 8e-05, 0.00409, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City417/insert", Data: [6]float64{31, 0.01029, 0.01029, 7e-05, 0.00226, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City418/insert", Data: [6]float64{31, 0.04059, 0.04059, 7e-05, 0.03108, 0.00098}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City419/insert", Data: [6]float64{31, 0.018019999999999998, 0.018019999999999998, 7e-05, 0.00453, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City420/insert", Data: [6]float64{31, 0.011649999999999999, 0.011649999999999999, 7e-05, 0.00262, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City421/insert", Data: [6]float64{31, 0.014710000000000003, 0.014710000000000003, 7e-05, 0.00304, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City422/insert", Data: [6]float64{31, 0.00812, 0.00812, 7e-05, 0.00086, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City423/insert", Data: [6]float64{31, 0.021480000000000006, 0.021480000000000006, 7e-05, 0.00776, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City424/insert", Data: [6]float64{31, 0.017669999999999998, 0.017669999999999998, 7e-05, 0.00455, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City425/insert", Data: [6]float64{31, 0.01412, 0.01412, 8e-05, 0.00254, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City426/insert", Data: [6]float64{31, 0.01243, 0.01243, 7e-05, 0.00424, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City427/insert", Data: [6]float64{31, 0.008710000000000002, 0.008710000000000002, 7e-05, 0.00099, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City428/insert", Data: [6]float64{31, 0.010340000000000004, 0.010340000000000004, 7e-05, 0.0025, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City429/insert", Data: [6]float64{31, 0.007920000000000002, 0.007920000000000002, 7e-05, 0.0011, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City430/insert", Data: [6]float64{31, 0.028720000000000002, 0.028720000000000002, 7e-05, 0.02078, 0.00043}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City431/insert", Data: [6]float64{31, 0.022419999999999995, 0.022419999999999995, 7e-05, 0.01115, 0.00013000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City432/insert", Data: [6]float64{31, 0.012230000000000001, 0.012230000000000001, 7e-05, 0.00267, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City433/insert", Data: [6]float64{31, 0.014830000000000001, 0.014830000000000001, 7e-05, 0.00464, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City434/insert", Data: [6]float64{31, 0.01808, 0.01808, 7e-05, 0.00463, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City435/insert", Data: [6]float64{31, 0.01409, 0.01409, 7e-05, 0.00323, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City436/insert", Data: [6]float64{31, 0.007980000000000001, 0.007980000000000001, 7e-05, 0.00085, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City437/insert", Data: [6]float64{31, 0.01048, 0.01048, 7e-05, 0.00315, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City438/insert", Data: [6]float64{31, 0.009990000000000002, 0.009990000000000002, 7e-05, 0.00121, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City439/insert", Data: [6]float64{31, 0.03147, 0.03147, 7e-05, 0.01619, 0.00028}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City440/insert", Data: [6]float64{31, 0.009790000000000002, 0.009790000000000002, 7e-05, 0.00101, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City441/insert", Data: [6]float64{31, 0.04551, 0.04551, 7e-05, 0.02849, 0.00084}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City442/insert", Data: [6]float64{31, 0.00953, 0.00953, 7e-05, 0.00092, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City443/insert", Data: [6]float64{31, 0.05558999999999999, 0.05558999999999999, 7e-05, 0.02647, 0.0009400000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City444/insert", Data: [6]float64{31, 0.01646, 0.01646, 7e-05, 0.00427, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City445/insert", Data: [6]float64{31, 0.01309, 0.01309, 8e-05, 0.00391, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City446/insert", Data: [6]float64{31, 0.024710000000000003, 0.024710000000000003, 7e-05, 0.01559, 0.00024}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City447/insert", Data: [6]float64{31, 0.01781, 0.01781, 7e-05, 0.00646, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City448/insert", Data: [6]float64{31, 0.014010000000000002, 0.014010000000000002, 7e-05, 0.00417, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City449/insert", Data: [6]float64{31, 0.00837, 0.00837, 7e-05, 0.00083, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City450/insert", Data: [6]float64{31, 0.008579999999999999, 0.008579999999999999, 7e-05, 0.001, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City451/insert", Data: [6]float64{31, 0.017740000000000002, 0.017740000000000002, 7e-05, 0.00456, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City452/insert", Data: [6]float64{31, 0.01732, 0.01732, 7e-05, 0.00412, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City453/insert", Data: [6]float64{31, 0.042230000000000004, 0.042230000000000004, 7e-05, 0.01384, 0.00035000000000000005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City454/insert", Data: [6]float64{31, 0.01712, 0.01712, 8e-05, 0.00779, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City455/insert", Data: [6]float64{31, 0.01181, 0.01181, 7e-05, 0.00311, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City456/insert", Data: [6]float64{31, 0.0109, 0.0109, 7e-05, 0.00195, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City457/insert", Data: [6]float64{31, 0.018860000000000002, 0.018860000000000002, 7e-05, 0.00395, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City458/insert", Data: [6]float64{31, 0.04132, 0.04132, 7e-05, 0.02895, 0.0008600000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City459/insert", Data: [6]float64{31, 0.01995, 0.01995, 7e-05, 0.01019, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City460/insert", Data: [6]float64{31, 0.0093, 0.0093, 7e-05, 0.00113, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City461/insert", Data: [6]float64{31, 0.034379999999999994, 0.034379999999999994, 7e-05, 0.01785, 0.00037000000000000005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City462/insert", Data: [6]float64{31, 0.03261, 0.03261, 7e-05, 0.01718, 0.00033}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City463/insert", Data: [6]float64{31, 0.013779999999999997, 0.013779999999999997, 7e-05, 0.00216, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City464/insert", Data: [6]float64{31, 0.01058, 0.01058, 7e-05, 0.00145, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City465/insert", Data: [6]float64{31, 0.012440000000000001, 0.012440000000000001, 7e-05, 0.00264, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City466/insert", Data: [6]float64{31, 0.02471, 0.02471, 7e-05, 0.0063, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City467/insert", Data: [6]float64{31, 0.011179999999999999, 0.011179999999999999, 7e-05, 0.00317, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City468/insert", Data: [6]float64{31, 0.013339999999999998, 0.013339999999999998, 7e-05, 0.00599, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City469/insert", Data: [6]float64{31, 0.01278, 0.01278, 7e-05, 0.00264, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City470/insert", Data: [6]float64{31, 0.014950000000000001, 0.014950000000000001, 7e-05, 0.00496, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City471/insert", Data: [6]float64{31, 0.01127, 0.01127, 7e-05, 0.00188, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City472/insert", Data: [6]float64{31, 0.008930000000000002, 0.008930000000000002, 7e-05, 0.00101, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City473/insert", Data: [6]float64{31, 0.017240000000000002, 0.017240000000000002, 7e-05, 0.00509, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City474/insert", Data: [6]float64{31, 0.01138, 0.01138, 7e-05, 0.00252, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City475/insert", Data: [6]float64{31, 0.030390000000000007, 0.030390000000000007, 7e-05, 0.01702, 0.00031}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City476/insert", Data: [6]float64{31, 0.03286000000000001, 0.03286000000000001, 7e-05, 0.00895, 0.00017}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City477/insert", Data: [6]float64{31, 0.018070000000000003, 0.018070000000000003, 8e-05, 0.00751, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City478/insert", Data: [6]float64{31, 0.015060000000000002, 0.015060000000000002, 7e-05, 0.00379, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City479/insert", Data: [6]float64{31, 0.022779999999999998, 0.022779999999999998, 7e-05, 0.00637, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City480/insert", Data: [6]float64{31, 0.008210000000000002, 0.008210000000000002, 7e-05, 0.00074, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City481/insert", Data: [6]float64{31, 0.016940000000000004, 0.016940000000000004, 8e-05, 0.00444, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City482/insert", Data: [6]float64{31, 0.013830000000000004, 0.013830000000000004, 7e-05, 0.00444, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City483/insert", Data: [6]float64{31, 0.01925, 0.01925, 7e-05, 0.01129, 0.00013}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City484/insert", Data: [6]float64{31, 0.01185, 0.01185, 7e-05, 0.00251, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City485/insert", Data: [6]float64{31, 0.009779999999999999, 0.009779999999999999, 7e-05, 0.00182, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City486/insert", Data: [6]float64{31, 0.01413, 0.01413, 7e-05, 0.00401, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City487/insert", Data: [6]float64{31, 0.01567, 0.01567, 7e-05, 0.00497, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City488/insert", Data: [6]float64{31, 0.013510000000000001, 0.013510000000000001, 7e-05, 0.00331, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City489/insert", Data: [6]float64{31, 0.01713, 0.01713, 7e-05, 0.00606, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City490/insert", Data: [6]float64{31, 0.024969999999999996, 0.024969999999999996, 8e-05, 0.00868, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City491/insert", Data: [6]float64{31, 0.0076, 0.0076, 7e-05, 0.00072, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City492/insert", Data: [6]float64{31, 0.01055, 0.01055, 7e-05, 0.0023, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City493/insert", Data: [6]float64{31, 0.027569999999999997, 0.027569999999999997, 7e-05, 0.00855, 0.00013000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City494/insert", Data: [6]float64{31, 0.00917, 0.00917, 7e-05, 0.00085, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City495/insert", Data: [6]float64{31, 0.013199999999999998, 0.013199999999999998, 7e-05, 0.00617, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City496/insert", Data: [6]float64{31, 0.02282, 0.02282, 7e-05, 0.00433, 6.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City497/insert", Data: [6]float64{31, 0.014300000000000002, 0.014300000000000002, 7e-05, 0.00399, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City498/insert", Data: [6]float64{31, 0.01649, 0.01649, 7e-05, 0.00545, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City499/insert", Data: [6]float64{31, 0.023969999999999995, 0.023969999999999995, 7e-05, 0.0058, 7.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City500/insert", Data: [6]float64{31, 0.03219, 0.03219, 7e-05, 0.01622, 0.00031}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City501/insert", Data: [6]float64{31, 0.032810000000000006, 0.032810000000000006, 7e-05, 0.0211, 0.00047000000000000004}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City502/insert", Data: [6]float64{31, 0.01115, 0.01115, 7e-05, 0.00294, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City503/insert", Data: [6]float64{31, 0.01721, 0.01721, 7e-05, 0.00465, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City504/insert", Data: [6]float64{31, 0.019749999999999997, 0.019749999999999997, 7e-05, 0.00573, 6.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City505/insert", Data: [6]float64{31, 0.03474, 0.03474, 7e-05, 0.01356, 0.00025}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City506/insert", Data: [6]float64{31, 0.01107, 0.01107, 7e-05, 0.0029, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City507/insert", Data: [6]float64{31, 0.02026, 0.02026, 7e-05, 0.00795, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City508/insert", Data: [6]float64{31, 0.01076, 0.01076, 7e-05, 0.0046, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City509/insert", Data: [6]float64{31, 0.00624, 0.00624, 7e-05, 0.0009, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City510/insert", Data: [6]float64{31, 0.01908, 0.01908, 7e-05, 0.00443, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City511/insert", Data: [6]float64{31, 0.020870000000000003, 0.020870000000000003, 7e-05, 0.01162, 0.00014}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City512/insert", Data: [6]float64{31, 0.03263999999999999, 0.03263999999999999, 7e-05, 0.01695, 0.00033}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City513/insert", Data: [6]float64{31, 0.01281, 0.01281, 7e-05, 0.0031, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City514/insert", Data: [6]float64{31, 0.008749999999999999, 0.008749999999999999, 7e-05, 0.00259, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City515/insert", Data: [6]float64{31, 0.009360000000000002, 0.009360000000000002, 7e-05, 0.00314, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City516/insert", Data: [6]float64{31, 0.0223, 0.0223, 7e-05, 0.0044, 7.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City517/insert", Data: [6]float64{31, 0.01202, 0.01202, 7e-05, 0.00309, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City518/insert", Data: [6]float64{31, 0.01148, 0.01148, 7e-05, 0.00415, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City519/insert", Data: [6]float64{31, 0.01754, 0.01754, 7e-05, 0.00831, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City520/insert", Data: [6]float64{31, 0.00664, 0.00664, 7e-05, 0.00065, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City521/insert", Data: [6]float64{31, 0.02639, 0.02639, 7e-05, 0.01613, 0.00027}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City522/insert", Data: [6]float64{31, 0.014300000000000004, 0.014300000000000004, 7e-05, 0.00517, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City523/insert", Data: [6]float64{31, 0.00701, 0.00701, 7e-05, 0.00069, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City524/insert", Data: [6]float64{31, 0.030180000000000005, 0.030180000000000005, 7e-05, 0.01513, 0.00025}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City525/insert", Data: [6]float64{31, 0.011140000000000002, 0.011140000000000002, 7e-05, 0.00218, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City526/insert", Data: [6]float64{31, 0.01689, 0.01689, 7e-05, 0.00499, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City527/insert", Data: [6]float64{31, 0.01024, 0.01024, 7e-05, 0.00171, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City528/insert", Data: [6]float64{31, 0.01434, 0.01434, 7e-05, 0.00674, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City529/insert", Data: [6]float64{31, 0.011640000000000001, 0.011640000000000001, 7e-05, 0.00428, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City530/insert", Data: [6]float64{31, 0.040619999999999996, 0.040619999999999996, 7e-05, 0.02426, 0.00064}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City531/insert", Data: [6]float64{31, 0.02276, 0.02276, 7e-05, 0.00797, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City532/insert", Data: [6]float64{31, 0.015199999999999997, 0.015199999999999997, 7e-05, 0.00333, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City533/insert", Data: [6]float64{31, 0.01884, 0.01884, 7e-05, 0.00574, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City534/insert", Data: [6]float64{31, 0.02099, 0.02099, 7e-05, 0.00911, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City535/insert", Data: [6]float64{31, 0.00808, 0.00808, 7e-05, 0.00073, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City536/insert", Data: [6]float64{31, 0.008339999999999998, 0.008339999999999998, 7e-05, 0.00132, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City537/insert", Data: [6]float64{31, 0.01259, 0.01259, 7e-05, 0.00448, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City538/insert", Data: [6]float64{31, 0.01535, 0.01535, 7e-05, 0.00613, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City539/insert", Data: [6]float64{31, 0.01089, 0.01089, 7e-05, 0.00441, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City540/insert", Data: [6]float64{31, 0.026889999999999997, 0.026889999999999997, 7e-05, 0.01163, 0.00016999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City541/insert", Data: [6]float64{31, 0.02137, 0.02137, 7e-05, 0.00847, 9.999999999999999e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City542/insert", Data: [6]float64{31, 0.020240000000000005, 0.020240000000000005, 7e-05, 0.00578, 6.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City543/insert", Data: [6]float64{31, 0.023440000000000003, 0.023440000000000003, 7e-05, 0.00906, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City544/insert", Data: [6]float64{31, 0.010600000000000002, 0.010600000000000002, 7e-05, 0.00338, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City545/insert", Data: [6]float64{31, 0.01508, 0.01508, 7e-05, 0.0043, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City546/insert", Data: [6]float64{31, 0.011119999999999998, 0.011119999999999998, 7e-05, 0.00511, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City547/insert", Data: [6]float64{31, 0.0072400000000000025, 0.0072400000000000025, 7e-05, 0.00062, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City548/insert", Data: [6]float64{31, 0.01659, 0.01659, 7e-05, 0.00553, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City549/insert", Data: [6]float64{31, 0.0091, 0.0091, 7e-05, 0.00129, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City550/insert", Data: [6]float64{31, 0.009260000000000001, 0.009260000000000001, 7e-05, 0.00202, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City551/insert", Data: [6]float64{31, 0.01016, 0.01016, 7e-05, 0.00321, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City552/insert", Data: [6]float64{31, 0.0145, 0.0145, 7e-05, 0.00379, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City553/insert", Data: [6]float64{31, 0.015390000000000003, 0.015390000000000003, 7e-05, 0.00527, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City554/insert", Data: [6]float64{31, 0.02644000000000001, 0.02644000000000001, 7e-05, 0.01554, 0.00026000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City555/insert", Data: [6]float64{31, 0.009349999999999999, 0.009349999999999999, 7e-05, 0.00244, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City556/insert", Data: [6]float64{31, 0.029020000000000004, 0.029020000000000004, 7e-05, 0.02225, 0.00049}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City557/insert", Data: [6]float64{31, 0.01662, 0.01662, 7e-05, 0.00442, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City558/insert", Data: [6]float64{31, 0.014600000000000004, 0.014600000000000004, 7e-05, 0.00712, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City559/insert", Data: [6]float64{31, 0.010169999999999998, 0.010169999999999998, 7e-05, 0.00207, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City560/insert", Data: [6]float64{31, 0.010689999999999998, 0.010689999999999998, 7e-05, 0.00346, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City561/insert", Data: [6]float64{31, 0.008830000000000001, 0.008830000000000001, 7e-05, 0.00165, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City562/insert", Data: [6]float64{31, 0.01702, 0.01702, 7e-05, 0.00434, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City563/insert", Data: [6]float64{31, 0.00813, 0.00813, 7e-05, 0.00108, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City564/insert", Data: [6]float64{31, 0.015420000000000001, 0.015420000000000001, 7e-05, 0.00447, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City565/insert", Data: [6]float64{31, 0.013660000000000002, 0.013660000000000002, 7e-05, 0.0054, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City566/insert", Data: [6]float64{31, 0.009920000000000002, 0.009920000000000002, 7e-05, 0.00266, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City567/insert", Data: [6]float64{31, 0.015330000000000002, 0.015330000000000002, 7e-05, 0.0049, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City568/insert", Data: [6]float64{31, 0.011720000000000001, 0.011720000000000001, 7e-05, 0.00496, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City569/insert", Data: [6]float64{31, 0.05303, 0.05303, 7e-05, 0.02203, 0.00071}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City570/insert", Data: [6]float64{31, 0.01818, 0.01818, 7e-05, 0.00526, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City571/insert", Data: [6]float64{31, 0.01806, 0.01806, 7e-05, 0.00575, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City572/insert", Data: [6]float64{31, 0.014940000000000002, 0.014940000000000002, 7e-05, 0.0036, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City573/insert", Data: [6]float64{31, 0.012439999999999998, 0.012439999999999998, 7e-05, 0.00358, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City574/insert", Data: [6]float64{31, 0.011070000000000002, 0.011070000000000002, 7e-05, 0.00254, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City575/insert", Data: [6]float64{31, 0.01898, 0.01898, 7e-05, 0.00773, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City576/insert", Data: [6]float64{31, 0.02513, 0.02513, 7e-05, 0.00899, 0.00016}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City577/insert", Data: [6]float64{31, 0.01086, 0.01086, 7e-05, 0.00324, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City578/insert", Data: [6]float64{31, 0.04229, 0.04229, 7e-05, 0.02088, 0.0005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City579/insert", Data: [6]float64{31, 0.01057, 0.01057, 7e-05, 0.00306, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City580/insert", Data: [6]float64{31, 0.03186, 0.03186, 7e-05, 0.01894, 0.0003800000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City581/insert", Data: [6]float64{31, 0.01573, 0.01573, 7e-05, 0.00837, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City582/insert", Data: [6]float64{31, 0.012150000000000001, 0.012150000000000001, 7e-05, 0.00316, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City583/insert", Data: [6]float64{31, 0.008580000000000003, 0.008580000000000003, 7e-05, 0.00102, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City584/insert", Data: [6]float64{31, 0.01291, 0.01291, 7e-05, 0.00578, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City585/insert", Data: [6]float64{31, 0.01907, 0.01907, 7e-05, 0.00594, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City586/insert", Data: [6]float64{31, 0.00732, 0.00732, 7e-05, 0.00069, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City587/insert", Data: [6]float64{31, 0.0078000000000000005, 0.0078000000000000005, 7e-05, 0.00108, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City588/insert", Data: [6]float64{31, 0.02179, 0.02179, 7e-05, 0.00626, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City589/insert", Data: [6]float64{31, 0.01238, 0.01238, 7e-05, 0.0037, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City590/insert", Data: [6]float64{31, 0.01337, 0.01337, 7e-05, 0.00286, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City591/insert", Data: [6]float64{31, 0.01151, 0.01151, 7e-05, 0.00467, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City592/insert", Data: [6]float64{31, 0.011880000000000002, 0.011880000000000002, 7e-05, 0.00343, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City593/insert", Data: [6]float64{31, 0.0132, 0.0132, 7e-05, 0.00383, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City594/insert", Data: [6]float64{31, 0.005980000000000001, 0.005980000000000001, 7e-05, 0.00065, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City595/insert", Data: [6]float64{31, 0.008589999999999999, 0.008589999999999999, 7e-05, 0.00084, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City596/insert", Data: [6]float64{31, 0.014790000000000001, 0.014790000000000001, 7e-05, 0.00892, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City597/insert", Data: [6]float64{31, 0.0188, 0.0188, 7e-05, 0.0053, 6.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City598/insert", Data: [6]float64{31, 0.011359999999999999, 0.011359999999999999, 7e-05, 0.00527, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City599/insert", Data: [6]float64{31, 0.01326, 0.01326, 7e-05, 0.00428, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City600/insert", Data: [6]float64{31, 0.0247, 0.0247, 7e-05, 0.01228, 0.00016999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City601/insert", Data: [6]float64{31, 0.02478, 0.02478, 7e-05, 0.0054, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City602/insert", Data: [6]float64{31, 0.02857, 0.02857, 7e-05, 0.00829, 0.00011999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City603/insert", Data: [6]float64{31, 0.00784, 0.00784, 7e-05, 0.00163, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City604/insert", Data: [6]float64{31, 0.03673, 0.03673, 7e-05, 0.02643, 0.00072}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City605/insert", Data: [6]float64{31, 0.01135, 0.01135, 7e-05, 0.00466, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City606/insert", Data: [6]float64{31, 0.00902, 0.00902, 7e-05, 0.00323, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City607/insert", Data: [6]float64{31, 0.01403, 0.01403, 7e-05, 0.00762, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City608/insert", Data: [6]float64{31, 0.013380000000000005, 0.013380000000000005, 7e-05, 0.00394, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City609/insert", Data: [6]float64{31, 0.00857, 0.00857, 7e-05, 0.00189, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City610/insert", Data: [6]float64{31, 0.008539999999999999, 0.008539999999999999, 7e-05, 0.00172, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City611/insert", Data: [6]float64{31, 0.013299999999999998, 0.013299999999999998, 7e-05, 0.00416, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City612/insert", Data: [6]float64{31, 0.011590000000000001, 0.011590000000000001, 7e-05, 0.00259, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City613/insert", Data: [6]float64{31, 0.014560000000000002, 0.014560000000000002, 7e-05, 0.00368, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City614/insert", Data: [6]float64{31, 0.01308, 0.01308, 7e-05, 0.00432, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City615/insert", Data: [6]float64{31, 0.006680000000000002, 0.006680000000000002, 7e-05, 0.00063, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City616/insert", Data: [6]float64{31, 0.01417, 0.01417, 7e-05, 0.00424, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City617/insert", Data: [6]float64{31, 0.02067, 0.02067, 7e-05, 0.00866, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City618/insert", Data: [6]float64{31, 0.02023, 0.02023, 7e-05, 0.01221, 0.00015}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City619/insert", Data: [6]float64{31, 0.010889999999999999, 0.010889999999999999, 7e-05, 0.00331, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City620/insert", Data: [6]float64{31, 0.0125, 0.0125, 7e-05, 0.00424, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City621/insert", Data: [6]float64{31, 0.015570000000000002, 0.015570000000000002, 7e-05, 0.0037, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City622/insert", Data: [6]float64{31, 0.01409, 0.01409, 7e-05, 0.00425, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City623/insert", Data: [6]float64{31, 0.01812, 0.01812, 7e-05, 0.00492, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City624/insert", Data: [6]float64{31, 0.01454, 0.01454, 7e-05, 0.00381, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City625/insert", Data: [6]float64{31, 0.022400000000000003, 0.022400000000000003, 7e-05, 0.01003, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City626/insert", Data: [6]float64{31, 0.024069999999999998, 0.024069999999999998, 7e-05, 0.00834, 0.00010999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City627/insert", Data: [6]float64{31, 0.01348, 0.01348, 7e-05, 0.00521, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City628/insert", Data: [6]float64{31, 0.02498, 0.02498, 7e-05, 0.01074, 0.00014000000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City629/insert", Data: [6]float64{31, 0.015549999999999998, 0.015549999999999998, 7e-05, 0.00575, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City630/insert", Data: [6]float64{31, 0.00762, 0.00762, 7e-05, 0.00083, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City631/insert", Data: [6]float64{31, 0.009430000000000001, 0.009430000000000001, 7e-05, 0.00183, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City632/insert", Data: [6]float64{31, 0.02893, 0.02893, 7e-05, 0.02026, 0.00041}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City633/insert", Data: [6]float64{31, 0.01445, 0.01445, 7e-05, 0.00419, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City634/insert", Data: [6]float64{31, 0.02721, 0.02721, 7e-05, 0.01349, 0.0002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City635/insert", Data: [6]float64{31, 0.01614, 0.01614, 7e-05, 0.00451, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City636/insert", Data: [6]float64{31, 0.01187, 0.01187, 7e-05, 0.00267, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City637/insert", Data: [6]float64{31, 0.023010000000000003, 0.023010000000000003, 7e-05, 0.00516, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City638/insert", Data: [6]float64{31, 0.01371, 0.01371, 7e-05, 0.00361, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City639/insert", Data: [6]float64{31, 0.024460000000000003, 0.024460000000000003, 7e-05, 0.00975, 0.00017}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City640/insert", Data: [6]float64{31, 0.01655, 0.01655, 7e-05, 0.00421, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City641/insert", Data: [6]float64{31, 0.00947, 0.00947, 7e-05, 0.0038, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City642/insert", Data: [6]float64{31, 0.01456, 0.01456, 7e-05, 0.00543, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City643/insert", Data: [6]float64{31, 0.01465, 0.01465, 7e-05, 0.0032, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City644/insert", Data: [6]float64{31, 0.008910000000000001, 0.008910000000000001, 7e-05, 0.00129, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City645/insert", Data: [6]float64{31, 0.008370000000000002, 0.008370000000000002, 7e-05, 0.00161, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City646/insert", Data: [6]float64{31, 0.005430000000000001, 0.005430000000000001, 7e-05, 0.00066, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City647/insert", Data: [6]float64{31, 0.01875, 0.01875, 7e-05, 0.00765, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City648/insert", Data: [6]float64{31, 0.007050000000000003, 0.007050000000000003, 7e-05, 0.00099, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City649/insert", Data: [6]float64{31, 0.01052, 0.01052, 7e-05, 0.00579, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City650/insert", Data: [6]float64{31, 0.0049, 0.0049, 7e-05, 0.00053, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City651/insert", Data: [6]float64{31, 0.00518, 0.00518, 7e-05, 0.00093, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City652/insert", Data: [6]float64{31, 0.03553, 0.03553, 7e-05, 0.02674, 0.0007300000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City653/insert", Data: [6]float64{31, 0.015529999999999999, 0.015529999999999999, 7e-05, 0.00598, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City654/insert", Data: [6]float64{31, 0.03339, 0.03339, 7e-05, 0.01774, 0.00038}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City655/insert", Data: [6]float64{31, 0.024470000000000006, 0.024470000000000006, 7e-05, 0.0149, 0.00024}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City656/insert", Data: [6]float64{31, 0.0059900000000000005, 0.0059900000000000005, 7e-05, 0.00093, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City657/insert", Data: [6]float64{31, 0.038439999999999995, 0.038439999999999995, 7e-05, 0.02525, 0.00067}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City658/insert", Data: [6]float64{31, 0.01403, 0.01403, 7e-05, 0.00434, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City659/insert", Data: [6]float64{31, 0.00609, 0.00609, 7e-05, 0.00081, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City660/insert", Data: [6]float64{31, 0.013830000000000002, 0.013830000000000002, 7e-05, 0.0051, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City661/insert", Data: [6]float64{31, 0.022319999999999996, 0.022319999999999996, 7e-05, 0.00883, 0.00016}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City662/insert", Data: [6]float64{31, 0.04442000000000001, 0.04442000000000001, 7e-05, 0.01752, 0.0003800000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City663/insert", Data: [6]float64{31, 0.013560000000000004, 0.013560000000000004, 7e-05, 0.008, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City664/insert", Data: [6]float64{31, 0.012669999999999999, 0.012669999999999999, 7e-05, 0.00388, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City665/insert", Data: [6]float64{31, 0.01334, 0.01334, 7e-05, 0.00477, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City666/insert", Data: [6]float64{31, 0.015040000000000001, 0.015040000000000001, 7e-05, 0.00649, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City667/insert", Data: [6]float64{31, 0.014400000000000001, 0.014400000000000001, 7e-05, 0.00507, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City668/insert", Data: [6]float64{31, 0.010210000000000002, 0.010210000000000002, 7e-05, 0.00462, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City669/insert", Data: [6]float64{31, 0.01344, 0.01344, 7e-05, 0.0044, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City670/insert", Data: [6]float64{31, 0.012700000000000003, 0.012700000000000003, 7e-05, 0.00417, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City671/insert", Data: [6]float64{31, 0.008300000000000004, 0.008300000000000004, 7e-05, 0.00205, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City672/insert", Data: [6]float64{31, 0.01851, 0.01851, 7e-05, 0.01158, 0.00013}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City673/insert", Data: [6]float64{31, 0.013330000000000002, 0.013330000000000002, 7e-05, 0.00758, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City674/insert", Data: [6]float64{31, 0.014520000000000003, 0.014520000000000003, 7e-05, 0.00625, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City675/insert", Data: [6]float64{31, 0.00865, 0.00865, 7e-05, 0.00233, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City676/insert", Data: [6]float64{31, 0.009720000000000001, 0.009720000000000001, 7e-05, 0.00444, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City677/insert", Data: [6]float64{31, 0.008070000000000003, 0.008070000000000003, 7e-05, 0.00167, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City678/insert", Data: [6]float64{31, 0.018549999999999997, 0.018549999999999997, 7e-05, 0.00989, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City679/insert", Data: [6]float64{31, 0.009470000000000001, 0.009470000000000001, 7e-05, 0.0041, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City680/insert", Data: [6]float64{31, 0.018749999999999996, 0.018749999999999996, 7e-05, 0.00884, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City681/insert", Data: [6]float64{31, 0.005540000000000001, 0.005540000000000001, 7e-05, 0.00063, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City682/insert", Data: [6]float64{31, 0.010760000000000002, 0.010760000000000002, 7e-05, 0.00495, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City683/insert", Data: [6]float64{31, 0.00623, 0.00623, 7e-05, 0.00105, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City684/insert", Data: [6]float64{31, 0.01578, 0.01578, 7e-05, 0.00358, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City685/insert", Data: [6]float64{31, 0.009230000000000002, 0.009230000000000002, 7e-05, 0.00281, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City686/insert", Data: [6]float64{31, 0.02162, 0.02162, 7e-05, 0.01201, 0.00015}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City687/insert", Data: [6]float64{31, 0.024680000000000004, 0.024680000000000004, 7e-05, 0.00859, 0.00015}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City688/insert", Data: [6]float64{31, 0.00771, 0.00771, 7e-05, 0.00278, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City689/insert", Data: [6]float64{31, 0.0066500000000000005, 0.0066500000000000005, 7e-05, 0.00168, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City690/insert", Data: [6]float64{31, 0.01002, 0.01002, 7e-05, 0.00514, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City691/insert", Data: [6]float64{31, 0.007270000000000001, 0.007270000000000001, 8e-05, 0.00123, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City692/insert", Data: [6]float64{31, 0.00732, 0.00732, 7e-05, 0.00147, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City693/insert", Data: [6]float64{31, 0.015240000000000002, 0.015240000000000002, 7e-05, 0.00684, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City694/insert", Data: [6]float64{31, 0.014760000000000002, 0.014760000000000002, 7e-05, 0.00814, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City695/insert", Data: [6]float64{31, 0.01183, 0.01183, 7e-05, 0.00683, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City696/insert", Data: [6]float64{31, 0.04116999999999999, 0.04116999999999999, 7e-05, 0.02143, 0.00059}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City697/insert", Data: [6]float64{31, 0.007260000000000001, 0.007260000000000001, 7e-05, 0.00211, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City698/insert", Data: [6]float64{31, 0.04682999999999998, 0.04682999999999998, 7e-05, 0.02131, 0.00081}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City699/insert", Data: [6]float64{31, 0.012800000000000002, 0.012800000000000002, 7e-05, 0.00401, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City700/insert", Data: [6]float64{31, 0.009010000000000002, 0.009010000000000002, 7e-05, 0.00398, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City701/insert", Data: [6]float64{31, 0.007540000000000002, 0.007540000000000002, 7e-05, 0.003, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City702/insert", Data: [6]float64{31, 0.014250000000000002, 0.014250000000000002, 7e-05, 0.00881, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City703/insert", Data: [6]float64{31, 0.006560000000000001, 0.006560000000000001, 7e-05, 0.00162, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City704/insert", Data: [6]float64{31, 0.03784, 0.03784, 7e-05, 0.02752, 0.0007800000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City705/insert", Data: [6]float64{31, 0.014910000000000001, 0.014910000000000001, 7e-05, 0.00539, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City706/insert", Data: [6]float64{31, 0.22379999999999994, 0.22379999999999994, 7e-05, 0.21574, 0.046560000000000004}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City707/insert", Data: [6]float64{31, 0.00694, 0.00694, 7e-05, 0.00124, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City708/insert", Data: [6]float64{31, 0.006190000000000001, 0.006190000000000001, 7e-05, 0.00137, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City709/insert", Data: [6]float64{31, 0.006590000000000001, 0.006590000000000001, 7e-05, 0.00088, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City710/insert", Data: [6]float64{31, 0.016270000000000003, 0.016270000000000003, 7e-05, 0.00785, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City711/insert", Data: [6]float64{31, 0.00971, 0.00971, 7e-05, 0.00296, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City712/insert", Data: [6]float64{31, 0.01593, 0.01593, 7e-05, 0.01056, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City713/insert", Data: [6]float64{31, 0.03676999999999999, 0.03676999999999999, 7e-05, 0.02249, 0.0006000000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City714/insert", Data: [6]float64{31, 0.010210000000000002, 0.010210000000000002, 7e-05, 0.00411, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City715/insert", Data: [6]float64{31, 0.005930000000000003, 0.005930000000000003, 7e-05, 0.00115, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City716/insert", Data: [6]float64{31, 0.006420000000000001, 0.006420000000000001, 7e-05, 0.00163, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City717/insert", Data: [6]float64{31, 0.03352, 0.03352, 7e-05, 0.02425, 0.0006000000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City718/insert", Data: [6]float64{31, 0.007120000000000001, 0.007120000000000001, 7e-05, 0.00192, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City719/insert", Data: [6]float64{31, 0.03736, 0.03736, 7e-05, 0.02317, 0.0005700000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City720/insert", Data: [6]float64{31, 0.005370000000000002, 0.005370000000000002, 7e-05, 0.00083, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City721/insert", Data: [6]float64{31, 0.00634, 0.00634, 7e-05, 0.0013, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City722/insert", Data: [6]float64{31, 0.00974, 0.00974, 7e-05, 0.00398, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City723/insert", Data: [6]float64{31, 0.030389999999999997, 0.030389999999999997, 7e-05, 0.01169, 0.00021999999999999998}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City724/insert", Data: [6]float64{31, 0.008769999999999998, 0.008769999999999998, 7e-05, 0.00356, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City725/insert", Data: [6]float64{31, 0.01294, 0.01294, 7e-05, 0.00465, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City726/insert", Data: [6]float64{31, 0.01623, 0.01623, 7e-05, 0.00589, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City727/insert", Data: [6]float64{31, 0.00742, 0.00742, 7e-05, 0.00257, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City728/insert", Data: [6]float64{31, 0.015600000000000001, 0.015600000000000001, 7e-05, 0.0065, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City729/insert", Data: [6]float64{31, 0.004670000000000002, 0.004670000000000002, 7e-05, 0.00089, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City730/insert", Data: [6]float64{31, 0.01643, 0.01643, 7e-05, 0.00882, 9e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City731/insert", Data: [6]float64{31, 0.010220000000000002, 0.010220000000000002, 7e-05, 0.00559, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City732/insert", Data: [6]float64{31, 0.006650000000000002, 0.006650000000000002, 7e-05, 0.00128, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City733/insert", Data: [6]float64{31, 0.019930000000000003, 0.019930000000000003, 7e-05, 0.00838, 8.999999999999999e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City734/insert", Data: [6]float64{31, 0.02365, 0.02365, 7e-05, 0.0098, 0.00016}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City735/insert", Data: [6]float64{31, 0.012230000000000003, 0.012230000000000003, 8e-05, 0.00673, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City736/insert", Data: [6]float64{31, 0.005710000000000001, 0.005710000000000001, 7e-05, 0.00077, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City737/insert", Data: [6]float64{31, 0.015870000000000002, 0.015870000000000002, 7e-05, 0.00778, 7.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City738/insert", Data: [6]float64{31, 0.043480000000000005, 0.043480000000000005, 7e-05, 0.03785, 0.00143}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City739/insert", Data: [6]float64{31, 0.00507, 0.00507, 7e-05, 0.00049, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City740/insert", Data: [6]float64{31, 0.02401, 0.02401, 7e-05, 0.01841, 0.00034}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City741/insert", Data: [6]float64{31, 0.007430000000000003, 0.007430000000000003, 8e-05, 0.00177, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City742/insert", Data: [6]float64{31, 0.004970000000000003, 0.004970000000000003, 7e-05, 0.00121, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City743/insert", Data: [6]float64{31, 0.029890000000000003, 0.029890000000000003, 7e-05, 0.02471, 0.00061}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City744/insert", Data: [6]float64{31, 0.008, 0.008, 7e-05, 0.00319, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City745/insert", Data: [6]float64{31, 0.006530000000000001, 0.006530000000000001, 7e-05, 0.00277, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City746/insert", Data: [6]float64{31, 0.011310000000000002, 0.011310000000000002, 7e-05, 0.00412, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City747/insert", Data: [6]float64{31, 0.00432, 0.00432, 7e-05, 0.00061, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City748/insert", Data: [6]float64{31, 0.01513, 0.01513, 7e-05, 0.01116, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City749/insert", Data: [6]float64{31, 0.037819999999999986, 0.037819999999999986, 7e-05, 0.0259, 0.00072}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City750/insert", Data: [6]float64{31, 0.007780000000000002, 0.007780000000000002, 7e-05, 0.00294, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City751/insert", Data: [6]float64{31, 0.004540000000000003, 0.004540000000000003, 7e-05, 0.00123, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City752/insert", Data: [6]float64{31, 0.03226, 0.03226, 7e-05, 0.02615, 0.00068}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City753/insert", Data: [6]float64{31, 0.007190000000000001, 0.007190000000000001, 7e-05, 0.00314, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City754/insert", Data: [6]float64{31, 0.015170000000000005, 0.015170000000000005, 7e-05, 0.00576, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City755/insert", Data: [6]float64{31, 0.009790000000000004, 0.009790000000000004, 7e-05, 0.00494, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City756/insert", Data: [6]float64{31, 0.011780000000000002, 0.011780000000000002, 7e-05, 0.004, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City757/insert", Data: [6]float64{31, 0.004550000000000001, 0.004550000000000001, 7e-05, 0.00064, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City758/insert", Data: [6]float64{31, 0.03601, 0.03601, 7e-05, 0.01782, 0.00042000000000000007}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City759/insert", Data: [6]float64{31, 0.005010000000000002, 0.005010000000000002, 7e-05, 0.00099, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City760/insert", Data: [6]float64{31, 0.007430000000000002, 0.007430000000000002, 7e-05, 0.00203, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City761/insert", Data: [6]float64{31, 0.01924, 0.01924, 7e-05, 0.0145, 0.00021}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City762/insert", Data: [6]float64{31, 0.003990000000000002, 0.003990000000000002, 7e-05, 0.00058, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City763/insert", Data: [6]float64{31, 0.007340000000000001, 0.007340000000000001, 7e-05, 0.00264, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City764/insert", Data: [6]float64{31, 0.005730000000000002, 0.005730000000000002, 7e-05, 0.00137, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City765/insert", Data: [6]float64{31, 0.01079, 0.01079, 7e-05, 0.0064, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City766/insert", Data: [6]float64{31, 0.005990000000000001, 0.005990000000000001, 7e-05, 0.00163, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City767/insert", Data: [6]float64{31, 0.03331, 0.03331, 7e-05, 0.01786, 0.00039000000000000005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City768/insert", Data: [6]float64{31, 0.011970000000000001, 0.011970000000000001, 7e-05, 0.00259, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City769/insert", Data: [6]float64{31, 0.02986, 0.02986, 7e-05, 0.0246, 0.00061}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City770/insert", Data: [6]float64{31, 0.0048800000000000015, 0.0048800000000000015, 7e-05, 0.00066, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City771/insert", Data: [6]float64{31, 0.01669, 0.01669, 7e-05, 0.01162, 0.00014}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City772/insert", Data: [6]float64{31, 0.005390000000000002, 0.005390000000000002, 7e-05, 0.00108, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City773/insert", Data: [6]float64{31, 0.06943999999999999, 0.06943999999999999, 7e-05, 0.02516, 0.00135}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City774/insert", Data: [6]float64{31, 0.02104, 0.02104, 7e-05, 0.0087, 0.00014000000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City775/insert", Data: [6]float64{31, 0.005420000000000002, 0.005420000000000002, 7e-05, 0.00066, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City776/insert", Data: [6]float64{31, 0.019080000000000003, 0.019080000000000003, 7e-05, 0.01101, 0.00013000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City777/insert", Data: [6]float64{31, 0.01463, 0.01463, 7e-05, 0.00675, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City778/insert", Data: [6]float64{31, 0.0050700000000000025, 0.0050700000000000025, 7e-05, 0.00066, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City779/insert", Data: [6]float64{31, 0.0108, 0.0108, 7e-05, 0.00402, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City780/insert", Data: [6]float64{31, 0.0409, 0.0409, 7e-05, 0.03056, 0.0009500000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City781/insert", Data: [6]float64{31, 0.02452, 0.02452, 7e-05, 0.01596, 0.00027}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City782/insert", Data: [6]float64{31, 0.012310000000000003, 0.012310000000000003, 7e-05, 0.00772, 6e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City783/insert", Data: [6]float64{31, 0.01663, 0.01663, 7e-05, 0.011, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City784/insert", Data: [6]float64{31, 0.011760000000000001, 0.011760000000000001, 7e-05, 0.00476, 3.0000000000000004e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City785/insert", Data: [6]float64{31, 0.026840000000000006, 0.026840000000000006, 7e-05, 0.0147, 0.00025}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City786/insert", Data: [6]float64{31, 0.013260000000000001, 0.013260000000000001, 7e-05, 0.00384, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City787/insert", Data: [6]float64{31, 0.00979, 0.00979, 7e-05, 0.00496, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City788/insert", Data: [6]float64{31, 0.0063200000000000036, 0.0063200000000000036, 8e-05, 0.00093, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City789/insert", Data: [6]float64{31, 0.005280000000000003, 0.005280000000000003, 7e-05, 0.00075, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City790/insert", Data: [6]float64{31, 0.032729999999999974, 0.032729999999999974, 7e-05, 0.02783, 0.00077}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City791/insert", Data: [6]float64{31, 0.02947, 0.02947, 7e-05, 0.02053, 0.00043000000000000004}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City792/insert", Data: [6]float64{31, 0.008980000000000004, 0.008980000000000004, 7e-05, 0.00281, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City793/insert", Data: [6]float64{31, 0.020350000000000007, 0.020350000000000007, 7e-05, 0.01598, 0.00026}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City794/insert", Data: [6]float64{31, 0.006400000000000002, 0.006400000000000002, 7e-05, 0.00198, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City795/insert", Data: [6]float64{31, 0.05075, 0.05075, 7e-05, 0.0268, 0.0010400000000000001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City796/insert", Data: [6]float64{31, 0.010900000000000002, 0.010900000000000002, 7e-05, 0.00584, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City797/insert", Data: [6]float64{31, 0.014860000000000002, 0.014860000000000002, 7e-05, 0.006, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City798/insert", Data: [6]float64{31, 0.005000000000000003, 0.005000000000000003, 7e-05, 0.00055, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City799/insert", Data: [6]float64{31, 0.011490000000000002, 0.011490000000000002, 7e-05, 0.00379, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City800/insert", Data: [6]float64{31, 0.02144, 0.02144, 7e-05, 0.01658, 0.00028}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City801/insert", Data: [6]float64{31, 0.030930000000000003, 0.030930000000000003, 7e-05, 0.02229, 0.00051}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City802/insert", Data: [6]float64{31, 0.013370000000000003, 0.013370000000000003, 7e-05, 0.00688, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City803/insert", Data: [6]float64{31, 0.009190000000000002, 0.009190000000000002, 7e-05, 0.00267, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City804/insert", Data: [6]float64{31, 0.019320000000000004, 0.019320000000000004, 7e-05, 0.01212, 0.00015999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City805/insert", Data: [6]float64{31, 0.004550000000000003, 0.004550000000000003, 7e-05, 0.00068, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City806/insert", Data: [6]float64{31, 0.025160000000000002, 0.025160000000000002, 7e-05, 0.01169, 0.00021999999999999998}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City807/insert", Data: [6]float64{31, 0.005360000000000002, 0.005360000000000002, 7e-05, 0.0007, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City808/insert", Data: [6]float64{31, 0.004680000000000001, 0.004680000000000001, 8e-05, 0.00049, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City809/insert", Data: [6]float64{31, 0.03319999999999998, 0.03319999999999998, 7e-05, 0.02609, 0.00068}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City810/insert", Data: [6]float64{31, 0.009960000000000004, 0.009960000000000004, 7e-05, 0.00506, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City811/insert", Data: [6]float64{31, 0.02167, 0.02167, 7e-05, 0.01718, 0.0003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City812/insert", Data: [6]float64{31, 0.019770000000000003, 0.019770000000000003, 7e-05, 0.01237, 0.00015999999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City813/insert", Data: [6]float64{31, 0.026959999999999998, 0.026959999999999998, 7e-05, 0.01536, 0.00027}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City814/insert", Data: [6]float64{31, 0.012880000000000003, 0.012880000000000003, 7e-05, 0.00816, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City815/insert", Data: [6]float64{31, 0.01936, 0.01936, 7e-05, 0.01015, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City816/insert", Data: [6]float64{31, 0.00829, 0.00829, 7e-05, 0.00365, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City817/insert", Data: [6]float64{31, 0.004850000000000004, 0.004850000000000004, 8e-05, 0.00072, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City818/insert", Data: [6]float64{31, 0.004590000000000002, 0.004590000000000002, 7e-05, 0.00057, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City819/insert", Data: [6]float64{31, 0.004740000000000004, 0.004740000000000004, 7e-05, 0.0007, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City820/insert", Data: [6]float64{31, 0.024380000000000006, 0.024380000000000006, 7e-05, 0.01649, 0.00028000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City821/insert", Data: [6]float64{31, 0.005120000000000003, 0.005120000000000003, 7e-05, 0.00077, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City822/insert", Data: [6]float64{31, 0.007870000000000002, 0.007870000000000002, 7e-05, 0.00289, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City823/insert", Data: [6]float64{31, 0.04348999999999999, 0.04348999999999999, 7e-05, 0.02629, 0.00085}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City824/insert", Data: [6]float64{31, 0.03411999999999999, 0.03411999999999999, 7e-05, 0.0247, 0.0006399999999999999}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City825/insert", Data: [6]float64{31, 0.02385, 0.02385, 8e-05, 0.01884, 0.00035}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City826/insert", Data: [6]float64{31, 0.011790000000000004, 0.011790000000000004, 8e-05, 0.00376, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City827/insert", Data: [6]float64{31, 0.006470000000000003, 0.006470000000000003, 7e-05, 0.00229, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City828/insert", Data: [6]float64{31, 0.005220000000000002, 0.005220000000000002, 7e-05, 0.0009, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City829/insert", Data: [6]float64{31, 0.026350000000000002, 0.026350000000000002, 7e-05, 0.01966, 0.00039}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City830/insert", Data: [6]float64{31, 0.00909, 0.00909, 7e-05, 0.00302, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City831/insert", Data: [6]float64{31, 0.005350000000000002, 0.005350000000000002, 7e-05, 0.00094, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City832/insert", Data: [6]float64{31, 0.01346, 0.01346, 7e-05, 0.00813, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City833/insert", Data: [6]float64{31, 0.005250000000000002, 0.005250000000000002, 8e-05, 0.00104, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City834/insert", Data: [6]float64{31, 0.006420000000000002, 0.006420000000000002, 8e-05, 0.00112, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City835/insert", Data: [6]float64{31, 0.04225, 0.04225, 7e-05, 0.03284, 0.00109}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City836/insert", Data: [6]float64{31, 0.01361, 0.01361, 7e-05, 0.00848, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City837/insert", Data: [6]float64{31, 0.008540000000000002, 0.008540000000000002, 7e-05, 0.00302, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City838/insert", Data: [6]float64{31, 0.06617999999999997, 0.06617999999999997, 7e-05, 0.0344, 0.0019000000000000002}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City839/insert", Data: [6]float64{31, 0.0336, 0.0336, 7e-05, 0.02808, 0.00079}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City840/insert", Data: [6]float64{31, 0.005270000000000003, 0.005270000000000003, 7e-05, 0.00105, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City841/insert", Data: [6]float64{31, 0.010350000000000003, 0.010350000000000003, 7e-05, 0.00404, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City842/insert", Data: [6]float64{31, 0.010660000000000001, 0.010660000000000001, 7e-05, 0.00549, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City843/insert", Data: [6]float64{31, 0.016970000000000002, 0.016970000000000002, 7e-05, 0.01098, 0.00012}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City844/insert", Data: [6]float64{31, 0.01433, 0.01433, 7e-05, 0.00501, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City845/insert", Data: [6]float64{31, 0.007020000000000001, 0.007020000000000001, 7e-05, 0.00112, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City846/insert", Data: [6]float64{31, 0.04421999999999997, 0.04421999999999997, 7e-05, 0.02998, 0.00098}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City847/insert", Data: [6]float64{31, 0.007540000000000001, 0.007540000000000001, 7e-05, 0.00341, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City848/insert", Data: [6]float64{31, 0.009770000000000003, 0.009770000000000003, 7e-05, 0.00351, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City849/insert", Data: [6]float64{31, 0.0054500000000000035, 0.0054500000000000035, 8e-05, 0.00119, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City850/insert", Data: [6]float64{31, 0.009350000000000002, 0.009350000000000002, 7e-05, 0.00327, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City851/insert", Data: [6]float64{31, 0.00841, 0.00841, 7e-05, 0.00345, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City852/insert", Data: [6]float64{31, 0.02523, 0.02523, 7e-05, 0.01455, 0.00023}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City853/insert", Data: [6]float64{31, 0.008690000000000002, 0.008690000000000002, 7e-05, 0.00285, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City854/insert", Data: [6]float64{31, 0.004780000000000003, 0.004780000000000003, 7e-05, 0.00084, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City855/insert", Data: [6]float64{31, 0.018849999999999995, 0.018849999999999995, 7e-05, 0.00974, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City856/insert", Data: [6]float64{31, 0.010020000000000003, 0.010020000000000003, 7e-05, 0.00617, 4e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City857/insert", Data: [6]float64{31, 0.005250000000000004, 0.005250000000000004, 8e-05, 0.00117, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City858/insert", Data: [6]float64{31, 0.020210000000000002, 0.020210000000000002, 7e-05, 0.01154, 0.00015}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City859/insert", Data: [6]float64{31, 0.024650000000000002, 0.024650000000000002, 7e-05, 0.01968, 0.00039}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City860/insert", Data: [6]float64{31, 0.028480000000000002, 0.028480000000000002, 7e-05, 0.01362, 0.00023}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City861/insert", Data: [6]float64{31, 0.040769999999999994, 0.040769999999999994, 8e-05, 0.02018, 0.00067}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City862/insert", Data: [6]float64{31, 0.06144999999999999, 0.06144999999999999, 7e-05, 0.03144, 0.00145}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City863/insert", Data: [6]float64{31, 0.030120000000000004, 0.030120000000000004, 7e-05, 0.01975, 0.00041}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City864/insert", Data: [6]float64{31, 0.00628, 0.00628, 7e-05, 0.00125, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City865/insert", Data: [6]float64{31, 0.02926, 0.02926, 7e-05, 0.01325, 0.00031}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City866/insert", Data: [6]float64{31, 0.005130000000000003, 0.005130000000000003, 7e-05, 0.00136, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City867/insert", Data: [6]float64{31, 0.0053500000000000015, 0.0053500000000000015, 7e-05, 0.00124, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City868/insert", Data: [6]float64{31, 0.004810000000000001, 0.004810000000000001, 7e-05, 0.00129, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City869/insert", Data: [6]float64{31, 0.01648, 0.01648, 7e-05, 0.00619, 7.000000000000001e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City870/insert", Data: [6]float64{31, 0.004640000000000003, 0.004640000000000003, 8e-05, 0.00101, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City871/insert", Data: [6]float64{31, 0.010310000000000003, 0.010310000000000003, 7e-05, 0.00558, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City872/insert", Data: [6]float64{31, 0.004910000000000003, 0.004910000000000003, 8e-05, 0.00106, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City873/insert", Data: [6]float64{31, 0.004550000000000003, 0.004550000000000003, 7e-05, 0.00094, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City874/insert", Data: [6]float64{31, 0.005250000000000003, 0.005250000000000003, 8e-05, 0.00118, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City875/insert", Data: [6]float64{31, 0.004320000000000002, 0.004320000000000002, 8e-05, 0.00086, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City876/insert", Data: [6]float64{31, 0.010860000000000003, 0.010860000000000003, 7e-05, 0.00572, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City877/insert", Data: [6]float64{31, 0.029560000000000003, 0.029560000000000003, 8e-05, 0.0254, 0.00065}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City878/insert", Data: [6]float64{31, 0.004730000000000002, 0.004730000000000002, 7e-05, 0.00097, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City879/insert", Data: [6]float64{31, 0.01406, 0.01406, 7e-05, 0.00866, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City880/insert", Data: [6]float64{31, 0.008610000000000001, 0.008610000000000001, 7e-05, 0.00422, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City881/insert", Data: [6]float64{31, 0.007870000000000004, 0.007870000000000004, 8e-05, 0.00386, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City882/insert", Data: [6]float64{31, 0.017570000000000002, 0.017570000000000002, 8e-05, 0.01336, 0.00018}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City883/insert", Data: [6]float64{31, 0.004200000000000002, 0.004200000000000002, 8e-05, 0.00079, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City884/insert", Data: [6]float64{31, 0.007370000000000003, 0.007370000000000003, 8e-05, 0.00342, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City885/insert", Data: [6]float64{31, 0.004220000000000002, 0.004220000000000002, 8e-05, 0.00067, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City886/insert", Data: [6]float64{31, 0.031200000000000002, 0.031200000000000002, 7e-05, 0.02565, 0.00066}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City887/insert", Data: [6]float64{31, 0.006030000000000002, 0.006030000000000002, 8e-05, 0.00229, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City888/insert", Data: [6]float64{31, 0.02821, 0.02821, 7e-05, 0.01146, 0.00020999999999999998}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City889/insert", Data: [6]float64{31, 0.02128, 0.02128, 7e-05, 0.00812, 0.00014}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City890/insert", Data: [6]float64{31, 0.004720000000000001, 0.004720000000000001, 8e-05, 0.00078, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City891/insert", Data: [6]float64{31, 0.014320000000000003, 0.014320000000000003, 7e-05, 0.00864, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City892/insert", Data: [6]float64{31, 0.00957, 0.00957, 8e-05, 0.0019, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City893/insert", Data: [6]float64{31, 0.0047300000000000016, 0.0047300000000000016, 8e-05, 0.0008, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City894/insert", Data: [6]float64{31, 0.005120000000000003, 0.005120000000000003, 7e-05, 0.00091, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City895/insert", Data: [6]float64{31, 0.025190000000000004, 0.025190000000000004, 8e-05, 0.01536, 0.00026000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City896/insert", Data: [6]float64{31, 0.023620000000000006, 0.023620000000000006, 7e-05, 0.01459, 0.00022}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City897/insert", Data: [6]float64{31, 0.005460000000000001, 0.005460000000000001, 7e-05, 0.00057, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City898/insert", Data: [6]float64{31, 0.007390000000000002, 0.007390000000000002, 8e-05, 0.00136, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City899/insert", Data: [6]float64{31, 0.06611999999999998, 0.06611999999999998, 8e-05, 0.03633, 0.0019199999999999998}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City900/insert", Data: [6]float64{31, 0.02688, 0.02688, 8e-05, 0.01704, 0.00030000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City901/insert", Data: [6]float64{31, 0.010920000000000003, 0.010920000000000003, 8e-05, 0.00536, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City902/insert", Data: [6]float64{31, 0.01159, 0.01159, 7e-05, 0.00579, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City903/insert", Data: [6]float64{31, 0.016210000000000002, 0.016210000000000002, 7e-05, 0.01061, 0.00011}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City904/insert", Data: [6]float64{31, 0.04205999999999997, 0.04205999999999997, 7e-05, 0.02804, 0.00086}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City905/insert", Data: [6]float64{31, 0.01838, 0.01838, 7e-05, 0.01319, 0.00017}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City906/insert", Data: [6]float64{31, 0.03813999999999998, 0.03813999999999998, 8e-05, 0.03109, 0.00097}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City907/insert", Data: [6]float64{31, 0.014760000000000002, 0.014760000000000002, 7e-05, 0.00618, 5e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City908/insert", Data: [6]float64{31, 0.01818, 0.01818, 7e-05, 0.0112, 0.00013}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City909/insert", Data: [6]float64{31, 0.025740000000000002, 0.025740000000000002, 8e-05, 0.01382, 0.00021}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City910/insert", Data: [6]float64{31, 0.009440000000000002, 0.009440000000000002, 8e-05, 0.00456, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City911/insert", Data: [6]float64{31, 0.006880000000000003, 0.006880000000000003, 8e-05, 0.00286, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City912/insert", Data: [6]float64{31, 0.03289999999999999, 0.03289999999999999, 8e-05, 0.02902, 0.00084}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City913/insert", Data: [6]float64{31, 0.027620000000000002, 0.027620000000000002, 7e-05, 0.0235, 0.00055}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City914/insert", Data: [6]float64{31, 0.034719999999999994, 0.034719999999999994, 8e-05, 0.03086, 0.00095}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City915/insert", Data: [6]float64{31, 0.009680000000000001, 0.009680000000000001, 8e-05, 0.00586, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City916/insert", Data: [6]float64{31, 0.004050000000000002, 0.004050000000000002, 8e-05, 0.00032, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City917/insert", Data: [6]float64{31, 0.0039200000000000025, 0.0039200000000000025, 7e-05, 0.00041, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City918/insert", Data: [6]float64{31, 0.004330000000000002, 0.004330000000000002, 7e-05, 0.00054, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City919/insert", Data: [6]float64{31, 0.005730000000000003, 0.005730000000000003, 7e-05, 0.00176, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City920/insert", Data: [6]float64{31, 0.003980000000000003, 0.003980000000000003, 7e-05, 0.0003, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City921/insert", Data: [6]float64{31, 0.004400000000000002, 0.004400000000000002, 8e-05, 0.00064, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City922/insert", Data: [6]float64{31, 0.029499999999999995, 0.029499999999999995, 7e-05, 0.01438, 0.00029}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City923/insert", Data: [6]float64{31, 0.02653, 0.02653, 7e-05, 0.02233, 0.0005}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City924/insert", Data: [6]float64{31, 0.004420000000000002, 0.004420000000000002, 8e-05, 0.00042, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City925/insert", Data: [6]float64{31, 0.004340000000000001, 0.004340000000000001, 7e-05, 0.00037, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City926/insert", Data: [6]float64{31, 0.03166, 0.03166, 7e-05, 0.02735, 0.00075}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City927/insert", Data: [6]float64{31, 0.03383999999999999, 0.03383999999999999, 7e-05, 0.02877, 0.00083}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City928/insert", Data: [6]float64{31, 0.0040700000000000024, 0.0040700000000000024, 7e-05, 0.00036, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City929/insert", Data: [6]float64{31, 0.005580000000000003, 0.005580000000000003, 8e-05, 0.00145, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City930/insert", Data: [6]float64{31, 0.0039000000000000016, 0.0039000000000000016, 7e-05, 0.0003, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City931/insert", Data: [6]float64{31, 0.010320000000000001, 0.010320000000000001, 8e-05, 0.00557, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City932/insert", Data: [6]float64{31, 0.03195999999999999, 0.03195999999999999, 7e-05, 0.02799, 0.00078}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City933/insert", Data: [6]float64{31, 0.009930000000000003, 0.009930000000000003, 7e-05, 0.00439, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City934/insert", Data: [6]float64{31, 0.003880000000000002, 0.003880000000000002, 7e-05, 0.00031, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City935/insert", Data: [6]float64{31, 0.008980000000000002, 0.008980000000000002, 8e-05, 0.00518, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City936/insert", Data: [6]float64{31, 0.004010000000000002, 0.004010000000000002, 7e-05, 0.00049, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City937/insert", Data: [6]float64{31, 0.003590000000000002, 0.003590000000000002, 7e-05, 0.00032, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City938/insert", Data: [6]float64{31, 0.02961, 0.02961, 7e-05, 0.02431, 0.00059}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City939/insert", Data: [6]float64{31, 0.013930000000000001, 0.013930000000000001, 7e-05, 0.00837, 7e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City940/insert", Data: [6]float64{31, 0.01889, 0.01889, 7e-05, 0.01523, 0.00023}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City941/insert", Data: [6]float64{31, 0.003910000000000002, 0.003910000000000002, 7e-05, 0.00033, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City942/insert", Data: [6]float64{31, 0.029290000000000004, 0.029290000000000004, 8e-05, 0.01386, 0.00031}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City943/insert", Data: [6]float64{31, 0.04140999999999998, 0.04140999999999998, 7e-05, 0.03762, 0.00142}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City944/insert", Data: [6]float64{31, 0.003500000000000003, 0.003500000000000003, 7e-05, 0.00036, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City945/insert", Data: [6]float64{31, 0.027170000000000003, 0.027170000000000003, 7e-05, 0.02331, 0.00054}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City946/insert", Data: [6]float64{31, 0.036250000000000004, 0.036250000000000004, 7e-05, 0.01716, 0.00041}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City947/insert", Data: [6]float64{31, 0.005320000000000002, 0.005320000000000002, 7e-05, 0.00168, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City948/insert", Data: [6]float64{31, 0.015470000000000003, 0.015470000000000003, 7e-05, 0.01158, 0.00013}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City949/insert", Data: [6]float64{31, 0.01552, 0.01552, 8e-05, 0.01177, 0.00014}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City950/insert", Data: [6]float64{31, 0.003960000000000003, 0.003960000000000003, 7e-05, 0.00038, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City951/insert", Data: [6]float64{31, 0.003900000000000003, 0.003900000000000003, 7e-05, 0.00037, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City952/insert", Data: [6]float64{31, 0.02093, 0.02093, 7e-05, 0.01704, 0.00029}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City953/insert", Data: [6]float64{31, 0.043269999999999975, 0.043269999999999975, 8e-05, 0.02367, 0.0007}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City954/insert", Data: [6]float64{31, 0.004710000000000001, 0.004710000000000001, 7e-05, 0.00065, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City955/insert", Data: [6]float64{31, 0.00474, 0.00474, 7e-05, 0.00051, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City956/insert", Data: [6]float64{31, 0.003980000000000002, 0.003980000000000002, 7e-05, 0.00047, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City957/insert", Data: [6]float64{31, 0.03293, 0.03293, 7e-05, 0.02124, 0.00051}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City958/insert", Data: [6]float64{31, 0.004340000000000003, 0.004340000000000003, 8e-05, 0.00037, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City959/insert", Data: [6]float64{31, 0.016120000000000002, 0.016120000000000002, 7e-05, 0.00966, 0.0001}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City960/insert", Data: [6]float64{31, 0.005210000000000001, 0.005210000000000001, 7e-05, 0.00114, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City961/insert", Data: [6]float64{31, 0.0039400000000000025, 0.0039400000000000025, 7e-05, 0.00038, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City962/insert", Data: [6]float64{31, 0.004570000000000002, 0.004570000000000002, 7e-05, 0.00118, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City963/insert", Data: [6]float64{31, 0.00442, 0.00442, 7e-05, 0.00076, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City964/insert", Data: [6]float64{31, 0.02559, 0.02559, 7e-05, 0.02054, 0.00042}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City965/insert", Data: [6]float64{31, 0.004980000000000002, 0.004980000000000002, 7e-05, 0.00129, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City966/insert", Data: [6]float64{31, 0.0038500000000000023, 0.0038500000000000023, 7e-05, 0.00036, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City967/insert", Data: [6]float64{31, 0.0037200000000000024, 0.0037200000000000024, 8e-05, 0.00032, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City968/insert", Data: [6]float64{31, 0.028239999999999998, 0.028239999999999998, 8e-05, 0.01275, 0.00030000000000000003}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City969/insert", Data: [6]float64{31, 0.003820000000000002, 0.003820000000000002, 7e-05, 0.00033, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City970/insert", Data: [6]float64{31, 0.015620000000000002, 0.015620000000000002, 8e-05, 0.01155, 0.00013}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City971/insert", Data: [6]float64{31, 0.0042400000000000024, 0.0042400000000000024, 7e-05, 0.00059, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City972/insert", Data: [6]float64{31, 0.028330000000000004, 0.028330000000000004, 7e-05, 0.01529, 0.00031}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City973/insert", Data: [6]float64{31, 0.029450000000000004, 0.029450000000000004, 7e-05, 0.02104, 0.00045000000000000004}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City974/insert", Data: [6]float64{31, 0.004590000000000003, 0.004590000000000003, 8e-05, 0.00063, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City975/insert", Data: [6]float64{31, 0.01827, 0.01827, 8e-05, 0.01382, 0.00019}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City976/insert", Data: [6]float64{31, 0.01718, 0.01718, 8e-05, 0.01308, 0.00017}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City977/insert", Data: [6]float64{31, 0.03017, 0.03017, 7e-05, 0.0253, 0.00064}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City978/insert", Data: [6]float64{31, 0.017470000000000003, 0.017470000000000003, 8e-05, 0.0129, 0.00017}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City979/insert", Data: [6]float64{31, 0.004540000000000003, 0.004540000000000003, 8e-05, 0.00058, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City980/insert", Data: [6]float64{31, 0.0070100000000000015, 0.0070100000000000015, 7e-05, 0.00295, 1e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City981/insert", Data: [6]float64{31, 0.0037400000000000016, 0.0037400000000000016, 8e-05, 0.00033, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City982/insert", Data: [6]float64{31, 0.0037100000000000015, 0.0037100000000000015, 7e-05, 0.0004, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City983/insert", Data: [6]float64{31, 0.009300000000000004, 0.009300000000000004, 8e-05, 0.00547, 3e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City984/insert", Data: [6]float64{31, 0.005080000000000004, 0.005080000000000004, 7e-05, 0.00136, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City985/insert", Data: [6]float64{31, 0.027290000000000002, 0.027290000000000002, 8e-05, 0.02051, 0.00043000000000000004}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City986/insert", Data: [6]float64{31, 0.004360000000000001, 0.004360000000000001, 8e-05, 0.0005, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City987/insert", Data: [6]float64{31, 0.010300000000000002, 0.010300000000000002, 8e-05, 0.00428, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City988/insert", Data: [6]float64{31, 0.03402, 0.03402, 7e-05, 0.02985, 0.00089}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City989/insert", Data: [6]float64{31, 0.004360000000000001, 0.004360000000000001, 8e-05, 0.00048, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City990/insert", Data: [6]float64{31, 0.004380000000000001, 0.004380000000000001, 8e-05, 0.0004, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City991/insert", Data: [6]float64{31, 0.011840000000000003, 0.011840000000000003, 8e-05, 0.00401, 2e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City992/insert", Data: [6]float64{31, 0.020540000000000003, 0.020540000000000003, 8e-05, 0.01501, 0.00023}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City993/insert", Data: [6]float64{31, 0.004620000000000002, 0.004620000000000002, 8e-05, 0.00059, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City994/insert", Data: [6]float64{31, 0.06958999999999997, 0.06958999999999997, 7e-05, 0.04748, 0.00244}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City995/insert", Data: [6]float64{31, 0.026510000000000002, 0.026510000000000002, 8e-05, 0.01233, 0.00026}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City996/insert", Data: [6]float64{31, 0.01237, 0.01237, 8e-05, 0.0088, 8e-05}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City997/insert", Data: [6]float64{31, 0.02997, 0.02997, 8e-05, 0.02659, 0.00071}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City998/insert", Data: [6]float64{31, 0.003670000000000002, 0.003670000000000002, 8e-05, 0.00027, 0}, Scoped: true},
		metric{Name: "Datastore/statement/MySQL/City999/insert", Data: [6]float64{31, 0.004980000000000001, 0.004980000000000001, 7e-05, 0.00167, 0}, Scoped: true},
		metric{Name: "Errors/WebTransaction/Uri/myblog2/mycity.php", Data: [6]float64{3, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Errors/all", Data: [6]float64{3, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Errors/allWeb", Data: [6]float64{3, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "HttpDispatcher", Data: [6]float64{361, 33.18916000000001, 0, 0.00041, 0.90685, 18.556699999999992}, Scoped: true},
		metric{Name: "Instance/Reporting", Data: [6]float64{21, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Memory/Physical", Data: [6]float64{361, 5061.906420000001, 0, 13.49219, 15.86719, 71101.64678999998}, Scoped: true},
		metric{Name: "Supportability/AnalyticsEvents/TotalEventsSeen", Data: [6]float64{361, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/AnalyticsEvents/TotalEventsSent", Data: [6]float64{361, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/Events/Customer/Seen", Data: [6]float64{0, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/Events/Customer/Sent", Data: [6]float64{0, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysql_query", Data: [6]float64{94, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::multi_query", Data: [6]float64{31000, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::mysqli", Data: [6]float64{106, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::query", Data: [6]float64{51, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::stmt_init", Data: [6]float64{24, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_connect", Data: [6]float64{41, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_prepare", Data: [6]float64{25, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_query", Data: [6]float64{75, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt::bind_param", Data: [6]float64{24, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt::execute", Data: [6]float64{24, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt::prepare", Data: [6]float64{24, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_bind_param", Data: [6]float64{41, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_execute", Data: [6]float64{25016, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_init", Data: [6]float64{16, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_prepare", Data: [6]float64{16, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/InstrumentedFunction/sleepy_time", Data: [6]float64{35, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/MetricsDropped", Data: [6]float64{1310, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/api/add_custom_tracer", Data: [6]float64{35, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/api/get_browser_timing_footer", Data: [6]float64{152, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/api/get_browser_timing_header", Data: [6]float64{152, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/api/name_transaction", Data: [6]float64{32, 0, 0, 0, 0, 0}, Scoped: true},
		metric{Name: "Supportability/execute/user/call_count", Data: [6]float64{361, 0.00035000000000000027, 0.00035000000000000027, 0, 1e-05, 0}, Scoped: true},
		metric{Name: "WebTransaction", Data: [6]float64{361, 33.18916000000001, 6.48498, 0.00041, 0.90685, 18.556699999999992}, Scoped: true},
		metric{Name: "WebTransaction/Custom/my display php page", Data: [6]float64{32, 0.5338700000000002, 0.5042800000000001, 0.0011, 0.03795, 0.01233}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/accentquery.php", Data: [6]float64{28, 0.66593, 0.40279, 0.00078, 0.09748, 0.038239999999999996}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/backslashquote.php", Data: [6]float64{23, 0.8756999999999999, 0.71145, 0.00394, 0.0558, 0.03807}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/displayfunct.php", Data: [6]float64{35, 1.2758399999999999, 0.82581, 0.00112, 0.16208, 0.08119000000000003}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/district.php", Data: [6]float64{16, 0.37147, 0.34234, 0.00213, 0.06418, 0.01435}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/districtoo.php", Data: [6]float64{24, 0.50529, 0.47634000000000004, 0.00141, 0.05316, 0.01607}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/mycity.php", Data: [6]float64{25, 9.297749999999999, 0.80901, 0.12335, 0.69294, 4.719230000000001}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_113.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_122.php", Data: [6]float64{1, 0.00057, 0.00057, 0.00057, 0.00057, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_125.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_129.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_150.php", Data: [6]float64{1, 0.00069, 0.00069, 0.00069, 0.00069, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_162.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_17.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_181.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_185.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_191.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_199.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_202.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_208.php", Data: [6]float64{1, 0.00069, 0.00069, 0.00069, 0.00069, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_210.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_22.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_222.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_226.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_227.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_243.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_25.php", Data: [6]float64{1, 0.00045, 0.00045, 0.00045, 0.00045, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_252.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_261.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_272.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_280.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_287.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_29.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_293.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_305.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_312.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_34.php", Data: [6]float64{1, 0.00054, 0.00054, 0.00054, 0.00054, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_346.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_352.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_353.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_357.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_360.php", Data: [6]float64{1, 0.00061, 0.00061, 0.00061, 0.00061, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_365.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_369.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_376.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_378.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_386.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_389.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_401.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_405.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_42.php", Data: [6]float64{1, 0.00057, 0.00057, 0.00057, 0.00057, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_437.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_44.php", Data: [6]float64{1, 0.00041, 0.00041, 0.00041, 0.00041, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_441.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_466.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_489.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_496.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_497.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_498.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_502.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_510.php", Data: [6]float64{1, 0.00064, 0.00064, 0.00064, 0.00064, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_513.php", Data: [6]float64{2, 0.00109, 0.00109, 0.00051, 0.00058, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_546.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_553.php", Data: [6]float64{1, 0.00045, 0.00045, 0.00045, 0.00045, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_559.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_568.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_576.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_578.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_582.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_591.php", Data: [6]float64{2, 0.00093, 0.00093, 0.00044, 0.00049, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_597.php", Data: [6]float64{2, 0.00095, 0.00095, 0.00047, 0.00048, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_600.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_602.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_610.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_62.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_626.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_635.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_642.php", Data: [6]float64{1, 0.00043, 0.00043, 0.00043, 0.00043, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_651.php", Data: [6]float64{1, 0.00064, 0.00064, 0.00064, 0.00064, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_654.php", Data: [6]float64{1, 0.00061, 0.00061, 0.00061, 0.00061, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_655.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_666.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_668.php", Data: [6]float64{1, 0.00057, 0.00057, 0.00057, 0.00057, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_680.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_686.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_707.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_708.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_730.php", Data: [6]float64{1, 0.0006, 0.0006, 0.0006, 0.0006, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_736.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_738.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_74.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_744.php", Data: [6]float64{1, 0.00054, 0.00054, 0.00054, 0.00054, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_745.php", Data: [6]float64{1, 0.00042, 0.00042, 0.00042, 0.00042, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_747.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_779.php", Data: [6]float64{1, 0.00061, 0.00061, 0.00061, 0.00061, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_780.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_803.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_812.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_828.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_829.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_839.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_85.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_857.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_865.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_867.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_87.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_870.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_872.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_882.php", Data: [6]float64{1, 0.00054, 0.00054, 0.00054, 0.00054, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_889.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_902.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_905.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_915.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_918.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_937.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_939.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_943.php", Data: [6]float64{2, 0.00107, 0.00107, 0.0005, 0.00057, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_944.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_946.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_957.php", Data: [6]float64{1, 0.00063, 0.00063, 0.00063, 0.00063, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_961.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_978.php", Data: [6]float64{1, 0.0006, 0.0006, 0.0006, 0.0006, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_995.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/poptables.php", Data: [6]float64{31, 18.863939999999996, 1.7271100000000001, 0.14208, 0.90685, 13.599230000000002}, Scoped: true},
		metric{Name: "WebTransaction/Uri/myblog2/subquery.php", Data: [6]float64{27, 0.73692, 0.6234000000000001, 0.00147, 0.10433, 0.03799000000000001}, Scoped: true},
		//
		//
		// Unscoped Only Below
		//
		//
		metric{Name: "apdex tolerated", Data: [6]float64{0, 2, 0, 7, 8, 0}},
		metric{Name: "one", Data: [6]float64{1, 2, 1, 2, 2, 4}},
		metric{Name: "Supportability\\/api\\/add_custom_parameter", Data: [6]float64{1, 0.00000, 0.00000, 0.00000, 0.00000, 0.00000}},
		metric{Name: "Supportability\\/execute\\/user\\/call_count", Data: [6]float64{1, 0.00000, 0.00000, 0.00000, 0.00000, 0.00000}},
		metric{Name: "Memory\\/Physical", Data: [6]float64{1, 9.92188, 0.00000, 9.92188, 9.92188, 98.4430}},
		metric{Name: "CPU\\/User Time", Data: [6]float64{1, 0.00100, 0.00000, 0.00100, 0.00100, 0.00000}},
		metric{Name: "CPU\\/User\\/Utilization", Data: [6]float64{1, 0.00000, 0.00000, 0.00000, 0.00000, 0.00000}},
		metric{Name: "OtherTransaction\\/php\\/php\\/hello.php", Data: [6]float64{1, 1.01136, 1.01136, 1.01136, 1.01136, 1.02286}},
		metric{Name: "OtherTransactionTotalTime\\/php\\/php\\/hello.php", Data: [6]float64{1, 1.01136, 1.01136, 1.01136, 1.01136, 1.02286}},
		metric{Name: "OtherTransaction\\/all", Data: [6]float64{1, 1.01136, 1.01136, 1.01136, 1.01136, 1.02286}},
		metric{Name: "OtherTransactionTotalTime", Data: [6]float64{1, 1.01136, 1.01136, 1.01136, 1.01136, 1.02286}},
		metric{Name: "Apdex", Data: [6]float64{332, 26, 3, 0.5, 0.5, 0}},
		metric{Name: "CPU/User Time", Data: [6]float64{361, 2.9386199999999874, 0, 0, 0.06499, 0.14663000000000007}},
		metric{Name: "CPU/User/Utilization", Data: [6]float64{361, 62.965800000000016, 0, 0, 1.28535, 54.57723999999999}},
		metric{Name: "Datastore/MySQL/all", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}},
		metric{Name: "Datastore/MySQL/allWeb", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}},
		metric{Name: "Datastore/all", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}},
		metric{Name: "Datastore/allWeb", Data: [6]float64{56260, 26.704100000000004, 26.704100000000004, 2e-05, 0.22667, 0.99884}},
		metric{Name: "Datastore/operation/MySQL/insert", Data: [6]float64{53000, 23.411430000000006, 23.411430000000006, 4e-05, 0.22667, 0.6342200000000001}},
		metric{Name: "Datastore/statement/MySQL/City100/insert", Data: [6]float64{31, 0.8240199999999998, 0.8240199999999998, 8e-05, 0.20662, 0.13516}},
		metric{Name: "Datastore/statement/MySQL/City1000/insert", Data: [6]float64{31, 0.0036000000000000025, 0.0036000000000000025, 8e-05, 0.00024, 0}},
		metric{Name: "Datastore/statement/MySQL/City1001/insert", Data: [6]float64{31, 0.0077300000000000025, 0.0077300000000000025, 8e-05, 0.00419, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City1002/insert", Data: [6]float64{31, 0.029580000000000002, 0.029580000000000002, 8e-05, 0.02557, 0.00065}},
		metric{Name: "Datastore/statement/MySQL/City1003/insert", Data: [6]float64{31, 0.045639999999999986, 0.045639999999999986, 8e-05, 0.02688, 0.00093}},
		metric{Name: "Datastore/statement/MySQL/City1004/insert", Data: [6]float64{31, 0.01601, 0.01601, 8e-05, 0.01236, 0.00015}},
		metric{Name: "Datastore/statement/MySQL/City1005/insert", Data: [6]float64{31, 0.0037800000000000025, 0.0037800000000000025, 8e-05, 0.00032, 0}},
		metric{Name: "Datastore/statement/MySQL/City1006/insert", Data: [6]float64{31, 0.0038200000000000013, 0.0038200000000000013, 8e-05, 0.0003, 0}},
		metric{Name: "Datastore/statement/MySQL/City1007/insert", Data: [6]float64{31, 0.01021, 0.01021, 8e-05, 0.00634, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City1008/insert", Data: [6]float64{31, 0.03393999999999999, 0.03393999999999999, 7e-05, 0.02871, 0.00082}},
		metric{Name: "Datastore/statement/MySQL/City1009/insert", Data: [6]float64{31, 0.003740000000000002, 0.003740000000000002, 8e-05, 0.0003, 0}},
		metric{Name: "Datastore/statement/MySQL/City101/insert", Data: [6]float64{31, 0.01243, 0.01243, 8e-05, 0.00422, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City1010/insert", Data: [6]float64{31, 0.0066900000000000015, 0.0066900000000000015, 8e-05, 0.00279, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1011/insert", Data: [6]float64{31, 0.004310000000000003, 0.004310000000000003, 8e-05, 0.00041, 0}},
		metric{Name: "Datastore/statement/MySQL/City1012/insert", Data: [6]float64{31, 0.021939999999999998, 0.021939999999999998, 8e-05, 0.01815, 0.00033}},
		metric{Name: "Datastore/statement/MySQL/City1013/insert", Data: [6]float64{31, 0.01472, 0.01472, 8e-05, 0.01084, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City1014/insert", Data: [6]float64{31, 0.01627, 0.01627, 8e-05, 0.0123, 0.00015}},
		metric{Name: "Datastore/statement/MySQL/City1015/insert", Data: [6]float64{31, 0.003930000000000001, 0.003930000000000001, 8e-05, 0.00036, 0}},
		metric{Name: "Datastore/statement/MySQL/City1016/insert", Data: [6]float64{31, 0.0060500000000000016, 0.0060500000000000016, 8e-05, 0.00213, 0}},
		metric{Name: "Datastore/statement/MySQL/City1017/insert", Data: [6]float64{31, 0.026320000000000003, 0.026320000000000003, 7e-05, 0.02105, 0.00044}},
		metric{Name: "Datastore/statement/MySQL/City1018/insert", Data: [6]float64{31, 0.03601999999999999, 0.03601999999999999, 7e-05, 0.02726, 0.00075}},
		metric{Name: "Datastore/statement/MySQL/City1019/insert", Data: [6]float64{31, 0.018070000000000003, 0.018070000000000003, 8e-05, 0.01439, 0.00021}},
		metric{Name: "Datastore/statement/MySQL/City102/insert", Data: [6]float64{31, 0.01621, 0.01621, 7e-05, 0.00282, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City1020/insert", Data: [6]float64{31, 0.0071400000000000005, 0.0071400000000000005, 8e-05, 0.00162, 0}},
		metric{Name: "Datastore/statement/MySQL/City1021/insert", Data: [6]float64{31, 0.011030000000000002, 0.011030000000000002, 8e-05, 0.00717, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City1022/insert", Data: [6]float64{31, 0.02203, 0.02203, 7e-05, 0.01506, 0.00024}},
		metric{Name: "Datastore/statement/MySQL/City1023/insert", Data: [6]float64{31, 0.016520000000000003, 0.016520000000000003, 7e-05, 0.00876, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City1024/insert", Data: [6]float64{31, 0.0042600000000000025, 0.0042600000000000025, 7e-05, 0.00049, 0}},
		metric{Name: "Datastore/statement/MySQL/City1025/insert", Data: [6]float64{31, 0.006030000000000001, 0.006030000000000001, 7e-05, 0.00228, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1026/insert", Data: [6]float64{31, 0.004180000000000001, 0.004180000000000001, 8e-05, 0.00038, 0}},
		metric{Name: "Datastore/statement/MySQL/City1027/insert", Data: [6]float64{31, 0.05949999999999998, 0.05949999999999998, 7e-05, 0.03733, 0.00157}},
		metric{Name: "Datastore/statement/MySQL/City1028/insert", Data: [6]float64{31, 0.008480000000000003, 0.008480000000000003, 8e-05, 0.00289, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1029/insert", Data: [6]float64{31, 0.003920000000000001, 0.003920000000000001, 8e-05, 0.00032, 0}},
		metric{Name: "Datastore/statement/MySQL/City103/insert", Data: [6]float64{31, 0.02198, 0.02198, 7e-05, 0.00749, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City1030/insert", Data: [6]float64{31, 0.004950000000000003, 0.004950000000000003, 8e-05, 0.00082, 0}},
		metric{Name: "Datastore/statement/MySQL/City1031/insert", Data: [6]float64{31, 0.004530000000000002, 0.004530000000000002, 8e-05, 0.00077, 0}},
		metric{Name: "Datastore/statement/MySQL/City1032/insert", Data: [6]float64{31, 0.012320000000000001, 0.012320000000000001, 8e-05, 0.00821, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City1033/insert", Data: [6]float64{31, 0.008360000000000003, 0.008360000000000003, 7e-05, 0.0044, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City1034/insert", Data: [6]float64{31, 0.003880000000000002, 0.003880000000000002, 8e-05, 0.00035, 0}},
		metric{Name: "Datastore/statement/MySQL/City1035/insert", Data: [6]float64{31, 0.057259999999999964, 0.057259999999999964, 7e-05, 0.01974, 0.0009699999999999999}},
		metric{Name: "Datastore/statement/MySQL/City1036/insert", Data: [6]float64{31, 0.004550000000000001, 0.004550000000000001, 7e-05, 0.00065, 0}},
		metric{Name: "Datastore/statement/MySQL/City1037/insert", Data: [6]float64{31, 0.013670000000000002, 0.013670000000000002, 7e-05, 0.00582, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City1038/insert", Data: [6]float64{31, 0.021030000000000004, 0.021030000000000004, 8e-05, 0.0141, 0.0002}},
		metric{Name: "Datastore/statement/MySQL/City1039/insert", Data: [6]float64{31, 0.009160000000000001, 0.009160000000000001, 7e-05, 0.00482, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City104/insert", Data: [6]float64{31, 0.017650000000000006, 0.017650000000000006, 7e-05, 0.00563, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City1040/insert", Data: [6]float64{31, 0.06812999999999997, 0.06812999999999997, 8e-05, 0.04324, 0.00231}},
		metric{Name: "Datastore/statement/MySQL/City1041/insert", Data: [6]float64{31, 0.006720000000000002, 0.006720000000000002, 7e-05, 0.00329, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1042/insert", Data: [6]float64{31, 0.028050000000000002, 0.028050000000000002, 8e-05, 0.02368, 0.00056}},
		metric{Name: "Datastore/statement/MySQL/City1043/insert", Data: [6]float64{31, 0.0060500000000000016, 0.0060500000000000016, 7e-05, 0.00234, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1044/insert", Data: [6]float64{31, 0.003940000000000002, 0.003940000000000002, 8e-05, 0.00031, 0}},
		metric{Name: "Datastore/statement/MySQL/City1045/insert", Data: [6]float64{31, 0.004240000000000002, 0.004240000000000002, 8e-05, 0.00035, 0}},
		metric{Name: "Datastore/statement/MySQL/City1046/insert", Data: [6]float64{31, 0.0042200000000000015, 0.0042200000000000015, 8e-05, 0.00046, 0}},
		metric{Name: "Datastore/statement/MySQL/City1047/insert", Data: [6]float64{31, 0.004210000000000003, 0.004210000000000003, 7e-05, 0.00046, 0}},
		metric{Name: "Datastore/statement/MySQL/City1048/insert", Data: [6]float64{31, 0.024540000000000003, 0.024540000000000003, 8e-05, 0.0206, 0.00042}},
		metric{Name: "Datastore/statement/MySQL/City1049/insert", Data: [6]float64{31, 0.020670000000000004, 0.020670000000000004, 7e-05, 0.01454, 0.00022}},
		metric{Name: "Datastore/statement/MySQL/City105/insert", Data: [6]float64{31, 0.01565, 0.01565, 7e-05, 0.0056, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City1050/insert", Data: [6]float64{31, 0.008220000000000002, 0.008220000000000002, 8e-05, 0.00462, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City1051/insert", Data: [6]float64{31, 0.031020000000000002, 0.031020000000000002, 8e-05, 0.01083, 0.00027}},
		metric{Name: "Datastore/statement/MySQL/City1052/insert", Data: [6]float64{31, 0.02222, 0.02222, 8e-05, 0.01579, 0.00025}},
		metric{Name: "Datastore/statement/MySQL/City1053/insert", Data: [6]float64{31, 0.03722999999999999, 0.03722999999999999, 8e-05, 0.01795, 0.00054}},
		metric{Name: "Datastore/statement/MySQL/City1054/insert", Data: [6]float64{31, 0.0234, 0.0234, 7e-05, 0.0181, 0.00033}},
		metric{Name: "Datastore/statement/MySQL/City1055/insert", Data: [6]float64{31, 0.015080000000000001, 0.015080000000000001, 7e-05, 0.01072, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City1056/insert", Data: [6]float64{31, 0.0041100000000000025, 0.0041100000000000025, 7e-05, 0.00036, 0}},
		metric{Name: "Datastore/statement/MySQL/City1057/insert", Data: [6]float64{31, 0.01841, 0.01841, 7e-05, 0.01095, 0.00013000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City1058/insert", Data: [6]float64{31, 0.011840000000000002, 0.011840000000000002, 7e-05, 0.00403, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City1059/insert", Data: [6]float64{31, 0.004110000000000002, 0.004110000000000002, 7e-05, 0.00026, 0}},
		metric{Name: "Datastore/statement/MySQL/City106/insert", Data: [6]float64{31, 0.01449, 0.01449, 7e-05, 0.0053, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City1060/insert", Data: [6]float64{31, 0.01948, 0.01948, 8e-05, 0.01028, 0.00014000000000000001}},
		metric{Name: "Datastore/statement/MySQL/City1061/insert", Data: [6]float64{31, 0.015200000000000003, 0.015200000000000003, 8e-05, 0.00396, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City1062/insert", Data: [6]float64{31, 0.00974, 0.00974, 8e-05, 0.00524, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City1063/insert", Data: [6]float64{31, 0.0040900000000000025, 0.0040900000000000025, 7e-05, 0.00033, 0}},
		metric{Name: "Datastore/statement/MySQL/City1064/insert", Data: [6]float64{31, 0.006470000000000003, 0.006470000000000003, 7e-05, 0.00194, 0}},
		metric{Name: "Datastore/statement/MySQL/City1065/insert", Data: [6]float64{31, 0.006200000000000002, 0.006200000000000002, 7e-05, 0.00214, 0}},
		metric{Name: "Datastore/statement/MySQL/City1066/insert", Data: [6]float64{31, 0.004190000000000002, 0.004190000000000002, 7e-05, 0.00035, 0}},
		metric{Name: "Datastore/statement/MySQL/City1067/insert", Data: [6]float64{31, 0.01814, 0.01814, 7e-05, 0.01405, 0.0002}},
		metric{Name: "Datastore/statement/MySQL/City1068/insert", Data: [6]float64{31, 0.01565, 0.01565, 7e-05, 0.01139, 0.00013}},
		metric{Name: "Datastore/statement/MySQL/City1069/insert", Data: [6]float64{31, 0.0057500000000000025, 0.0057500000000000025, 7e-05, 0.00191, 0}},
		metric{Name: "Datastore/statement/MySQL/City107/insert", Data: [6]float64{31, 0.01267, 0.01267, 7e-05, 0.00386, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1070/insert", Data: [6]float64{31, 0.008950000000000003, 0.008950000000000003, 7e-05, 0.00462, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City1071/insert", Data: [6]float64{31, 0.003880000000000001, 0.003880000000000001, 7e-05, 0.00043, 0}},
		metric{Name: "Datastore/statement/MySQL/City1072/insert", Data: [6]float64{31, 0.004090000000000002, 0.004090000000000002, 7e-05, 0.0003, 0}},
		metric{Name: "Datastore/statement/MySQL/City1073/insert", Data: [6]float64{31, 0.02066, 0.02066, 7e-05, 0.01499, 0.00022}},
		metric{Name: "Datastore/statement/MySQL/City1074/insert", Data: [6]float64{31, 0.038029999999999994, 0.038029999999999994, 7e-05, 0.02756, 0.0008}},
		metric{Name: "Datastore/statement/MySQL/City1075/insert", Data: [6]float64{31, 0.018249999999999995, 0.018249999999999995, 7e-05, 0.00967, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City1076/insert", Data: [6]float64{31, 0.003860000000000001, 0.003860000000000001, 7e-05, 0.00038, 0}},
		metric{Name: "Datastore/statement/MySQL/City1077/insert", Data: [6]float64{31, 0.003570000000000002, 0.003570000000000002, 7e-05, 0.00025, 0}},
		metric{Name: "Datastore/statement/MySQL/City1078/insert", Data: [6]float64{31, 0.008020000000000001, 0.008020000000000001, 7e-05, 0.00362, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1079/insert", Data: [6]float64{31, 0.008150000000000003, 0.008150000000000003, 7e-05, 0.00448, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City108/insert", Data: [6]float64{31, 0.010989999999999998, 0.010989999999999998, 7e-05, 0.00215, 0}},
		metric{Name: "Datastore/statement/MySQL/City1080/insert", Data: [6]float64{31, 0.004220000000000001, 0.004220000000000001, 7e-05, 0.00054, 0}},
		metric{Name: "Datastore/statement/MySQL/City1081/insert", Data: [6]float64{31, 0.03114, 0.03114, 7e-05, 0.02802, 0.00078}},
		metric{Name: "Datastore/statement/MySQL/City1082/insert", Data: [6]float64{31, 0.034849999999999985, 0.034849999999999985, 8e-05, 0.01574, 0.00039000000000000005}},
		metric{Name: "Datastore/statement/MySQL/City1083/insert", Data: [6]float64{31, 0.006560000000000002, 0.006560000000000002, 7e-05, 0.00313, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1084/insert", Data: [6]float64{31, 0.016800000000000002, 0.016800000000000002, 8e-05, 0.01069, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City1085/insert", Data: [6]float64{31, 0.03347999999999999, 0.03347999999999999, 7e-05, 0.02705, 0.00074}},
		metric{Name: "Datastore/statement/MySQL/City1086/insert", Data: [6]float64{31, 0.007150000000000002, 0.007150000000000002, 7e-05, 0.00303, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City1087/insert", Data: [6]float64{31, 0.003560000000000002, 0.003560000000000002, 8e-05, 0.00026, 0}},
		metric{Name: "Datastore/statement/MySQL/City1088/insert", Data: [6]float64{31, 0.004210000000000001, 0.004210000000000001, 8e-05, 0.00057, 0}},
		metric{Name: "Datastore/statement/MySQL/City1089/insert", Data: [6]float64{31, 0.007280000000000001, 0.007280000000000001, 7e-05, 0.00263, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City109/insert", Data: [6]float64{31, 0.010090000000000002, 0.010090000000000002, 7e-05, 0.00167, 0}},
		metric{Name: "Datastore/statement/MySQL/City1090/insert", Data: [6]float64{31, 0.004620000000000003, 0.004620000000000003, 7e-05, 0.00047, 0}},
		metric{Name: "Datastore/statement/MySQL/City1091/insert", Data: [6]float64{31, 0.011540000000000007, 0.011540000000000007, 7e-05, 0.00513, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City1092/insert", Data: [6]float64{31, 0.03387999999999997, 0.03387999999999997, 8e-05, 0.02608, 0.0006900000000000001}},
		metric{Name: "Datastore/statement/MySQL/City1093/insert", Data: [6]float64{31, 0.017220000000000003, 0.017220000000000003, 7e-05, 0.01324, 0.00018}},
		metric{Name: "Datastore/statement/MySQL/City1094/insert", Data: [6]float64{31, 0.06561999999999998, 0.06561999999999998, 8e-05, 0.02835, 0.00158}},
		metric{Name: "Datastore/statement/MySQL/City110/insert", Data: [6]float64{31, 0.012260000000000004, 0.012260000000000004, 7e-05, 0.00199, 0}},
		metric{Name: "Datastore/statement/MySQL/City111/insert", Data: [6]float64{31, 0.02963, 0.02963, 7e-05, 0.0206, 0.00042}},
		metric{Name: "Datastore/statement/MySQL/City112/insert", Data: [6]float64{31, 0.009950000000000002, 0.009950000000000002, 7e-05, 0.00159, 0}},
		metric{Name: "Datastore/statement/MySQL/City113/insert", Data: [6]float64{31, 0.010130000000000002, 0.010130000000000002, 7e-05, 0.00177, 0}},
		metric{Name: "Datastore/statement/MySQL/City114/insert", Data: [6]float64{31, 0.009220000000000002, 0.009220000000000002, 7e-05, 0.00174, 0}},
		metric{Name: "Datastore/statement/MySQL/City115/insert", Data: [6]float64{31, 0.008430000000000002, 0.008430000000000002, 7e-05, 0.00106, 0}},
		metric{Name: "Datastore/statement/MySQL/City116/insert", Data: [6]float64{31, 0.011060000000000002, 0.011060000000000002, 7e-05, 0.00229, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City117/insert", Data: [6]float64{31, 0.011550000000000003, 0.011550000000000003, 7e-05, 0.00215, 0}},
		metric{Name: "Datastore/statement/MySQL/City118/insert", Data: [6]float64{31, 0.009450000000000002, 0.009450000000000002, 7e-05, 0.00118, 0}},
		metric{Name: "Datastore/statement/MySQL/City119/insert", Data: [6]float64{31, 0.008730000000000002, 0.008730000000000002, 7e-05, 0.00118, 0}},
		metric{Name: "Datastore/statement/MySQL/City120/insert", Data: [6]float64{31, 0.018600000000000005, 0.018600000000000005, 7e-05, 0.0102, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City121/insert", Data: [6]float64{31, 0.012660000000000003, 0.012660000000000003, 7e-05, 0.0047, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City122/insert", Data: [6]float64{31, 0.01788, 0.01788, 7e-05, 0.00367, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City123/insert", Data: [6]float64{31, 0.025110000000000004, 0.025110000000000004, 7e-05, 0.01601, 0.00027}},
		metric{Name: "Datastore/statement/MySQL/City124/insert", Data: [6]float64{31, 0.01037, 0.01037, 7e-05, 0.0018, 0}},
		metric{Name: "Datastore/statement/MySQL/City125/insert", Data: [6]float64{31, 0.007809999999999999, 0.007809999999999999, 7e-05, 0.00104, 0}},
		metric{Name: "Datastore/statement/MySQL/City126/insert", Data: [6]float64{31, 0.009050000000000002, 0.009050000000000002, 7e-05, 0.001, 0}},
		metric{Name: "Datastore/statement/MySQL/City127/insert", Data: [6]float64{31, 0.018989999999999996, 0.018989999999999996, 7e-05, 0.01073, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City128/insert", Data: [6]float64{31, 0.008300000000000004, 0.008300000000000004, 7e-05, 0.00143, 0}},
		metric{Name: "Datastore/statement/MySQL/City129/insert", Data: [6]float64{31, 0.008920000000000004, 0.008920000000000004, 7e-05, 0.00153, 0}},
		metric{Name: "Datastore/statement/MySQL/City130/insert", Data: [6]float64{31, 0.009450000000000002, 0.009450000000000002, 7e-05, 0.00165, 0}},
		metric{Name: "Datastore/statement/MySQL/City131/insert", Data: [6]float64{31, 0.0151, 0.0151, 7e-05, 0.00755, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City132/insert", Data: [6]float64{31, 0.0071200000000000005, 0.0071200000000000005, 7e-05, 0.00098, 0}},
		metric{Name: "Datastore/statement/MySQL/City133/insert", Data: [6]float64{31, 0.02159, 0.02159, 7e-05, 0.01466, 0.00021}},
		metric{Name: "Datastore/statement/MySQL/City134/insert", Data: [6]float64{31, 0.00693, 0.00693, 7e-05, 0.00097, 0}},
		metric{Name: "Datastore/statement/MySQL/City135/insert", Data: [6]float64{31, 0.01277, 0.01277, 7e-05, 0.00573, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City136/insert", Data: [6]float64{31, 0.008560000000000002, 0.008560000000000002, 7e-05, 0.00112, 0}},
		metric{Name: "Datastore/statement/MySQL/City137/insert", Data: [6]float64{31, 0.009430000000000001, 0.009430000000000001, 7e-05, 0.00298, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City138/insert", Data: [6]float64{31, 0.01058, 0.01058, 7e-05, 0.00319, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City139/insert", Data: [6]float64{31, 0.009880000000000002, 0.009880000000000002, 7e-05, 0.00274, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City140/insert", Data: [6]float64{31, 0.015870000000000002, 0.015870000000000002, 7e-05, 0.00774, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City141/insert", Data: [6]float64{31, 0.03871, 0.03871, 7e-05, 0.02521, 0.0006600000000000001}},
		metric{Name: "Datastore/statement/MySQL/City142/insert", Data: [6]float64{31, 0.00794, 0.00794, 7e-05, 0.00101, 0}},
		metric{Name: "Datastore/statement/MySQL/City143/insert", Data: [6]float64{31, 0.01088, 0.01088, 7e-05, 0.00334, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City144/insert", Data: [6]float64{31, 0.007349999999999998, 0.007349999999999998, 7e-05, 0.00106, 0}},
		metric{Name: "Datastore/statement/MySQL/City145/insert", Data: [6]float64{31, 0.008159999999999999, 0.008159999999999999, 7e-05, 0.0012, 0}},
		metric{Name: "Datastore/statement/MySQL/City146/insert", Data: [6]float64{31, 0.029589999999999998, 0.029589999999999998, 7e-05, 0.01876, 0.00035}},
		metric{Name: "Datastore/statement/MySQL/City147/insert", Data: [6]float64{31, 0.01404, 0.01404, 7e-05, 0.00647, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City148/insert", Data: [6]float64{31, 0.02865, 0.02865, 7e-05, 0.0103, 0.00017}},
		metric{Name: "Datastore/statement/MySQL/City149/insert", Data: [6]float64{31, 0.03610000000000001, 0.03610000000000001, 7e-05, 0.02732, 0.00075}},
		metric{Name: "Datastore/statement/MySQL/City150/insert", Data: [6]float64{31, 0.007470000000000001, 0.007470000000000001, 7e-05, 0.00096, 0}},
		metric{Name: "Datastore/statement/MySQL/City151/insert", Data: [6]float64{31, 0.013230000000000006, 0.013230000000000006, 7e-05, 0.00458, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City152/insert", Data: [6]float64{31, 0.011340000000000003, 0.011340000000000003, 7e-05, 0.00296, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City153/insert", Data: [6]float64{31, 0.03273, 0.03273, 7e-05, 0.01266, 0.00023}},
		metric{Name: "Datastore/statement/MySQL/City154/insert", Data: [6]float64{31, 0.009830000000000002, 0.009830000000000002, 7e-05, 0.0027, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City155/insert", Data: [6]float64{31, 0.008780000000000001, 0.008780000000000001, 7e-05, 0.00114, 0}},
		metric{Name: "Datastore/statement/MySQL/City156/insert", Data: [6]float64{31, 0.03438999999999999, 0.03438999999999999, 7e-05, 0.02226, 0.00051}},
		metric{Name: "Datastore/statement/MySQL/City157/insert", Data: [6]float64{31, 0.008300000000000002, 0.008300000000000002, 7e-05, 0.001, 0}},
		metric{Name: "Datastore/statement/MySQL/City158/insert", Data: [6]float64{31, 0.008190000000000003, 0.008190000000000003, 7e-05, 0.00112, 0}},
		metric{Name: "Datastore/statement/MySQL/City159/insert", Data: [6]float64{31, 0.00831, 0.00831, 7e-05, 0.00112, 0}},
		metric{Name: "Datastore/statement/MySQL/City160/insert", Data: [6]float64{31, 0.027860000000000003, 0.027860000000000003, 7e-05, 0.01951, 0.00038}},
		metric{Name: "Datastore/statement/MySQL/City161/insert", Data: [6]float64{31, 0.017960000000000004, 0.017960000000000004, 7e-05, 0.01012, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City162/insert", Data: [6]float64{31, 0.007709999999999999, 0.007709999999999999, 7e-05, 0.00114, 0}},
		metric{Name: "Datastore/statement/MySQL/City163/insert", Data: [6]float64{31, 0.009510000000000005, 0.009510000000000005, 7e-05, 0.00109, 0}},
		metric{Name: "Datastore/statement/MySQL/City164/insert", Data: [6]float64{31, 0.010100000000000005, 0.010100000000000005, 7e-05, 0.00141, 0}},
		metric{Name: "Datastore/statement/MySQL/City165/insert", Data: [6]float64{31, 0.00834, 0.00834, 7e-05, 0.00097, 0}},
		metric{Name: "Datastore/statement/MySQL/City166/insert", Data: [6]float64{31, 0.014629999999999999, 0.014629999999999999, 7e-05, 0.00365, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City167/insert", Data: [6]float64{31, 0.019500000000000003, 0.019500000000000003, 7e-05, 0.01049, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City168/insert", Data: [6]float64{31, 0.00741, 0.00741, 7e-05, 0.0009, 0}},
		metric{Name: "Datastore/statement/MySQL/City169/insert", Data: [6]float64{31, 0.02917, 0.02917, 7e-05, 0.01622, 0.00027}},
		metric{Name: "Datastore/statement/MySQL/City170/insert", Data: [6]float64{31, 0.030470000000000004, 0.030470000000000004, 7e-05, 0.01261, 0.00022}},
		metric{Name: "Datastore/statement/MySQL/City171/insert", Data: [6]float64{31, 0.007549999999999999, 0.007549999999999999, 7e-05, 0.00113, 0}},
		metric{Name: "Datastore/statement/MySQL/City172/insert", Data: [6]float64{31, 0.008220000000000002, 0.008220000000000002, 7e-05, 0.00116, 0}},
		metric{Name: "Datastore/statement/MySQL/City173/insert", Data: [6]float64{31, 0.010980000000000002, 0.010980000000000002, 7e-05, 0.00255, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City174/insert", Data: [6]float64{31, 0.01675, 0.01675, 7e-05, 0.00699, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City175/insert", Data: [6]float64{31, 0.014010000000000002, 0.014010000000000002, 7e-05, 0.00312, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City176/insert", Data: [6]float64{31, 0.011140000000000006, 0.011140000000000006, 7e-05, 0.00305, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City177/insert", Data: [6]float64{31, 0.016810000000000002, 0.016810000000000002, 7e-05, 0.00768, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City178/insert", Data: [6]float64{31, 0.007389999999999999, 0.007389999999999999, 7e-05, 0.00115, 0}},
		metric{Name: "Datastore/statement/MySQL/City179/insert", Data: [6]float64{31, 0.014900000000000004, 0.014900000000000004, 7e-05, 0.00474, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City180/insert", Data: [6]float64{31, 0.013340000000000001, 0.013340000000000001, 7e-05, 0.00477, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City181/insert", Data: [6]float64{31, 0.010290000000000004, 0.010290000000000004, 7e-05, 0.00175, 0}},
		metric{Name: "Datastore/statement/MySQL/City182/insert", Data: [6]float64{31, 0.008009999999999998, 0.008009999999999998, 7e-05, 0.00118, 0}},
		metric{Name: "Datastore/statement/MySQL/City183/insert", Data: [6]float64{31, 0.00873, 0.00873, 7e-05, 0.00126, 0}},
		metric{Name: "Datastore/statement/MySQL/City184/insert", Data: [6]float64{31, 0.009210000000000003, 0.009210000000000003, 7e-05, 0.00172, 0}},
		metric{Name: "Datastore/statement/MySQL/City185/insert", Data: [6]float64{31, 0.00883, 0.00883, 7e-05, 0.0012, 0}},
		metric{Name: "Datastore/statement/MySQL/City186/insert", Data: [6]float64{31, 0.01839, 0.01839, 7e-05, 0.00929, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City187/insert", Data: [6]float64{31, 0.009420000000000001, 0.009420000000000001, 7e-05, 0.00134, 0}},
		metric{Name: "Datastore/statement/MySQL/City188/insert", Data: [6]float64{31, 0.049100000000000005, 0.049100000000000005, 7e-05, 0.02553, 0.00081}},
		metric{Name: "Datastore/statement/MySQL/City189/insert", Data: [6]float64{31, 0.018189999999999998, 0.018189999999999998, 7e-05, 0.00516, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City190/insert", Data: [6]float64{31, 0.010490000000000001, 0.010490000000000001, 7e-05, 0.00239, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City191/insert", Data: [6]float64{31, 0.011730000000000003, 0.011730000000000003, 7e-05, 0.00222, 0}},
		metric{Name: "Datastore/statement/MySQL/City192/insert", Data: [6]float64{31, 0.008400000000000001, 0.008400000000000001, 7e-05, 0.00136, 0}},
		metric{Name: "Datastore/statement/MySQL/City193/insert", Data: [6]float64{31, 0.022780000000000005, 0.022780000000000005, 7e-05, 0.01209, 0.00015999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City194/insert", Data: [6]float64{31, 0.020200000000000003, 0.020200000000000003, 7e-05, 0.01261, 0.00016}},
		metric{Name: "Datastore/statement/MySQL/City195/insert", Data: [6]float64{31, 0.011749999999999998, 0.011749999999999998, 7e-05, 0.00218, 0}},
		metric{Name: "Datastore/statement/MySQL/City196/insert", Data: [6]float64{31, 0.008079999999999999, 0.008079999999999999, 7e-05, 0.00139, 0}},
		metric{Name: "Datastore/statement/MySQL/City197/insert", Data: [6]float64{31, 0.016300000000000005, 0.016300000000000005, 7e-05, 0.00811, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City198/insert", Data: [6]float64{31, 0.010110000000000003, 0.010110000000000003, 7e-05, 0.00192, 0}},
		metric{Name: "Datastore/statement/MySQL/City199/insert", Data: [6]float64{31, 0.008100000000000001, 0.008100000000000001, 7e-05, 0.00112, 0}},
		metric{Name: "Datastore/statement/MySQL/City200/insert", Data: [6]float64{31, 0.007749999999999998, 0.007749999999999998, 7e-05, 0.0012, 0}},
		metric{Name: "Datastore/statement/MySQL/City201/insert", Data: [6]float64{31, 0.01495, 0.01495, 7e-05, 0.00524, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City202/insert", Data: [6]float64{31, 0.009440000000000002, 0.009440000000000002, 7e-05, 0.00188, 0}},
		metric{Name: "Datastore/statement/MySQL/City203/insert", Data: [6]float64{31, 0.007829999999999998, 0.007829999999999998, 7e-05, 0.00105, 0}},
		metric{Name: "Datastore/statement/MySQL/City204/insert", Data: [6]float64{31, 0.02313, 0.02313, 7e-05, 0.00833, 0.00010999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City205/insert", Data: [6]float64{31, 0.03678999999999999, 0.03678999999999999, 8e-05, 0.02367, 0.0006}},
		metric{Name: "Datastore/statement/MySQL/City206/insert", Data: [6]float64{31, 0.011290000000000001, 0.011290000000000001, 7e-05, 0.00143, 0}},
		metric{Name: "Datastore/statement/MySQL/City207/insert", Data: [6]float64{31, 0.027540000000000005, 0.027540000000000005, 7e-05, 0.01078, 0.00016}},
		metric{Name: "Datastore/statement/MySQL/City208/insert", Data: [6]float64{31, 0.010100000000000001, 0.010100000000000001, 7e-05, 0.00366, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City209/insert", Data: [6]float64{31, 0.0129, 0.0129, 7e-05, 0.00619, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City210/insert", Data: [6]float64{31, 0.009030000000000002, 0.009030000000000002, 7e-05, 0.00208, 0}},
		metric{Name: "Datastore/statement/MySQL/City211/insert", Data: [6]float64{31, 0.02022, 0.02022, 7e-05, 0.00759, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City212/insert", Data: [6]float64{31, 0.03062, 0.03062, 7e-05, 0.01474, 0.00028000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City213/insert", Data: [6]float64{31, 0.016700000000000003, 0.016700000000000003, 7e-05, 0.00611, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City214/insert", Data: [6]float64{31, 0.029880000000000004, 0.029880000000000004, 7e-05, 0.0146, 0.00025}},
		metric{Name: "Datastore/statement/MySQL/City215/insert", Data: [6]float64{31, 0.0070399999999999985, 0.0070399999999999985, 7e-05, 0.00079, 0}},
		metric{Name: "Datastore/statement/MySQL/City216/insert", Data: [6]float64{31, 0.00721, 0.00721, 7e-05, 0.00091, 0}},
		metric{Name: "Datastore/statement/MySQL/City217/insert", Data: [6]float64{31, 0.06302, 0.06302, 7e-05, 0.02977, 0.0015899999999999998}},
		metric{Name: "Datastore/statement/MySQL/City218/insert", Data: [6]float64{31, 0.00902, 0.00902, 7e-05, 0.00202, 0}},
		metric{Name: "Datastore/statement/MySQL/City219/insert", Data: [6]float64{31, 0.01595, 0.01595, 7e-05, 0.00607, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City220/insert", Data: [6]float64{31, 0.01633, 0.01633, 7e-05, 0.006, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City221/insert", Data: [6]float64{31, 0.01559, 0.01559, 7e-05, 0.00507, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City222/insert", Data: [6]float64{31, 0.025089999999999998, 0.025089999999999998, 7e-05, 0.01654, 0.00028000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City223/insert", Data: [6]float64{31, 0.01391, 0.01391, 7e-05, 0.00496, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City224/insert", Data: [6]float64{31, 0.00767, 0.00767, 7e-05, 0.00125, 0}},
		metric{Name: "Datastore/statement/MySQL/City225/insert", Data: [6]float64{31, 0.026690000000000002, 0.026690000000000002, 7e-05, 0.01222, 0.00016999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City226/insert", Data: [6]float64{31, 0.013000000000000001, 0.013000000000000001, 7e-05, 0.00598, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City227/insert", Data: [6]float64{31, 0.014800000000000004, 0.014800000000000004, 7e-05, 0.00748, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City228/insert", Data: [6]float64{31, 0.012070000000000003, 0.012070000000000003, 7e-05, 0.00348, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City229/insert", Data: [6]float64{31, 0.01446, 0.01446, 7e-05, 0.0077, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City230/insert", Data: [6]float64{31, 0.007260000000000001, 0.007260000000000001, 7e-05, 0.00126, 0}},
		metric{Name: "Datastore/statement/MySQL/City231/insert", Data: [6]float64{31, 0.010270000000000003, 0.010270000000000003, 7e-05, 0.00291, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City232/insert", Data: [6]float64{31, 0.03822999999999999, 0.03822999999999999, 7e-05, 0.0131, 0.00027}},
		metric{Name: "Datastore/statement/MySQL/City233/insert", Data: [6]float64{31, 0.05641000000000001, 0.05641000000000001, 7e-05, 0.02542, 0.00091}},
		metric{Name: "Datastore/statement/MySQL/City234/insert", Data: [6]float64{31, 0.016120000000000006, 0.016120000000000006, 7e-05, 0.00568, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City235/insert", Data: [6]float64{31, 0.018850000000000002, 0.018850000000000002, 7e-05, 0.01195, 0.00014}},
		metric{Name: "Datastore/statement/MySQL/City236/insert", Data: [6]float64{31, 0.011400000000000004, 0.011400000000000004, 7e-05, 0.00419, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City237/insert", Data: [6]float64{31, 0.015470000000000005, 0.015470000000000005, 7e-05, 0.00535, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City238/insert", Data: [6]float64{31, 0.011910000000000002, 0.011910000000000002, 7e-05, 0.00462, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City239/insert", Data: [6]float64{31, 0.008280000000000001, 0.008280000000000001, 7e-05, 0.00203, 0}},
		metric{Name: "Datastore/statement/MySQL/City240/insert", Data: [6]float64{31, 0.013030000000000002, 0.013030000000000002, 7e-05, 0.00642, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City241/insert", Data: [6]float64{31, 0.01922, 0.01922, 7e-05, 0.00862, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City242/insert", Data: [6]float64{31, 0.019830000000000004, 0.019830000000000004, 7e-05, 0.00871, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City243/insert", Data: [6]float64{31, 0.010330000000000004, 0.010330000000000004, 7e-05, 0.00331, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City244/insert", Data: [6]float64{31, 0.01285, 0.01285, 7e-05, 0.0066, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City245/insert", Data: [6]float64{31, 0.018540000000000004, 0.018540000000000004, 7e-05, 0.01001, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City246/insert", Data: [6]float64{31, 0.012520000000000003, 0.012520000000000003, 7e-05, 0.00326, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City247/insert", Data: [6]float64{31, 0.01553, 0.01553, 7e-05, 0.00779, 7.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City248/insert", Data: [6]float64{31, 0.012100000000000001, 0.012100000000000001, 7e-05, 0.00305, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City249/insert", Data: [6]float64{31, 0.014920000000000001, 0.014920000000000001, 7e-05, 0.00394, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City250/insert", Data: [6]float64{31, 0.020370000000000003, 0.020370000000000003, 7e-05, 0.01362, 0.00019}},
		metric{Name: "Datastore/statement/MySQL/City251/insert", Data: [6]float64{31, 0.01205, 0.01205, 7e-05, 0.00634, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City252/insert", Data: [6]float64{31, 0.014250000000000004, 0.014250000000000004, 7e-05, 0.00648, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City253/insert", Data: [6]float64{31, 0.0066999999999999985, 0.0066999999999999985, 7e-05, 0.0011, 0}},
		metric{Name: "Datastore/statement/MySQL/City254/insert", Data: [6]float64{31, 0.008389999999999996, 0.008389999999999996, 7e-05, 0.00183, 0}},
		metric{Name: "Datastore/statement/MySQL/City255/insert", Data: [6]float64{31, 0.045970000000000004, 0.045970000000000004, 7e-05, 0.03152, 0.00102}},
		metric{Name: "Datastore/statement/MySQL/City256/insert", Data: [6]float64{31, 0.00842, 0.00842, 7e-05, 0.00106, 0}},
		metric{Name: "Datastore/statement/MySQL/City257/insert", Data: [6]float64{31, 0.016260000000000004, 0.016260000000000004, 7e-05, 0.00722, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City258/insert", Data: [6]float64{31, 0.01666, 0.01666, 7e-05, 0.0065, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City259/insert", Data: [6]float64{31, 0.01003, 0.01003, 7e-05, 0.00135, 0}},
		metric{Name: "Datastore/statement/MySQL/City260/insert", Data: [6]float64{31, 0.015020000000000002, 0.015020000000000002, 7e-05, 0.00432, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City261/insert", Data: [6]float64{31, 0.021290000000000003, 0.021290000000000003, 7e-05, 0.00981, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City262/insert", Data: [6]float64{31, 0.03193999999999999, 0.03193999999999999, 7e-05, 0.01951, 0.00039000000000000005}},
		metric{Name: "Datastore/statement/MySQL/City263/insert", Data: [6]float64{31, 0.009289999999999998, 0.009289999999999998, 7e-05, 0.00114, 0}},
		metric{Name: "Datastore/statement/MySQL/City264/insert", Data: [6]float64{31, 0.00847, 0.00847, 7e-05, 0.00114, 0}},
		metric{Name: "Datastore/statement/MySQL/City265/insert", Data: [6]float64{31, 0.013260000000000001, 0.013260000000000001, 7e-05, 0.00496, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City266/insert", Data: [6]float64{31, 0.014320000000000005, 0.014320000000000005, 7e-05, 0.0059, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City267/insert", Data: [6]float64{31, 0.017740000000000002, 0.017740000000000002, 7e-05, 0.00995, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City268/insert", Data: [6]float64{31, 0.009570000000000002, 0.009570000000000002, 7e-05, 0.00129, 0}},
		metric{Name: "Datastore/statement/MySQL/City269/insert", Data: [6]float64{31, 0.023300000000000005, 0.023300000000000005, 7e-05, 0.01071, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City270/insert", Data: [6]float64{31, 0.009450000000000002, 0.009450000000000002, 7e-05, 0.00115, 0}},
		metric{Name: "Datastore/statement/MySQL/City271/insert", Data: [6]float64{31, 0.011030000000000002, 0.011030000000000002, 7e-05, 0.00165, 0}},
		metric{Name: "Datastore/statement/MySQL/City272/insert", Data: [6]float64{31, 0.019690000000000003, 0.019690000000000003, 7e-05, 0.00848, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City273/insert", Data: [6]float64{31, 0.00957, 0.00957, 7e-05, 0.00159, 0}},
		metric{Name: "Datastore/statement/MySQL/City274/insert", Data: [6]float64{31, 0.03062, 0.03062, 7e-05, 0.01655, 0.00029}},
		metric{Name: "Datastore/statement/MySQL/City275/insert", Data: [6]float64{31, 0.019370000000000002, 0.019370000000000002, 7e-05, 0.00568, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City276/insert", Data: [6]float64{31, 0.009880000000000003, 0.009880000000000003, 7e-05, 0.00122, 0}},
		metric{Name: "Datastore/statement/MySQL/City277/insert", Data: [6]float64{31, 0.029029999999999997, 0.029029999999999997, 7e-05, 0.01697, 0.00029}},
		metric{Name: "Datastore/statement/MySQL/City278/insert", Data: [6]float64{31, 0.05147000000000001, 0.05147000000000001, 7e-05, 0.03355, 0.00116}},
		metric{Name: "Datastore/statement/MySQL/City279/insert", Data: [6]float64{31, 0.01823, 0.01823, 7e-05, 0.01047, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City280/insert", Data: [6]float64{31, 0.02148, 0.02148, 7e-05, 0.00463, 5.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City281/insert", Data: [6]float64{31, 0.01794, 0.01794, 7e-05, 0.00324, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City282/insert", Data: [6]float64{31, 0.035149999999999994, 0.035149999999999994, 7e-05, 0.02539, 0.00064}},
		metric{Name: "Datastore/statement/MySQL/City283/insert", Data: [6]float64{31, 0.009790000000000005, 0.009790000000000005, 7e-05, 0.00229, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City284/insert", Data: [6]float64{31, 0.013920000000000002, 0.013920000000000002, 7e-05, 0.00261, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City285/insert", Data: [6]float64{31, 0.00866, 0.00866, 7e-05, 0.00125, 0}},
		metric{Name: "Datastore/statement/MySQL/City286/insert", Data: [6]float64{31, 0.028640000000000002, 0.028640000000000002, 7e-05, 0.01268, 0.00019}},
		metric{Name: "Datastore/statement/MySQL/City287/insert", Data: [6]float64{31, 0.01781, 0.01781, 7e-05, 0.00626, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City288/insert", Data: [6]float64{31, 0.013090000000000001, 0.013090000000000001, 7e-05, 0.00263, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City289/insert", Data: [6]float64{31, 0.010740000000000003, 0.010740000000000003, 7e-05, 0.00201, 0}},
		metric{Name: "Datastore/statement/MySQL/City290/insert", Data: [6]float64{31, 0.02665, 0.02665, 7e-05, 0.01012, 0.00013000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City291/insert", Data: [6]float64{31, 0.01648, 0.01648, 7e-05, 0.00415, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City292/insert", Data: [6]float64{31, 0.008150000000000003, 0.008150000000000003, 7e-05, 0.00098, 0}},
		metric{Name: "Datastore/statement/MySQL/City293/insert", Data: [6]float64{31, 0.010520000000000002, 0.010520000000000002, 7e-05, 0.00168, 0}},
		metric{Name: "Datastore/statement/MySQL/City294/insert", Data: [6]float64{31, 0.010620000000000001, 0.010620000000000001, 7e-05, 0.00264, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City295/insert", Data: [6]float64{31, 0.01679, 0.01679, 7e-05, 0.00751, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City296/insert", Data: [6]float64{31, 0.05086999999999999, 0.05086999999999999, 7e-05, 0.03414, 0.00122}},
		metric{Name: "Datastore/statement/MySQL/City297/insert", Data: [6]float64{31, 0.22410999999999995, 0.22410999999999995, 7e-05, 0.21356, 0.04561}},
		metric{Name: "Datastore/statement/MySQL/City298/insert", Data: [6]float64{31, 0.01573, 0.01573, 7e-05, 0.00289, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City299/insert", Data: [6]float64{31, 0.016959999999999996, 0.016959999999999996, 7e-05, 0.00596, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City300/insert", Data: [6]float64{31, 0.019330000000000003, 0.019330000000000003, 7e-05, 0.00995, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City301/insert", Data: [6]float64{31, 0.009999999999999998, 0.009999999999999998, 7e-05, 0.00172, 0}},
		metric{Name: "Datastore/statement/MySQL/City302/insert", Data: [6]float64{31, 0.014710000000000003, 0.014710000000000003, 7e-05, 0.00746, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City303/insert", Data: [6]float64{31, 0.020050000000000002, 0.020050000000000002, 7e-05, 0.01086, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City304/insert", Data: [6]float64{31, 0.01862, 0.01862, 7e-05, 0.00463, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City305/insert", Data: [6]float64{31, 0.011789999999999998, 0.011789999999999998, 7e-05, 0.00212, 0}},
		metric{Name: "Datastore/statement/MySQL/City306/insert", Data: [6]float64{31, 0.016360000000000003, 0.016360000000000003, 7e-05, 0.00411, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City307/insert", Data: [6]float64{31, 0.020040000000000006, 0.020040000000000006, 7e-05, 0.0073, 7.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City308/insert", Data: [6]float64{31, 0.023690000000000003, 0.023690000000000003, 7e-05, 0.00783, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City309/insert", Data: [6]float64{31, 0.010940000000000004, 0.010940000000000004, 7e-05, 0.00177, 0}},
		metric{Name: "Datastore/statement/MySQL/City310/insert", Data: [6]float64{31, 0.021460000000000003, 0.021460000000000003, 7e-05, 0.00415, 6.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City311/insert", Data: [6]float64{31, 0.010560000000000002, 0.010560000000000002, 7e-05, 0.00117, 0}},
		metric{Name: "Datastore/statement/MySQL/City312/insert", Data: [6]float64{31, 0.028279999999999996, 0.028279999999999996, 7e-05, 0.01951, 0.00038}},
		metric{Name: "Datastore/statement/MySQL/City313/insert", Data: [6]float64{31, 0.0159, 0.0159, 7e-05, 0.00344, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City314/insert", Data: [6]float64{31, 0.021679999999999998, 0.021679999999999998, 7e-05, 0.00493, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City315/insert", Data: [6]float64{31, 0.017360000000000004, 0.017360000000000004, 7e-05, 0.0029, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City316/insert", Data: [6]float64{31, 0.014260000000000002, 0.014260000000000002, 7e-05, 0.00493, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City317/insert", Data: [6]float64{31, 0.060980000000000006, 0.060980000000000006, 7e-05, 0.02032, 0.00083}},
		metric{Name: "Datastore/statement/MySQL/City318/insert", Data: [6]float64{31, 0.02583, 0.02583, 7e-05, 0.01124, 0.00015999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City319/insert", Data: [6]float64{31, 0.02702, 0.02702, 7e-05, 0.00941, 0.00013000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City320/insert", Data: [6]float64{31, 0.01776, 0.01776, 7e-05, 0.00433, 5.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City321/insert", Data: [6]float64{31, 0.018560000000000004, 0.018560000000000004, 7e-05, 0.00287, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City322/insert", Data: [6]float64{31, 0.015410000000000005, 0.015410000000000005, 7e-05, 0.00407, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City323/insert", Data: [6]float64{31, 0.014910000000000001, 0.014910000000000001, 7e-05, 0.00558, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City324/insert", Data: [6]float64{31, 0.016550000000000002, 0.016550000000000002, 7e-05, 0.00699, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City325/insert", Data: [6]float64{31, 0.00944, 0.00944, 7e-05, 0.00234, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City326/insert", Data: [6]float64{31, 0.028180000000000004, 0.028180000000000004, 7e-05, 0.0152, 0.00026000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City327/insert", Data: [6]float64{31, 0.010390000000000005, 0.010390000000000005, 7e-05, 0.00415, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City328/insert", Data: [6]float64{31, 0.02093, 0.02093, 7e-05, 0.01237, 0.00015999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City329/insert", Data: [6]float64{31, 0.008139999999999998, 0.008139999999999998, 7e-05, 0.00159, 0}},
		metric{Name: "Datastore/statement/MySQL/City330/insert", Data: [6]float64{31, 0.015300000000000005, 0.015300000000000005, 7e-05, 0.00437, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City331/insert", Data: [6]float64{31, 0.02082, 0.02082, 7e-05, 0.00666, 7.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City332/insert", Data: [6]float64{31, 0.01007, 0.01007, 7e-05, 0.00273, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City333/insert", Data: [6]float64{31, 0.017980000000000006, 0.017980000000000006, 7e-05, 0.00824, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City334/insert", Data: [6]float64{31, 0.015210000000000003, 0.015210000000000003, 7e-05, 0.00559, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City335/insert", Data: [6]float64{31, 0.01835, 0.01835, 7e-05, 0.00459, 5.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City336/insert", Data: [6]float64{31, 0.019670000000000003, 0.019670000000000003, 7e-05, 0.00779, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City337/insert", Data: [6]float64{31, 0.013210000000000003, 0.013210000000000003, 7e-05, 0.00621, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City338/insert", Data: [6]float64{31, 0.0068200000000000005, 0.0068200000000000005, 7e-05, 0.00069, 0}},
		metric{Name: "Datastore/statement/MySQL/City339/insert", Data: [6]float64{31, 0.013279999999999998, 0.013279999999999998, 7e-05, 0.00359, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City340/insert", Data: [6]float64{31, 0.00822, 0.00822, 7e-05, 0.00158, 0}},
		metric{Name: "Datastore/statement/MySQL/City341/insert", Data: [6]float64{31, 0.00734, 0.00734, 7e-05, 0.0012, 0}},
		metric{Name: "Datastore/statement/MySQL/City342/insert", Data: [6]float64{31, 0.012150000000000001, 0.012150000000000001, 7e-05, 0.00332, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City343/insert", Data: [6]float64{31, 0.01155, 0.01155, 7e-05, 0.00295, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City344/insert", Data: [6]float64{31, 0.02581, 0.02581, 7e-05, 0.0067, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City345/insert", Data: [6]float64{31, 0.012080000000000002, 0.012080000000000002, 7e-05, 0.00273, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City346/insert", Data: [6]float64{31, 0.006810000000000001, 0.006810000000000001, 7e-05, 0.00065, 0}},
		metric{Name: "Datastore/statement/MySQL/City347/insert", Data: [6]float64{31, 0.012360000000000003, 0.012360000000000003, 7e-05, 0.00419, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City348/insert", Data: [6]float64{31, 0.011840000000000002, 0.011840000000000002, 7e-05, 0.00549, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City349/insert", Data: [6]float64{31, 0.04577, 0.04577, 7e-05, 0.03398, 0.00117}},
		metric{Name: "Datastore/statement/MySQL/City350/insert", Data: [6]float64{31, 0.016530000000000003, 0.016530000000000003, 7e-05, 0.00666, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City351/insert", Data: [6]float64{31, 0.015350000000000004, 0.015350000000000004, 7e-05, 0.00479, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City352/insert", Data: [6]float64{31, 0.013859999999999999, 0.013859999999999999, 7e-05, 0.00278, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City353/insert", Data: [6]float64{31, 0.00987, 0.00987, 7e-05, 0.00414, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City354/insert", Data: [6]float64{31, 0.0214, 0.0214, 7e-05, 0.00771, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City355/insert", Data: [6]float64{31, 0.03605, 0.03605, 7e-05, 0.01311, 0.00027000000000000006}},
		metric{Name: "Datastore/statement/MySQL/City356/insert", Data: [6]float64{31, 0.01933, 0.01933, 7e-05, 0.00684, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City357/insert", Data: [6]float64{31, 0.04825, 0.04825, 7e-05, 0.02964, 0.0009200000000000001}},
		metric{Name: "Datastore/statement/MySQL/City358/insert", Data: [6]float64{31, 0.011070000000000002, 0.011070000000000002, 7e-05, 0.00289, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City359/insert", Data: [6]float64{31, 0.0211, 0.0211, 7e-05, 0.01115, 0.00013000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City360/insert", Data: [6]float64{31, 0.01143, 0.01143, 7e-05, 0.00246, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City361/insert", Data: [6]float64{31, 0.0204, 0.0204, 7e-05, 0.01138, 0.00013}},
		metric{Name: "Datastore/statement/MySQL/City362/insert", Data: [6]float64{31, 0.010039999999999999, 0.010039999999999999, 7e-05, 0.00198, 0}},
		metric{Name: "Datastore/statement/MySQL/City363/insert", Data: [6]float64{31, 0.011260000000000003, 0.011260000000000003, 7e-05, 0.00367, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City364/insert", Data: [6]float64{31, 0.03483000000000001, 0.03483000000000001, 7e-05, 0.01325, 0.00029000000000000006}},
		metric{Name: "Datastore/statement/MySQL/City365/insert", Data: [6]float64{31, 0.02536999999999999, 0.02536999999999999, 7e-05, 0.01637, 0.00028000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City366/insert", Data: [6]float64{31, 0.00847, 0.00847, 7e-05, 0.00129, 0}},
		metric{Name: "Datastore/statement/MySQL/City367/insert", Data: [6]float64{31, 0.01531, 0.01531, 7e-05, 0.00548, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City368/insert", Data: [6]float64{31, 0.010140000000000001, 0.010140000000000001, 7e-05, 0.00268, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City369/insert", Data: [6]float64{31, 0.013990000000000002, 0.013990000000000002, 7e-05, 0.00339, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City370/insert", Data: [6]float64{31, 0.012680000000000002, 0.012680000000000002, 7e-05, 0.00406, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City371/insert", Data: [6]float64{31, 0.00921, 0.00921, 7e-05, 0.00167, 0}},
		metric{Name: "Datastore/statement/MySQL/City372/insert", Data: [6]float64{31, 0.0086, 0.0086, 7e-05, 0.00196, 0}},
		metric{Name: "Datastore/statement/MySQL/City373/insert", Data: [6]float64{31, 0.012490000000000001, 0.012490000000000001, 7e-05, 0.00471, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City374/insert", Data: [6]float64{31, 0.015630000000000005, 0.015630000000000005, 8e-05, 0.00715, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City375/insert", Data: [6]float64{31, 0.01593, 0.01593, 7e-05, 0.00457, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City376/insert", Data: [6]float64{31, 0.018250000000000002, 0.018250000000000002, 7e-05, 0.0069, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City377/insert", Data: [6]float64{31, 0.010930000000000002, 0.010930000000000002, 7e-05, 0.00181, 0}},
		metric{Name: "Datastore/statement/MySQL/City378/insert", Data: [6]float64{31, 0.01109, 0.01109, 7e-05, 0.00199, 0}},
		metric{Name: "Datastore/statement/MySQL/City379/insert", Data: [6]float64{31, 0.021790000000000004, 0.021790000000000004, 7e-05, 0.00736, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City380/insert", Data: [6]float64{31, 0.008650000000000001, 0.008650000000000001, 7e-05, 0.00129, 0}},
		metric{Name: "Datastore/statement/MySQL/City381/insert", Data: [6]float64{31, 0.00724, 0.00724, 7e-05, 0.00064, 0}},
		metric{Name: "Datastore/statement/MySQL/City382/insert", Data: [6]float64{31, 0.013890000000000001, 0.013890000000000001, 7e-05, 0.00386, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City383/insert", Data: [6]float64{31, 0.03162, 0.03162, 7e-05, 0.01801, 0.00033000000000000005}},
		metric{Name: "Datastore/statement/MySQL/City384/insert", Data: [6]float64{31, 0.03063, 0.03063, 7e-05, 0.01241, 0.00017999999999999998}},
		metric{Name: "Datastore/statement/MySQL/City385/insert", Data: [6]float64{31, 0.009620000000000002, 0.009620000000000002, 7e-05, 0.00138, 0}},
		metric{Name: "Datastore/statement/MySQL/City386/insert", Data: [6]float64{31, 0.016710000000000006, 0.016710000000000006, 7e-05, 0.00482, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City387/insert", Data: [6]float64{31, 0.01533, 0.01533, 7e-05, 0.00675, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City388/insert", Data: [6]float64{31, 0.01233, 0.01233, 7e-05, 0.00277, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City389/insert", Data: [6]float64{31, 0.016369999999999996, 0.016369999999999996, 7e-05, 0.00516, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City390/insert", Data: [6]float64{31, 0.018639999999999997, 0.018639999999999997, 7e-05, 0.00445, 5.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City391/insert", Data: [6]float64{31, 0.010730000000000002, 0.010730000000000002, 7e-05, 0.00256, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City392/insert", Data: [6]float64{31, 0.009220000000000002, 0.009220000000000002, 7e-05, 0.00122, 0}},
		metric{Name: "Datastore/statement/MySQL/City393/insert", Data: [6]float64{31, 0.01056, 0.01056, 7e-05, 0.0031, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City394/insert", Data: [6]float64{31, 0.02643, 0.02643, 7e-05, 0.01651, 0.00028000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City395/insert", Data: [6]float64{31, 0.0168, 0.0168, 7e-05, 0.00419, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City396/insert", Data: [6]float64{31, 0.014700000000000001, 0.014700000000000001, 7e-05, 0.0046, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City397/insert", Data: [6]float64{31, 0.01475, 0.01475, 7e-05, 0.00494, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City398/insert", Data: [6]float64{31, 0.01006, 0.01006, 7e-05, 0.00249, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City399/insert", Data: [6]float64{31, 0.015990000000000004, 0.015990000000000004, 7e-05, 0.0082, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City400/insert", Data: [6]float64{31, 0.015960000000000002, 0.015960000000000002, 7e-05, 0.00536, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City401/insert", Data: [6]float64{31, 0.0369, 0.0369, 7e-05, 0.01977, 0.00042}},
		metric{Name: "Datastore/statement/MySQL/City402/insert", Data: [6]float64{31, 0.013, 0.013, 7e-05, 0.00305, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City403/insert", Data: [6]float64{31, 0.0093, 0.0093, 7e-05, 0.00196, 0}},
		metric{Name: "Datastore/statement/MySQL/City404/insert", Data: [6]float64{31, 0.013669999999999998, 0.013669999999999998, 7e-05, 0.00288, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City405/insert", Data: [6]float64{31, 0.020429999999999997, 0.020429999999999997, 7e-05, 0.0078, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City406/insert", Data: [6]float64{31, 0.019799999999999998, 0.019799999999999998, 7e-05, 0.01223, 0.00015}},
		metric{Name: "Datastore/statement/MySQL/City407/insert", Data: [6]float64{31, 0.00736, 0.00736, 7e-05, 0.00062, 0}},
		metric{Name: "Datastore/statement/MySQL/City408/insert", Data: [6]float64{31, 0.013759999999999996, 0.013759999999999996, 7e-05, 0.00621, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City409/insert", Data: [6]float64{31, 0.01366, 0.01366, 7e-05, 0.00382, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City410/insert", Data: [6]float64{31, 0.010879999999999999, 0.010879999999999999, 7e-05, 0.00303, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City411/insert", Data: [6]float64{31, 0.011660000000000004, 0.011660000000000004, 7e-05, 0.00373, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City412/insert", Data: [6]float64{31, 0.013069999999999998, 0.013069999999999998, 7e-05, 0.00351, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City413/insert", Data: [6]float64{31, 0.01564, 0.01564, 7e-05, 0.00626, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City414/insert", Data: [6]float64{31, 0.008980000000000002, 0.008980000000000002, 7e-05, 0.00098, 0}},
		metric{Name: "Datastore/statement/MySQL/City415/insert", Data: [6]float64{31, 0.016309999999999995, 0.016309999999999995, 8e-05, 0.00541, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City416/insert", Data: [6]float64{31, 0.023499999999999997, 0.023499999999999997, 8e-05, 0.00409, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City417/insert", Data: [6]float64{31, 0.01029, 0.01029, 7e-05, 0.00226, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City418/insert", Data: [6]float64{31, 0.04059, 0.04059, 7e-05, 0.03108, 0.00098}},
		metric{Name: "Datastore/statement/MySQL/City419/insert", Data: [6]float64{31, 0.018019999999999998, 0.018019999999999998, 7e-05, 0.00453, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City420/insert", Data: [6]float64{31, 0.011649999999999999, 0.011649999999999999, 7e-05, 0.00262, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City421/insert", Data: [6]float64{31, 0.014710000000000003, 0.014710000000000003, 7e-05, 0.00304, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City422/insert", Data: [6]float64{31, 0.00812, 0.00812, 7e-05, 0.00086, 0}},
		metric{Name: "Datastore/statement/MySQL/City423/insert", Data: [6]float64{31, 0.021480000000000006, 0.021480000000000006, 7e-05, 0.00776, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City424/insert", Data: [6]float64{31, 0.017669999999999998, 0.017669999999999998, 7e-05, 0.00455, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City425/insert", Data: [6]float64{31, 0.01412, 0.01412, 8e-05, 0.00254, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City426/insert", Data: [6]float64{31, 0.01243, 0.01243, 7e-05, 0.00424, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City427/insert", Data: [6]float64{31, 0.008710000000000002, 0.008710000000000002, 7e-05, 0.00099, 0}},
		metric{Name: "Datastore/statement/MySQL/City428/insert", Data: [6]float64{31, 0.010340000000000004, 0.010340000000000004, 7e-05, 0.0025, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City429/insert", Data: [6]float64{31, 0.007920000000000002, 0.007920000000000002, 7e-05, 0.0011, 0}},
		metric{Name: "Datastore/statement/MySQL/City430/insert", Data: [6]float64{31, 0.028720000000000002, 0.028720000000000002, 7e-05, 0.02078, 0.00043}},
		metric{Name: "Datastore/statement/MySQL/City431/insert", Data: [6]float64{31, 0.022419999999999995, 0.022419999999999995, 7e-05, 0.01115, 0.00013000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City432/insert", Data: [6]float64{31, 0.012230000000000001, 0.012230000000000001, 7e-05, 0.00267, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City433/insert", Data: [6]float64{31, 0.014830000000000001, 0.014830000000000001, 7e-05, 0.00464, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City434/insert", Data: [6]float64{31, 0.01808, 0.01808, 7e-05, 0.00463, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City435/insert", Data: [6]float64{31, 0.01409, 0.01409, 7e-05, 0.00323, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City436/insert", Data: [6]float64{31, 0.007980000000000001, 0.007980000000000001, 7e-05, 0.00085, 0}},
		metric{Name: "Datastore/statement/MySQL/City437/insert", Data: [6]float64{31, 0.01048, 0.01048, 7e-05, 0.00315, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City438/insert", Data: [6]float64{31, 0.009990000000000002, 0.009990000000000002, 7e-05, 0.00121, 0}},
		metric{Name: "Datastore/statement/MySQL/City439/insert", Data: [6]float64{31, 0.03147, 0.03147, 7e-05, 0.01619, 0.00028}},
		metric{Name: "Datastore/statement/MySQL/City440/insert", Data: [6]float64{31, 0.009790000000000002, 0.009790000000000002, 7e-05, 0.00101, 0}},
		metric{Name: "Datastore/statement/MySQL/City441/insert", Data: [6]float64{31, 0.04551, 0.04551, 7e-05, 0.02849, 0.00084}},
		metric{Name: "Datastore/statement/MySQL/City442/insert", Data: [6]float64{31, 0.00953, 0.00953, 7e-05, 0.00092, 0}},
		metric{Name: "Datastore/statement/MySQL/City443/insert", Data: [6]float64{31, 0.05558999999999999, 0.05558999999999999, 7e-05, 0.02647, 0.0009400000000000001}},
		metric{Name: "Datastore/statement/MySQL/City444/insert", Data: [6]float64{31, 0.01646, 0.01646, 7e-05, 0.00427, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City445/insert", Data: [6]float64{31, 0.01309, 0.01309, 8e-05, 0.00391, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City446/insert", Data: [6]float64{31, 0.024710000000000003, 0.024710000000000003, 7e-05, 0.01559, 0.00024}},
		metric{Name: "Datastore/statement/MySQL/City447/insert", Data: [6]float64{31, 0.01781, 0.01781, 7e-05, 0.00646, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City448/insert", Data: [6]float64{31, 0.014010000000000002, 0.014010000000000002, 7e-05, 0.00417, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City449/insert", Data: [6]float64{31, 0.00837, 0.00837, 7e-05, 0.00083, 0}},
		metric{Name: "Datastore/statement/MySQL/City450/insert", Data: [6]float64{31, 0.008579999999999999, 0.008579999999999999, 7e-05, 0.001, 0}},
		metric{Name: "Datastore/statement/MySQL/City451/insert", Data: [6]float64{31, 0.017740000000000002, 0.017740000000000002, 7e-05, 0.00456, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City452/insert", Data: [6]float64{31, 0.01732, 0.01732, 7e-05, 0.00412, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City453/insert", Data: [6]float64{31, 0.042230000000000004, 0.042230000000000004, 7e-05, 0.01384, 0.00035000000000000005}},
		metric{Name: "Datastore/statement/MySQL/City454/insert", Data: [6]float64{31, 0.01712, 0.01712, 8e-05, 0.00779, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City455/insert", Data: [6]float64{31, 0.01181, 0.01181, 7e-05, 0.00311, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City456/insert", Data: [6]float64{31, 0.0109, 0.0109, 7e-05, 0.00195, 0}},
		metric{Name: "Datastore/statement/MySQL/City457/insert", Data: [6]float64{31, 0.018860000000000002, 0.018860000000000002, 7e-05, 0.00395, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City458/insert", Data: [6]float64{31, 0.04132, 0.04132, 7e-05, 0.02895, 0.0008600000000000001}},
		metric{Name: "Datastore/statement/MySQL/City459/insert", Data: [6]float64{31, 0.01995, 0.01995, 7e-05, 0.01019, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City460/insert", Data: [6]float64{31, 0.0093, 0.0093, 7e-05, 0.00113, 0}},
		metric{Name: "Datastore/statement/MySQL/City461/insert", Data: [6]float64{31, 0.034379999999999994, 0.034379999999999994, 7e-05, 0.01785, 0.00037000000000000005}},
		metric{Name: "Datastore/statement/MySQL/City462/insert", Data: [6]float64{31, 0.03261, 0.03261, 7e-05, 0.01718, 0.00033}},
		metric{Name: "Datastore/statement/MySQL/City463/insert", Data: [6]float64{31, 0.013779999999999997, 0.013779999999999997, 7e-05, 0.00216, 0}},
		metric{Name: "Datastore/statement/MySQL/City464/insert", Data: [6]float64{31, 0.01058, 0.01058, 7e-05, 0.00145, 0}},
		metric{Name: "Datastore/statement/MySQL/City465/insert", Data: [6]float64{31, 0.012440000000000001, 0.012440000000000001, 7e-05, 0.00264, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City466/insert", Data: [6]float64{31, 0.02471, 0.02471, 7e-05, 0.0063, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City467/insert", Data: [6]float64{31, 0.011179999999999999, 0.011179999999999999, 7e-05, 0.00317, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City468/insert", Data: [6]float64{31, 0.013339999999999998, 0.013339999999999998, 7e-05, 0.00599, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City469/insert", Data: [6]float64{31, 0.01278, 0.01278, 7e-05, 0.00264, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City470/insert", Data: [6]float64{31, 0.014950000000000001, 0.014950000000000001, 7e-05, 0.00496, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City471/insert", Data: [6]float64{31, 0.01127, 0.01127, 7e-05, 0.00188, 0}},
		metric{Name: "Datastore/statement/MySQL/City472/insert", Data: [6]float64{31, 0.008930000000000002, 0.008930000000000002, 7e-05, 0.00101, 0}},
		metric{Name: "Datastore/statement/MySQL/City473/insert", Data: [6]float64{31, 0.017240000000000002, 0.017240000000000002, 7e-05, 0.00509, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City474/insert", Data: [6]float64{31, 0.01138, 0.01138, 7e-05, 0.00252, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City475/insert", Data: [6]float64{31, 0.030390000000000007, 0.030390000000000007, 7e-05, 0.01702, 0.00031}},
		metric{Name: "Datastore/statement/MySQL/City476/insert", Data: [6]float64{31, 0.03286000000000001, 0.03286000000000001, 7e-05, 0.00895, 0.00017}},
		metric{Name: "Datastore/statement/MySQL/City477/insert", Data: [6]float64{31, 0.018070000000000003, 0.018070000000000003, 8e-05, 0.00751, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City478/insert", Data: [6]float64{31, 0.015060000000000002, 0.015060000000000002, 7e-05, 0.00379, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City479/insert", Data: [6]float64{31, 0.022779999999999998, 0.022779999999999998, 7e-05, 0.00637, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City480/insert", Data: [6]float64{31, 0.008210000000000002, 0.008210000000000002, 7e-05, 0.00074, 0}},
		metric{Name: "Datastore/statement/MySQL/City481/insert", Data: [6]float64{31, 0.016940000000000004, 0.016940000000000004, 8e-05, 0.00444, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City482/insert", Data: [6]float64{31, 0.013830000000000004, 0.013830000000000004, 7e-05, 0.00444, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City483/insert", Data: [6]float64{31, 0.01925, 0.01925, 7e-05, 0.01129, 0.00013}},
		metric{Name: "Datastore/statement/MySQL/City484/insert", Data: [6]float64{31, 0.01185, 0.01185, 7e-05, 0.00251, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City485/insert", Data: [6]float64{31, 0.009779999999999999, 0.009779999999999999, 7e-05, 0.00182, 0}},
		metric{Name: "Datastore/statement/MySQL/City486/insert", Data: [6]float64{31, 0.01413, 0.01413, 7e-05, 0.00401, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City487/insert", Data: [6]float64{31, 0.01567, 0.01567, 7e-05, 0.00497, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City488/insert", Data: [6]float64{31, 0.013510000000000001, 0.013510000000000001, 7e-05, 0.00331, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City489/insert", Data: [6]float64{31, 0.01713, 0.01713, 7e-05, 0.00606, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City490/insert", Data: [6]float64{31, 0.024969999999999996, 0.024969999999999996, 8e-05, 0.00868, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City491/insert", Data: [6]float64{31, 0.0076, 0.0076, 7e-05, 0.00072, 0}},
		metric{Name: "Datastore/statement/MySQL/City492/insert", Data: [6]float64{31, 0.01055, 0.01055, 7e-05, 0.0023, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City493/insert", Data: [6]float64{31, 0.027569999999999997, 0.027569999999999997, 7e-05, 0.00855, 0.00013000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City494/insert", Data: [6]float64{31, 0.00917, 0.00917, 7e-05, 0.00085, 0}},
		metric{Name: "Datastore/statement/MySQL/City495/insert", Data: [6]float64{31, 0.013199999999999998, 0.013199999999999998, 7e-05, 0.00617, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City496/insert", Data: [6]float64{31, 0.02282, 0.02282, 7e-05, 0.00433, 6.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City497/insert", Data: [6]float64{31, 0.014300000000000002, 0.014300000000000002, 7e-05, 0.00399, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City498/insert", Data: [6]float64{31, 0.01649, 0.01649, 7e-05, 0.00545, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City499/insert", Data: [6]float64{31, 0.023969999999999995, 0.023969999999999995, 7e-05, 0.0058, 7.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City500/insert", Data: [6]float64{31, 0.03219, 0.03219, 7e-05, 0.01622, 0.00031}},
		metric{Name: "Datastore/statement/MySQL/City501/insert", Data: [6]float64{31, 0.032810000000000006, 0.032810000000000006, 7e-05, 0.0211, 0.00047000000000000004}},
		metric{Name: "Datastore/statement/MySQL/City502/insert", Data: [6]float64{31, 0.01115, 0.01115, 7e-05, 0.00294, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City503/insert", Data: [6]float64{31, 0.01721, 0.01721, 7e-05, 0.00465, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City504/insert", Data: [6]float64{31, 0.019749999999999997, 0.019749999999999997, 7e-05, 0.00573, 6.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City505/insert", Data: [6]float64{31, 0.03474, 0.03474, 7e-05, 0.01356, 0.00025}},
		metric{Name: "Datastore/statement/MySQL/City506/insert", Data: [6]float64{31, 0.01107, 0.01107, 7e-05, 0.0029, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City507/insert", Data: [6]float64{31, 0.02026, 0.02026, 7e-05, 0.00795, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City508/insert", Data: [6]float64{31, 0.01076, 0.01076, 7e-05, 0.0046, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City509/insert", Data: [6]float64{31, 0.00624, 0.00624, 7e-05, 0.0009, 0}},
		metric{Name: "Datastore/statement/MySQL/City510/insert", Data: [6]float64{31, 0.01908, 0.01908, 7e-05, 0.00443, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City511/insert", Data: [6]float64{31, 0.020870000000000003, 0.020870000000000003, 7e-05, 0.01162, 0.00014}},
		metric{Name: "Datastore/statement/MySQL/City512/insert", Data: [6]float64{31, 0.03263999999999999, 0.03263999999999999, 7e-05, 0.01695, 0.00033}},
		metric{Name: "Datastore/statement/MySQL/City513/insert", Data: [6]float64{31, 0.01281, 0.01281, 7e-05, 0.0031, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City514/insert", Data: [6]float64{31, 0.008749999999999999, 0.008749999999999999, 7e-05, 0.00259, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City515/insert", Data: [6]float64{31, 0.009360000000000002, 0.009360000000000002, 7e-05, 0.00314, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City516/insert", Data: [6]float64{31, 0.0223, 0.0223, 7e-05, 0.0044, 7.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City517/insert", Data: [6]float64{31, 0.01202, 0.01202, 7e-05, 0.00309, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City518/insert", Data: [6]float64{31, 0.01148, 0.01148, 7e-05, 0.00415, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City519/insert", Data: [6]float64{31, 0.01754, 0.01754, 7e-05, 0.00831, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City520/insert", Data: [6]float64{31, 0.00664, 0.00664, 7e-05, 0.00065, 0}},
		metric{Name: "Datastore/statement/MySQL/City521/insert", Data: [6]float64{31, 0.02639, 0.02639, 7e-05, 0.01613, 0.00027}},
		metric{Name: "Datastore/statement/MySQL/City522/insert", Data: [6]float64{31, 0.014300000000000004, 0.014300000000000004, 7e-05, 0.00517, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City523/insert", Data: [6]float64{31, 0.00701, 0.00701, 7e-05, 0.00069, 0}},
		metric{Name: "Datastore/statement/MySQL/City524/insert", Data: [6]float64{31, 0.030180000000000005, 0.030180000000000005, 7e-05, 0.01513, 0.00025}},
		metric{Name: "Datastore/statement/MySQL/City525/insert", Data: [6]float64{31, 0.011140000000000002, 0.011140000000000002, 7e-05, 0.00218, 0}},
		metric{Name: "Datastore/statement/MySQL/City526/insert", Data: [6]float64{31, 0.01689, 0.01689, 7e-05, 0.00499, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City527/insert", Data: [6]float64{31, 0.01024, 0.01024, 7e-05, 0.00171, 0}},
		metric{Name: "Datastore/statement/MySQL/City528/insert", Data: [6]float64{31, 0.01434, 0.01434, 7e-05, 0.00674, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City529/insert", Data: [6]float64{31, 0.011640000000000001, 0.011640000000000001, 7e-05, 0.00428, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City530/insert", Data: [6]float64{31, 0.040619999999999996, 0.040619999999999996, 7e-05, 0.02426, 0.00064}},
		metric{Name: "Datastore/statement/MySQL/City531/insert", Data: [6]float64{31, 0.02276, 0.02276, 7e-05, 0.00797, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City532/insert", Data: [6]float64{31, 0.015199999999999997, 0.015199999999999997, 7e-05, 0.00333, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City533/insert", Data: [6]float64{31, 0.01884, 0.01884, 7e-05, 0.00574, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City534/insert", Data: [6]float64{31, 0.02099, 0.02099, 7e-05, 0.00911, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City535/insert", Data: [6]float64{31, 0.00808, 0.00808, 7e-05, 0.00073, 0}},
		metric{Name: "Datastore/statement/MySQL/City536/insert", Data: [6]float64{31, 0.008339999999999998, 0.008339999999999998, 7e-05, 0.00132, 0}},
		metric{Name: "Datastore/statement/MySQL/City537/insert", Data: [6]float64{31, 0.01259, 0.01259, 7e-05, 0.00448, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City538/insert", Data: [6]float64{31, 0.01535, 0.01535, 7e-05, 0.00613, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City539/insert", Data: [6]float64{31, 0.01089, 0.01089, 7e-05, 0.00441, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City540/insert", Data: [6]float64{31, 0.026889999999999997, 0.026889999999999997, 7e-05, 0.01163, 0.00016999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City541/insert", Data: [6]float64{31, 0.02137, 0.02137, 7e-05, 0.00847, 9.999999999999999e-05}},
		metric{Name: "Datastore/statement/MySQL/City542/insert", Data: [6]float64{31, 0.020240000000000005, 0.020240000000000005, 7e-05, 0.00578, 6.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City543/insert", Data: [6]float64{31, 0.023440000000000003, 0.023440000000000003, 7e-05, 0.00906, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City544/insert", Data: [6]float64{31, 0.010600000000000002, 0.010600000000000002, 7e-05, 0.00338, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City545/insert", Data: [6]float64{31, 0.01508, 0.01508, 7e-05, 0.0043, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City546/insert", Data: [6]float64{31, 0.011119999999999998, 0.011119999999999998, 7e-05, 0.00511, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City547/insert", Data: [6]float64{31, 0.0072400000000000025, 0.0072400000000000025, 7e-05, 0.00062, 0}},
		metric{Name: "Datastore/statement/MySQL/City548/insert", Data: [6]float64{31, 0.01659, 0.01659, 7e-05, 0.00553, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City549/insert", Data: [6]float64{31, 0.0091, 0.0091, 7e-05, 0.00129, 0}},
		metric{Name: "Datastore/statement/MySQL/City550/insert", Data: [6]float64{31, 0.009260000000000001, 0.009260000000000001, 7e-05, 0.00202, 0}},
		metric{Name: "Datastore/statement/MySQL/City551/insert", Data: [6]float64{31, 0.01016, 0.01016, 7e-05, 0.00321, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City552/insert", Data: [6]float64{31, 0.0145, 0.0145, 7e-05, 0.00379, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City553/insert", Data: [6]float64{31, 0.015390000000000003, 0.015390000000000003, 7e-05, 0.00527, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City554/insert", Data: [6]float64{31, 0.02644000000000001, 0.02644000000000001, 7e-05, 0.01554, 0.00026000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City555/insert", Data: [6]float64{31, 0.009349999999999999, 0.009349999999999999, 7e-05, 0.00244, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City556/insert", Data: [6]float64{31, 0.029020000000000004, 0.029020000000000004, 7e-05, 0.02225, 0.00049}},
		metric{Name: "Datastore/statement/MySQL/City557/insert", Data: [6]float64{31, 0.01662, 0.01662, 7e-05, 0.00442, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City558/insert", Data: [6]float64{31, 0.014600000000000004, 0.014600000000000004, 7e-05, 0.00712, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City559/insert", Data: [6]float64{31, 0.010169999999999998, 0.010169999999999998, 7e-05, 0.00207, 0}},
		metric{Name: "Datastore/statement/MySQL/City560/insert", Data: [6]float64{31, 0.010689999999999998, 0.010689999999999998, 7e-05, 0.00346, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City561/insert", Data: [6]float64{31, 0.008830000000000001, 0.008830000000000001, 7e-05, 0.00165, 0}},
		metric{Name: "Datastore/statement/MySQL/City562/insert", Data: [6]float64{31, 0.01702, 0.01702, 7e-05, 0.00434, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City563/insert", Data: [6]float64{31, 0.00813, 0.00813, 7e-05, 0.00108, 0}},
		metric{Name: "Datastore/statement/MySQL/City564/insert", Data: [6]float64{31, 0.015420000000000001, 0.015420000000000001, 7e-05, 0.00447, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City565/insert", Data: [6]float64{31, 0.013660000000000002, 0.013660000000000002, 7e-05, 0.0054, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City566/insert", Data: [6]float64{31, 0.009920000000000002, 0.009920000000000002, 7e-05, 0.00266, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City567/insert", Data: [6]float64{31, 0.015330000000000002, 0.015330000000000002, 7e-05, 0.0049, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City568/insert", Data: [6]float64{31, 0.011720000000000001, 0.011720000000000001, 7e-05, 0.00496, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City569/insert", Data: [6]float64{31, 0.05303, 0.05303, 7e-05, 0.02203, 0.00071}},
		metric{Name: "Datastore/statement/MySQL/City570/insert", Data: [6]float64{31, 0.01818, 0.01818, 7e-05, 0.00526, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City571/insert", Data: [6]float64{31, 0.01806, 0.01806, 7e-05, 0.00575, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City572/insert", Data: [6]float64{31, 0.014940000000000002, 0.014940000000000002, 7e-05, 0.0036, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City573/insert", Data: [6]float64{31, 0.012439999999999998, 0.012439999999999998, 7e-05, 0.00358, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City574/insert", Data: [6]float64{31, 0.011070000000000002, 0.011070000000000002, 7e-05, 0.00254, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City575/insert", Data: [6]float64{31, 0.01898, 0.01898, 7e-05, 0.00773, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City576/insert", Data: [6]float64{31, 0.02513, 0.02513, 7e-05, 0.00899, 0.00016}},
		metric{Name: "Datastore/statement/MySQL/City577/insert", Data: [6]float64{31, 0.01086, 0.01086, 7e-05, 0.00324, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City578/insert", Data: [6]float64{31, 0.04229, 0.04229, 7e-05, 0.02088, 0.0005}},
		metric{Name: "Datastore/statement/MySQL/City579/insert", Data: [6]float64{31, 0.01057, 0.01057, 7e-05, 0.00306, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City580/insert", Data: [6]float64{31, 0.03186, 0.03186, 7e-05, 0.01894, 0.0003800000000000001}},
		metric{Name: "Datastore/statement/MySQL/City581/insert", Data: [6]float64{31, 0.01573, 0.01573, 7e-05, 0.00837, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City582/insert", Data: [6]float64{31, 0.012150000000000001, 0.012150000000000001, 7e-05, 0.00316, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City583/insert", Data: [6]float64{31, 0.008580000000000003, 0.008580000000000003, 7e-05, 0.00102, 0}},
		metric{Name: "Datastore/statement/MySQL/City584/insert", Data: [6]float64{31, 0.01291, 0.01291, 7e-05, 0.00578, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City585/insert", Data: [6]float64{31, 0.01907, 0.01907, 7e-05, 0.00594, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City586/insert", Data: [6]float64{31, 0.00732, 0.00732, 7e-05, 0.00069, 0}},
		metric{Name: "Datastore/statement/MySQL/City587/insert", Data: [6]float64{31, 0.0078000000000000005, 0.0078000000000000005, 7e-05, 0.00108, 0}},
		metric{Name: "Datastore/statement/MySQL/City588/insert", Data: [6]float64{31, 0.02179, 0.02179, 7e-05, 0.00626, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City589/insert", Data: [6]float64{31, 0.01238, 0.01238, 7e-05, 0.0037, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City590/insert", Data: [6]float64{31, 0.01337, 0.01337, 7e-05, 0.00286, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City591/insert", Data: [6]float64{31, 0.01151, 0.01151, 7e-05, 0.00467, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City592/insert", Data: [6]float64{31, 0.011880000000000002, 0.011880000000000002, 7e-05, 0.00343, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City593/insert", Data: [6]float64{31, 0.0132, 0.0132, 7e-05, 0.00383, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City594/insert", Data: [6]float64{31, 0.005980000000000001, 0.005980000000000001, 7e-05, 0.00065, 0}},
		metric{Name: "Datastore/statement/MySQL/City595/insert", Data: [6]float64{31, 0.008589999999999999, 0.008589999999999999, 7e-05, 0.00084, 0}},
		metric{Name: "Datastore/statement/MySQL/City596/insert", Data: [6]float64{31, 0.014790000000000001, 0.014790000000000001, 7e-05, 0.00892, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City597/insert", Data: [6]float64{31, 0.0188, 0.0188, 7e-05, 0.0053, 6.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City598/insert", Data: [6]float64{31, 0.011359999999999999, 0.011359999999999999, 7e-05, 0.00527, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City599/insert", Data: [6]float64{31, 0.01326, 0.01326, 7e-05, 0.00428, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City600/insert", Data: [6]float64{31, 0.0247, 0.0247, 7e-05, 0.01228, 0.00016999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City601/insert", Data: [6]float64{31, 0.02478, 0.02478, 7e-05, 0.0054, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City602/insert", Data: [6]float64{31, 0.02857, 0.02857, 7e-05, 0.00829, 0.00011999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City603/insert", Data: [6]float64{31, 0.00784, 0.00784, 7e-05, 0.00163, 0}},
		metric{Name: "Datastore/statement/MySQL/City604/insert", Data: [6]float64{31, 0.03673, 0.03673, 7e-05, 0.02643, 0.00072}},
		metric{Name: "Datastore/statement/MySQL/City605/insert", Data: [6]float64{31, 0.01135, 0.01135, 7e-05, 0.00466, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City606/insert", Data: [6]float64{31, 0.00902, 0.00902, 7e-05, 0.00323, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City607/insert", Data: [6]float64{31, 0.01403, 0.01403, 7e-05, 0.00762, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City608/insert", Data: [6]float64{31, 0.013380000000000005, 0.013380000000000005, 7e-05, 0.00394, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City609/insert", Data: [6]float64{31, 0.00857, 0.00857, 7e-05, 0.00189, 0}},
		metric{Name: "Datastore/statement/MySQL/City610/insert", Data: [6]float64{31, 0.008539999999999999, 0.008539999999999999, 7e-05, 0.00172, 0}},
		metric{Name: "Datastore/statement/MySQL/City611/insert", Data: [6]float64{31, 0.013299999999999998, 0.013299999999999998, 7e-05, 0.00416, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City612/insert", Data: [6]float64{31, 0.011590000000000001, 0.011590000000000001, 7e-05, 0.00259, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City613/insert", Data: [6]float64{31, 0.014560000000000002, 0.014560000000000002, 7e-05, 0.00368, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City614/insert", Data: [6]float64{31, 0.01308, 0.01308, 7e-05, 0.00432, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City615/insert", Data: [6]float64{31, 0.006680000000000002, 0.006680000000000002, 7e-05, 0.00063, 0}},
		metric{Name: "Datastore/statement/MySQL/City616/insert", Data: [6]float64{31, 0.01417, 0.01417, 7e-05, 0.00424, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City617/insert", Data: [6]float64{31, 0.02067, 0.02067, 7e-05, 0.00866, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City618/insert", Data: [6]float64{31, 0.02023, 0.02023, 7e-05, 0.01221, 0.00015}},
		metric{Name: "Datastore/statement/MySQL/City619/insert", Data: [6]float64{31, 0.010889999999999999, 0.010889999999999999, 7e-05, 0.00331, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City620/insert", Data: [6]float64{31, 0.0125, 0.0125, 7e-05, 0.00424, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City621/insert", Data: [6]float64{31, 0.015570000000000002, 0.015570000000000002, 7e-05, 0.0037, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City622/insert", Data: [6]float64{31, 0.01409, 0.01409, 7e-05, 0.00425, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City623/insert", Data: [6]float64{31, 0.01812, 0.01812, 7e-05, 0.00492, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City624/insert", Data: [6]float64{31, 0.01454, 0.01454, 7e-05, 0.00381, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City625/insert", Data: [6]float64{31, 0.022400000000000003, 0.022400000000000003, 7e-05, 0.01003, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City626/insert", Data: [6]float64{31, 0.024069999999999998, 0.024069999999999998, 7e-05, 0.00834, 0.00010999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City627/insert", Data: [6]float64{31, 0.01348, 0.01348, 7e-05, 0.00521, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City628/insert", Data: [6]float64{31, 0.02498, 0.02498, 7e-05, 0.01074, 0.00014000000000000001}},
		metric{Name: "Datastore/statement/MySQL/City629/insert", Data: [6]float64{31, 0.015549999999999998, 0.015549999999999998, 7e-05, 0.00575, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City630/insert", Data: [6]float64{31, 0.00762, 0.00762, 7e-05, 0.00083, 0}},
		metric{Name: "Datastore/statement/MySQL/City631/insert", Data: [6]float64{31, 0.009430000000000001, 0.009430000000000001, 7e-05, 0.00183, 0}},
		metric{Name: "Datastore/statement/MySQL/City632/insert", Data: [6]float64{31, 0.02893, 0.02893, 7e-05, 0.02026, 0.00041}},
		metric{Name: "Datastore/statement/MySQL/City633/insert", Data: [6]float64{31, 0.01445, 0.01445, 7e-05, 0.00419, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City634/insert", Data: [6]float64{31, 0.02721, 0.02721, 7e-05, 0.01349, 0.0002}},
		metric{Name: "Datastore/statement/MySQL/City635/insert", Data: [6]float64{31, 0.01614, 0.01614, 7e-05, 0.00451, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City636/insert", Data: [6]float64{31, 0.01187, 0.01187, 7e-05, 0.00267, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City637/insert", Data: [6]float64{31, 0.023010000000000003, 0.023010000000000003, 7e-05, 0.00516, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City638/insert", Data: [6]float64{31, 0.01371, 0.01371, 7e-05, 0.00361, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City639/insert", Data: [6]float64{31, 0.024460000000000003, 0.024460000000000003, 7e-05, 0.00975, 0.00017}},
		metric{Name: "Datastore/statement/MySQL/City640/insert", Data: [6]float64{31, 0.01655, 0.01655, 7e-05, 0.00421, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City641/insert", Data: [6]float64{31, 0.00947, 0.00947, 7e-05, 0.0038, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City642/insert", Data: [6]float64{31, 0.01456, 0.01456, 7e-05, 0.00543, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City643/insert", Data: [6]float64{31, 0.01465, 0.01465, 7e-05, 0.0032, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City644/insert", Data: [6]float64{31, 0.008910000000000001, 0.008910000000000001, 7e-05, 0.00129, 0}},
		metric{Name: "Datastore/statement/MySQL/City645/insert", Data: [6]float64{31, 0.008370000000000002, 0.008370000000000002, 7e-05, 0.00161, 0}},
		metric{Name: "Datastore/statement/MySQL/City646/insert", Data: [6]float64{31, 0.005430000000000001, 0.005430000000000001, 7e-05, 0.00066, 0}},
		metric{Name: "Datastore/statement/MySQL/City647/insert", Data: [6]float64{31, 0.01875, 0.01875, 7e-05, 0.00765, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City648/insert", Data: [6]float64{31, 0.007050000000000003, 0.007050000000000003, 7e-05, 0.00099, 0}},
		metric{Name: "Datastore/statement/MySQL/City649/insert", Data: [6]float64{31, 0.01052, 0.01052, 7e-05, 0.00579, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City650/insert", Data: [6]float64{31, 0.0049, 0.0049, 7e-05, 0.00053, 0}},
		metric{Name: "Datastore/statement/MySQL/City651/insert", Data: [6]float64{31, 0.00518, 0.00518, 7e-05, 0.00093, 0}},
		metric{Name: "Datastore/statement/MySQL/City652/insert", Data: [6]float64{31, 0.03553, 0.03553, 7e-05, 0.02674, 0.0007300000000000001}},
		metric{Name: "Datastore/statement/MySQL/City653/insert", Data: [6]float64{31, 0.015529999999999999, 0.015529999999999999, 7e-05, 0.00598, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City654/insert", Data: [6]float64{31, 0.03339, 0.03339, 7e-05, 0.01774, 0.00038}},
		metric{Name: "Datastore/statement/MySQL/City655/insert", Data: [6]float64{31, 0.024470000000000006, 0.024470000000000006, 7e-05, 0.0149, 0.00024}},
		metric{Name: "Datastore/statement/MySQL/City656/insert", Data: [6]float64{31, 0.0059900000000000005, 0.0059900000000000005, 7e-05, 0.00093, 0}},
		metric{Name: "Datastore/statement/MySQL/City657/insert", Data: [6]float64{31, 0.038439999999999995, 0.038439999999999995, 7e-05, 0.02525, 0.00067}},
		metric{Name: "Datastore/statement/MySQL/City658/insert", Data: [6]float64{31, 0.01403, 0.01403, 7e-05, 0.00434, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City659/insert", Data: [6]float64{31, 0.00609, 0.00609, 7e-05, 0.00081, 0}},
		metric{Name: "Datastore/statement/MySQL/City660/insert", Data: [6]float64{31, 0.013830000000000002, 0.013830000000000002, 7e-05, 0.0051, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City661/insert", Data: [6]float64{31, 0.022319999999999996, 0.022319999999999996, 7e-05, 0.00883, 0.00016}},
		metric{Name: "Datastore/statement/MySQL/City662/insert", Data: [6]float64{31, 0.04442000000000001, 0.04442000000000001, 7e-05, 0.01752, 0.0003800000000000001}},
		metric{Name: "Datastore/statement/MySQL/City663/insert", Data: [6]float64{31, 0.013560000000000004, 0.013560000000000004, 7e-05, 0.008, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City664/insert", Data: [6]float64{31, 0.012669999999999999, 0.012669999999999999, 7e-05, 0.00388, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City665/insert", Data: [6]float64{31, 0.01334, 0.01334, 7e-05, 0.00477, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City666/insert", Data: [6]float64{31, 0.015040000000000001, 0.015040000000000001, 7e-05, 0.00649, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City667/insert", Data: [6]float64{31, 0.014400000000000001, 0.014400000000000001, 7e-05, 0.00507, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City668/insert", Data: [6]float64{31, 0.010210000000000002, 0.010210000000000002, 7e-05, 0.00462, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City669/insert", Data: [6]float64{31, 0.01344, 0.01344, 7e-05, 0.0044, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City670/insert", Data: [6]float64{31, 0.012700000000000003, 0.012700000000000003, 7e-05, 0.00417, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City671/insert", Data: [6]float64{31, 0.008300000000000004, 0.008300000000000004, 7e-05, 0.00205, 0}},
		metric{Name: "Datastore/statement/MySQL/City672/insert", Data: [6]float64{31, 0.01851, 0.01851, 7e-05, 0.01158, 0.00013}},
		metric{Name: "Datastore/statement/MySQL/City673/insert", Data: [6]float64{31, 0.013330000000000002, 0.013330000000000002, 7e-05, 0.00758, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City674/insert", Data: [6]float64{31, 0.014520000000000003, 0.014520000000000003, 7e-05, 0.00625, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City675/insert", Data: [6]float64{31, 0.00865, 0.00865, 7e-05, 0.00233, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City676/insert", Data: [6]float64{31, 0.009720000000000001, 0.009720000000000001, 7e-05, 0.00444, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City677/insert", Data: [6]float64{31, 0.008070000000000003, 0.008070000000000003, 7e-05, 0.00167, 0}},
		metric{Name: "Datastore/statement/MySQL/City678/insert", Data: [6]float64{31, 0.018549999999999997, 0.018549999999999997, 7e-05, 0.00989, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City679/insert", Data: [6]float64{31, 0.009470000000000001, 0.009470000000000001, 7e-05, 0.0041, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City680/insert", Data: [6]float64{31, 0.018749999999999996, 0.018749999999999996, 7e-05, 0.00884, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City681/insert", Data: [6]float64{31, 0.005540000000000001, 0.005540000000000001, 7e-05, 0.00063, 0}},
		metric{Name: "Datastore/statement/MySQL/City682/insert", Data: [6]float64{31, 0.010760000000000002, 0.010760000000000002, 7e-05, 0.00495, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City683/insert", Data: [6]float64{31, 0.00623, 0.00623, 7e-05, 0.00105, 0}},
		metric{Name: "Datastore/statement/MySQL/City684/insert", Data: [6]float64{31, 0.01578, 0.01578, 7e-05, 0.00358, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City685/insert", Data: [6]float64{31, 0.009230000000000002, 0.009230000000000002, 7e-05, 0.00281, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City686/insert", Data: [6]float64{31, 0.02162, 0.02162, 7e-05, 0.01201, 0.00015}},
		metric{Name: "Datastore/statement/MySQL/City687/insert", Data: [6]float64{31, 0.024680000000000004, 0.024680000000000004, 7e-05, 0.00859, 0.00015}},
		metric{Name: "Datastore/statement/MySQL/City688/insert", Data: [6]float64{31, 0.00771, 0.00771, 7e-05, 0.00278, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City689/insert", Data: [6]float64{31, 0.0066500000000000005, 0.0066500000000000005, 7e-05, 0.00168, 0}},
		metric{Name: "Datastore/statement/MySQL/City690/insert", Data: [6]float64{31, 0.01002, 0.01002, 7e-05, 0.00514, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City691/insert", Data: [6]float64{31, 0.007270000000000001, 0.007270000000000001, 8e-05, 0.00123, 0}},
		metric{Name: "Datastore/statement/MySQL/City692/insert", Data: [6]float64{31, 0.00732, 0.00732, 7e-05, 0.00147, 0}},
		metric{Name: "Datastore/statement/MySQL/City693/insert", Data: [6]float64{31, 0.015240000000000002, 0.015240000000000002, 7e-05, 0.00684, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City694/insert", Data: [6]float64{31, 0.014760000000000002, 0.014760000000000002, 7e-05, 0.00814, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City695/insert", Data: [6]float64{31, 0.01183, 0.01183, 7e-05, 0.00683, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City696/insert", Data: [6]float64{31, 0.04116999999999999, 0.04116999999999999, 7e-05, 0.02143, 0.00059}},
		metric{Name: "Datastore/statement/MySQL/City697/insert", Data: [6]float64{31, 0.007260000000000001, 0.007260000000000001, 7e-05, 0.00211, 0}},
		metric{Name: "Datastore/statement/MySQL/City698/insert", Data: [6]float64{31, 0.04682999999999998, 0.04682999999999998, 7e-05, 0.02131, 0.00081}},
		metric{Name: "Datastore/statement/MySQL/City699/insert", Data: [6]float64{31, 0.012800000000000002, 0.012800000000000002, 7e-05, 0.00401, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City700/insert", Data: [6]float64{31, 0.009010000000000002, 0.009010000000000002, 7e-05, 0.00398, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City701/insert", Data: [6]float64{31, 0.007540000000000002, 0.007540000000000002, 7e-05, 0.003, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City702/insert", Data: [6]float64{31, 0.014250000000000002, 0.014250000000000002, 7e-05, 0.00881, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City703/insert", Data: [6]float64{31, 0.006560000000000001, 0.006560000000000001, 7e-05, 0.00162, 0}},
		metric{Name: "Datastore/statement/MySQL/City704/insert", Data: [6]float64{31, 0.03784, 0.03784, 7e-05, 0.02752, 0.0007800000000000001}},
		metric{Name: "Datastore/statement/MySQL/City705/insert", Data: [6]float64{31, 0.014910000000000001, 0.014910000000000001, 7e-05, 0.00539, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City706/insert", Data: [6]float64{31, 0.22379999999999994, 0.22379999999999994, 7e-05, 0.21574, 0.046560000000000004}},
		metric{Name: "Datastore/statement/MySQL/City707/insert", Data: [6]float64{31, 0.00694, 0.00694, 7e-05, 0.00124, 0}},
		metric{Name: "Datastore/statement/MySQL/City708/insert", Data: [6]float64{31, 0.006190000000000001, 0.006190000000000001, 7e-05, 0.00137, 0}},
		metric{Name: "Datastore/statement/MySQL/City709/insert", Data: [6]float64{31, 0.006590000000000001, 0.006590000000000001, 7e-05, 0.00088, 0}},
		metric{Name: "Datastore/statement/MySQL/City710/insert", Data: [6]float64{31, 0.016270000000000003, 0.016270000000000003, 7e-05, 0.00785, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City711/insert", Data: [6]float64{31, 0.00971, 0.00971, 7e-05, 0.00296, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City712/insert", Data: [6]float64{31, 0.01593, 0.01593, 7e-05, 0.01056, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City713/insert", Data: [6]float64{31, 0.03676999999999999, 0.03676999999999999, 7e-05, 0.02249, 0.0006000000000000001}},
		metric{Name: "Datastore/statement/MySQL/City714/insert", Data: [6]float64{31, 0.010210000000000002, 0.010210000000000002, 7e-05, 0.00411, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City715/insert", Data: [6]float64{31, 0.005930000000000003, 0.005930000000000003, 7e-05, 0.00115, 0}},
		metric{Name: "Datastore/statement/MySQL/City716/insert", Data: [6]float64{31, 0.006420000000000001, 0.006420000000000001, 7e-05, 0.00163, 0}},
		metric{Name: "Datastore/statement/MySQL/City717/insert", Data: [6]float64{31, 0.03352, 0.03352, 7e-05, 0.02425, 0.0006000000000000001}},
		metric{Name: "Datastore/statement/MySQL/City718/insert", Data: [6]float64{31, 0.007120000000000001, 0.007120000000000001, 7e-05, 0.00192, 0}},
		metric{Name: "Datastore/statement/MySQL/City719/insert", Data: [6]float64{31, 0.03736, 0.03736, 7e-05, 0.02317, 0.0005700000000000001}},
		metric{Name: "Datastore/statement/MySQL/City720/insert", Data: [6]float64{31, 0.005370000000000002, 0.005370000000000002, 7e-05, 0.00083, 0}},
		metric{Name: "Datastore/statement/MySQL/City721/insert", Data: [6]float64{31, 0.00634, 0.00634, 7e-05, 0.0013, 0}},
		metric{Name: "Datastore/statement/MySQL/City722/insert", Data: [6]float64{31, 0.00974, 0.00974, 7e-05, 0.00398, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City723/insert", Data: [6]float64{31, 0.030389999999999997, 0.030389999999999997, 7e-05, 0.01169, 0.00021999999999999998}},
		metric{Name: "Datastore/statement/MySQL/City724/insert", Data: [6]float64{31, 0.008769999999999998, 0.008769999999999998, 7e-05, 0.00356, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City725/insert", Data: [6]float64{31, 0.01294, 0.01294, 7e-05, 0.00465, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City726/insert", Data: [6]float64{31, 0.01623, 0.01623, 7e-05, 0.00589, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City727/insert", Data: [6]float64{31, 0.00742, 0.00742, 7e-05, 0.00257, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City728/insert", Data: [6]float64{31, 0.015600000000000001, 0.015600000000000001, 7e-05, 0.0065, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City729/insert", Data: [6]float64{31, 0.004670000000000002, 0.004670000000000002, 7e-05, 0.00089, 0}},
		metric{Name: "Datastore/statement/MySQL/City730/insert", Data: [6]float64{31, 0.01643, 0.01643, 7e-05, 0.00882, 9e-05}},
		metric{Name: "Datastore/statement/MySQL/City731/insert", Data: [6]float64{31, 0.010220000000000002, 0.010220000000000002, 7e-05, 0.00559, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City732/insert", Data: [6]float64{31, 0.006650000000000002, 0.006650000000000002, 7e-05, 0.00128, 0}},
		metric{Name: "Datastore/statement/MySQL/City733/insert", Data: [6]float64{31, 0.019930000000000003, 0.019930000000000003, 7e-05, 0.00838, 8.999999999999999e-05}},
		metric{Name: "Datastore/statement/MySQL/City734/insert", Data: [6]float64{31, 0.02365, 0.02365, 7e-05, 0.0098, 0.00016}},
		metric{Name: "Datastore/statement/MySQL/City735/insert", Data: [6]float64{31, 0.012230000000000003, 0.012230000000000003, 8e-05, 0.00673, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City736/insert", Data: [6]float64{31, 0.005710000000000001, 0.005710000000000001, 7e-05, 0.00077, 0}},
		metric{Name: "Datastore/statement/MySQL/City737/insert", Data: [6]float64{31, 0.015870000000000002, 0.015870000000000002, 7e-05, 0.00778, 7.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City738/insert", Data: [6]float64{31, 0.043480000000000005, 0.043480000000000005, 7e-05, 0.03785, 0.00143}},
		metric{Name: "Datastore/statement/MySQL/City739/insert", Data: [6]float64{31, 0.00507, 0.00507, 7e-05, 0.00049, 0}},
		metric{Name: "Datastore/statement/MySQL/City740/insert", Data: [6]float64{31, 0.02401, 0.02401, 7e-05, 0.01841, 0.00034}},
		metric{Name: "Datastore/statement/MySQL/City741/insert", Data: [6]float64{31, 0.007430000000000003, 0.007430000000000003, 8e-05, 0.00177, 0}},
		metric{Name: "Datastore/statement/MySQL/City742/insert", Data: [6]float64{31, 0.004970000000000003, 0.004970000000000003, 7e-05, 0.00121, 0}},
		metric{Name: "Datastore/statement/MySQL/City743/insert", Data: [6]float64{31, 0.029890000000000003, 0.029890000000000003, 7e-05, 0.02471, 0.00061}},
		metric{Name: "Datastore/statement/MySQL/City744/insert", Data: [6]float64{31, 0.008, 0.008, 7e-05, 0.00319, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City745/insert", Data: [6]float64{31, 0.006530000000000001, 0.006530000000000001, 7e-05, 0.00277, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City746/insert", Data: [6]float64{31, 0.011310000000000002, 0.011310000000000002, 7e-05, 0.00412, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City747/insert", Data: [6]float64{31, 0.00432, 0.00432, 7e-05, 0.00061, 0}},
		metric{Name: "Datastore/statement/MySQL/City748/insert", Data: [6]float64{31, 0.01513, 0.01513, 7e-05, 0.01116, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City749/insert", Data: [6]float64{31, 0.037819999999999986, 0.037819999999999986, 7e-05, 0.0259, 0.00072}},
		metric{Name: "Datastore/statement/MySQL/City750/insert", Data: [6]float64{31, 0.007780000000000002, 0.007780000000000002, 7e-05, 0.00294, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City751/insert", Data: [6]float64{31, 0.004540000000000003, 0.004540000000000003, 7e-05, 0.00123, 0}},
		metric{Name: "Datastore/statement/MySQL/City752/insert", Data: [6]float64{31, 0.03226, 0.03226, 7e-05, 0.02615, 0.00068}},
		metric{Name: "Datastore/statement/MySQL/City753/insert", Data: [6]float64{31, 0.007190000000000001, 0.007190000000000001, 7e-05, 0.00314, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City754/insert", Data: [6]float64{31, 0.015170000000000005, 0.015170000000000005, 7e-05, 0.00576, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City755/insert", Data: [6]float64{31, 0.009790000000000004, 0.009790000000000004, 7e-05, 0.00494, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City756/insert", Data: [6]float64{31, 0.011780000000000002, 0.011780000000000002, 7e-05, 0.004, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City757/insert", Data: [6]float64{31, 0.004550000000000001, 0.004550000000000001, 7e-05, 0.00064, 0}},
		metric{Name: "Datastore/statement/MySQL/City758/insert", Data: [6]float64{31, 0.03601, 0.03601, 7e-05, 0.01782, 0.00042000000000000007}},
		metric{Name: "Datastore/statement/MySQL/City759/insert", Data: [6]float64{31, 0.005010000000000002, 0.005010000000000002, 7e-05, 0.00099, 0}},
		metric{Name: "Datastore/statement/MySQL/City760/insert", Data: [6]float64{31, 0.007430000000000002, 0.007430000000000002, 7e-05, 0.00203, 0}},
		metric{Name: "Datastore/statement/MySQL/City761/insert", Data: [6]float64{31, 0.01924, 0.01924, 7e-05, 0.0145, 0.00021}},
		metric{Name: "Datastore/statement/MySQL/City762/insert", Data: [6]float64{31, 0.003990000000000002, 0.003990000000000002, 7e-05, 0.00058, 0}},
		metric{Name: "Datastore/statement/MySQL/City763/insert", Data: [6]float64{31, 0.007340000000000001, 0.007340000000000001, 7e-05, 0.00264, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City764/insert", Data: [6]float64{31, 0.005730000000000002, 0.005730000000000002, 7e-05, 0.00137, 0}},
		metric{Name: "Datastore/statement/MySQL/City765/insert", Data: [6]float64{31, 0.01079, 0.01079, 7e-05, 0.0064, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City766/insert", Data: [6]float64{31, 0.005990000000000001, 0.005990000000000001, 7e-05, 0.00163, 0}},
		metric{Name: "Datastore/statement/MySQL/City767/insert", Data: [6]float64{31, 0.03331, 0.03331, 7e-05, 0.01786, 0.00039000000000000005}},
		metric{Name: "Datastore/statement/MySQL/City768/insert", Data: [6]float64{31, 0.011970000000000001, 0.011970000000000001, 7e-05, 0.00259, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City769/insert", Data: [6]float64{31, 0.02986, 0.02986, 7e-05, 0.0246, 0.00061}},
		metric{Name: "Datastore/statement/MySQL/City770/insert", Data: [6]float64{31, 0.0048800000000000015, 0.0048800000000000015, 7e-05, 0.00066, 0}},
		metric{Name: "Datastore/statement/MySQL/City771/insert", Data: [6]float64{31, 0.01669, 0.01669, 7e-05, 0.01162, 0.00014}},
		metric{Name: "Datastore/statement/MySQL/City772/insert", Data: [6]float64{31, 0.005390000000000002, 0.005390000000000002, 7e-05, 0.00108, 0}},
		metric{Name: "Datastore/statement/MySQL/City773/insert", Data: [6]float64{31, 0.06943999999999999, 0.06943999999999999, 7e-05, 0.02516, 0.00135}},
		metric{Name: "Datastore/statement/MySQL/City774/insert", Data: [6]float64{31, 0.02104, 0.02104, 7e-05, 0.0087, 0.00014000000000000001}},
		metric{Name: "Datastore/statement/MySQL/City775/insert", Data: [6]float64{31, 0.005420000000000002, 0.005420000000000002, 7e-05, 0.00066, 0}},
		metric{Name: "Datastore/statement/MySQL/City776/insert", Data: [6]float64{31, 0.019080000000000003, 0.019080000000000003, 7e-05, 0.01101, 0.00013000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City777/insert", Data: [6]float64{31, 0.01463, 0.01463, 7e-05, 0.00675, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City778/insert", Data: [6]float64{31, 0.0050700000000000025, 0.0050700000000000025, 7e-05, 0.00066, 0}},
		metric{Name: "Datastore/statement/MySQL/City779/insert", Data: [6]float64{31, 0.0108, 0.0108, 7e-05, 0.00402, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City780/insert", Data: [6]float64{31, 0.0409, 0.0409, 7e-05, 0.03056, 0.0009500000000000001}},
		metric{Name: "Datastore/statement/MySQL/City781/insert", Data: [6]float64{31, 0.02452, 0.02452, 7e-05, 0.01596, 0.00027}},
		metric{Name: "Datastore/statement/MySQL/City782/insert", Data: [6]float64{31, 0.012310000000000003, 0.012310000000000003, 7e-05, 0.00772, 6e-05}},
		metric{Name: "Datastore/statement/MySQL/City783/insert", Data: [6]float64{31, 0.01663, 0.01663, 7e-05, 0.011, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City784/insert", Data: [6]float64{31, 0.011760000000000001, 0.011760000000000001, 7e-05, 0.00476, 3.0000000000000004e-05}},
		metric{Name: "Datastore/statement/MySQL/City785/insert", Data: [6]float64{31, 0.026840000000000006, 0.026840000000000006, 7e-05, 0.0147, 0.00025}},
		metric{Name: "Datastore/statement/MySQL/City786/insert", Data: [6]float64{31, 0.013260000000000001, 0.013260000000000001, 7e-05, 0.00384, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City787/insert", Data: [6]float64{31, 0.00979, 0.00979, 7e-05, 0.00496, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City788/insert", Data: [6]float64{31, 0.0063200000000000036, 0.0063200000000000036, 8e-05, 0.00093, 0}},
		metric{Name: "Datastore/statement/MySQL/City789/insert", Data: [6]float64{31, 0.005280000000000003, 0.005280000000000003, 7e-05, 0.00075, 0}},
		metric{Name: "Datastore/statement/MySQL/City790/insert", Data: [6]float64{31, 0.032729999999999974, 0.032729999999999974, 7e-05, 0.02783, 0.00077}},
		metric{Name: "Datastore/statement/MySQL/City791/insert", Data: [6]float64{31, 0.02947, 0.02947, 7e-05, 0.02053, 0.00043000000000000004}},
		metric{Name: "Datastore/statement/MySQL/City792/insert", Data: [6]float64{31, 0.008980000000000004, 0.008980000000000004, 7e-05, 0.00281, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City793/insert", Data: [6]float64{31, 0.020350000000000007, 0.020350000000000007, 7e-05, 0.01598, 0.00026}},
		metric{Name: "Datastore/statement/MySQL/City794/insert", Data: [6]float64{31, 0.006400000000000002, 0.006400000000000002, 7e-05, 0.00198, 0}},
		metric{Name: "Datastore/statement/MySQL/City795/insert", Data: [6]float64{31, 0.05075, 0.05075, 7e-05, 0.0268, 0.0010400000000000001}},
		metric{Name: "Datastore/statement/MySQL/City796/insert", Data: [6]float64{31, 0.010900000000000002, 0.010900000000000002, 7e-05, 0.00584, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City797/insert", Data: [6]float64{31, 0.014860000000000002, 0.014860000000000002, 7e-05, 0.006, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City798/insert", Data: [6]float64{31, 0.005000000000000003, 0.005000000000000003, 7e-05, 0.00055, 0}},
		metric{Name: "Datastore/statement/MySQL/City799/insert", Data: [6]float64{31, 0.011490000000000002, 0.011490000000000002, 7e-05, 0.00379, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City800/insert", Data: [6]float64{31, 0.02144, 0.02144, 7e-05, 0.01658, 0.00028}},
		metric{Name: "Datastore/statement/MySQL/City801/insert", Data: [6]float64{31, 0.030930000000000003, 0.030930000000000003, 7e-05, 0.02229, 0.00051}},
		metric{Name: "Datastore/statement/MySQL/City802/insert", Data: [6]float64{31, 0.013370000000000003, 0.013370000000000003, 7e-05, 0.00688, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City803/insert", Data: [6]float64{31, 0.009190000000000002, 0.009190000000000002, 7e-05, 0.00267, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City804/insert", Data: [6]float64{31, 0.019320000000000004, 0.019320000000000004, 7e-05, 0.01212, 0.00015999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City805/insert", Data: [6]float64{31, 0.004550000000000003, 0.004550000000000003, 7e-05, 0.00068, 0}},
		metric{Name: "Datastore/statement/MySQL/City806/insert", Data: [6]float64{31, 0.025160000000000002, 0.025160000000000002, 7e-05, 0.01169, 0.00021999999999999998}},
		metric{Name: "Datastore/statement/MySQL/City807/insert", Data: [6]float64{31, 0.005360000000000002, 0.005360000000000002, 7e-05, 0.0007, 0}},
		metric{Name: "Datastore/statement/MySQL/City808/insert", Data: [6]float64{31, 0.004680000000000001, 0.004680000000000001, 8e-05, 0.00049, 0}},
		metric{Name: "Datastore/statement/MySQL/City809/insert", Data: [6]float64{31, 0.03319999999999998, 0.03319999999999998, 7e-05, 0.02609, 0.00068}},
		metric{Name: "Datastore/statement/MySQL/City810/insert", Data: [6]float64{31, 0.009960000000000004, 0.009960000000000004, 7e-05, 0.00506, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City811/insert", Data: [6]float64{31, 0.02167, 0.02167, 7e-05, 0.01718, 0.0003}},
		metric{Name: "Datastore/statement/MySQL/City812/insert", Data: [6]float64{31, 0.019770000000000003, 0.019770000000000003, 7e-05, 0.01237, 0.00015999999999999999}},
		metric{Name: "Datastore/statement/MySQL/City813/insert", Data: [6]float64{31, 0.026959999999999998, 0.026959999999999998, 7e-05, 0.01536, 0.00027}},
		metric{Name: "Datastore/statement/MySQL/City814/insert", Data: [6]float64{31, 0.012880000000000003, 0.012880000000000003, 7e-05, 0.00816, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City815/insert", Data: [6]float64{31, 0.01936, 0.01936, 7e-05, 0.01015, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City816/insert", Data: [6]float64{31, 0.00829, 0.00829, 7e-05, 0.00365, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City817/insert", Data: [6]float64{31, 0.004850000000000004, 0.004850000000000004, 8e-05, 0.00072, 0}},
		metric{Name: "Datastore/statement/MySQL/City818/insert", Data: [6]float64{31, 0.004590000000000002, 0.004590000000000002, 7e-05, 0.00057, 0}},
		metric{Name: "Datastore/statement/MySQL/City819/insert", Data: [6]float64{31, 0.004740000000000004, 0.004740000000000004, 7e-05, 0.0007, 0}},
		metric{Name: "Datastore/statement/MySQL/City820/insert", Data: [6]float64{31, 0.024380000000000006, 0.024380000000000006, 7e-05, 0.01649, 0.00028000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City821/insert", Data: [6]float64{31, 0.005120000000000003, 0.005120000000000003, 7e-05, 0.00077, 0}},
		metric{Name: "Datastore/statement/MySQL/City822/insert", Data: [6]float64{31, 0.007870000000000002, 0.007870000000000002, 7e-05, 0.00289, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City823/insert", Data: [6]float64{31, 0.04348999999999999, 0.04348999999999999, 7e-05, 0.02629, 0.00085}},
		metric{Name: "Datastore/statement/MySQL/City824/insert", Data: [6]float64{31, 0.03411999999999999, 0.03411999999999999, 7e-05, 0.0247, 0.0006399999999999999}},
		metric{Name: "Datastore/statement/MySQL/City825/insert", Data: [6]float64{31, 0.02385, 0.02385, 8e-05, 0.01884, 0.00035}},
		metric{Name: "Datastore/statement/MySQL/City826/insert", Data: [6]float64{31, 0.011790000000000004, 0.011790000000000004, 8e-05, 0.00376, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City827/insert", Data: [6]float64{31, 0.006470000000000003, 0.006470000000000003, 7e-05, 0.00229, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City828/insert", Data: [6]float64{31, 0.005220000000000002, 0.005220000000000002, 7e-05, 0.0009, 0}},
		metric{Name: "Datastore/statement/MySQL/City829/insert", Data: [6]float64{31, 0.026350000000000002, 0.026350000000000002, 7e-05, 0.01966, 0.00039}},
		metric{Name: "Datastore/statement/MySQL/City830/insert", Data: [6]float64{31, 0.00909, 0.00909, 7e-05, 0.00302, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City831/insert", Data: [6]float64{31, 0.005350000000000002, 0.005350000000000002, 7e-05, 0.00094, 0}},
		metric{Name: "Datastore/statement/MySQL/City832/insert", Data: [6]float64{31, 0.01346, 0.01346, 7e-05, 0.00813, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City833/insert", Data: [6]float64{31, 0.005250000000000002, 0.005250000000000002, 8e-05, 0.00104, 0}},
		metric{Name: "Datastore/statement/MySQL/City834/insert", Data: [6]float64{31, 0.006420000000000002, 0.006420000000000002, 8e-05, 0.00112, 0}},
		metric{Name: "Datastore/statement/MySQL/City835/insert", Data: [6]float64{31, 0.04225, 0.04225, 7e-05, 0.03284, 0.00109}},
		metric{Name: "Datastore/statement/MySQL/City836/insert", Data: [6]float64{31, 0.01361, 0.01361, 7e-05, 0.00848, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City837/insert", Data: [6]float64{31, 0.008540000000000002, 0.008540000000000002, 7e-05, 0.00302, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City838/insert", Data: [6]float64{31, 0.06617999999999997, 0.06617999999999997, 7e-05, 0.0344, 0.0019000000000000002}},
		metric{Name: "Datastore/statement/MySQL/City839/insert", Data: [6]float64{31, 0.0336, 0.0336, 7e-05, 0.02808, 0.00079}},
		metric{Name: "Datastore/statement/MySQL/City840/insert", Data: [6]float64{31, 0.005270000000000003, 0.005270000000000003, 7e-05, 0.00105, 0}},
		metric{Name: "Datastore/statement/MySQL/City841/insert", Data: [6]float64{31, 0.010350000000000003, 0.010350000000000003, 7e-05, 0.00404, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City842/insert", Data: [6]float64{31, 0.010660000000000001, 0.010660000000000001, 7e-05, 0.00549, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City843/insert", Data: [6]float64{31, 0.016970000000000002, 0.016970000000000002, 7e-05, 0.01098, 0.00012}},
		metric{Name: "Datastore/statement/MySQL/City844/insert", Data: [6]float64{31, 0.01433, 0.01433, 7e-05, 0.00501, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City845/insert", Data: [6]float64{31, 0.007020000000000001, 0.007020000000000001, 7e-05, 0.00112, 0}},
		metric{Name: "Datastore/statement/MySQL/City846/insert", Data: [6]float64{31, 0.04421999999999997, 0.04421999999999997, 7e-05, 0.02998, 0.00098}},
		metric{Name: "Datastore/statement/MySQL/City847/insert", Data: [6]float64{31, 0.007540000000000001, 0.007540000000000001, 7e-05, 0.00341, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City848/insert", Data: [6]float64{31, 0.009770000000000003, 0.009770000000000003, 7e-05, 0.00351, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City849/insert", Data: [6]float64{31, 0.0054500000000000035, 0.0054500000000000035, 8e-05, 0.00119, 0}},
		metric{Name: "Datastore/statement/MySQL/City850/insert", Data: [6]float64{31, 0.009350000000000002, 0.009350000000000002, 7e-05, 0.00327, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City851/insert", Data: [6]float64{31, 0.00841, 0.00841, 7e-05, 0.00345, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City852/insert", Data: [6]float64{31, 0.02523, 0.02523, 7e-05, 0.01455, 0.00023}},
		metric{Name: "Datastore/statement/MySQL/City853/insert", Data: [6]float64{31, 0.008690000000000002, 0.008690000000000002, 7e-05, 0.00285, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City854/insert", Data: [6]float64{31, 0.004780000000000003, 0.004780000000000003, 7e-05, 0.00084, 0}},
		metric{Name: "Datastore/statement/MySQL/City855/insert", Data: [6]float64{31, 0.018849999999999995, 0.018849999999999995, 7e-05, 0.00974, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City856/insert", Data: [6]float64{31, 0.010020000000000003, 0.010020000000000003, 7e-05, 0.00617, 4e-05}},
		metric{Name: "Datastore/statement/MySQL/City857/insert", Data: [6]float64{31, 0.005250000000000004, 0.005250000000000004, 8e-05, 0.00117, 0}},
		metric{Name: "Datastore/statement/MySQL/City858/insert", Data: [6]float64{31, 0.020210000000000002, 0.020210000000000002, 7e-05, 0.01154, 0.00015}},
		metric{Name: "Datastore/statement/MySQL/City859/insert", Data: [6]float64{31, 0.024650000000000002, 0.024650000000000002, 7e-05, 0.01968, 0.00039}},
		metric{Name: "Datastore/statement/MySQL/City860/insert", Data: [6]float64{31, 0.028480000000000002, 0.028480000000000002, 7e-05, 0.01362, 0.00023}},
		metric{Name: "Datastore/statement/MySQL/City861/insert", Data: [6]float64{31, 0.040769999999999994, 0.040769999999999994, 8e-05, 0.02018, 0.00067}},
		metric{Name: "Datastore/statement/MySQL/City862/insert", Data: [6]float64{31, 0.06144999999999999, 0.06144999999999999, 7e-05, 0.03144, 0.00145}},
		metric{Name: "Datastore/statement/MySQL/City863/insert", Data: [6]float64{31, 0.030120000000000004, 0.030120000000000004, 7e-05, 0.01975, 0.00041}},
		metric{Name: "Datastore/statement/MySQL/City864/insert", Data: [6]float64{31, 0.00628, 0.00628, 7e-05, 0.00125, 0}},
		metric{Name: "Datastore/statement/MySQL/City865/insert", Data: [6]float64{31, 0.02926, 0.02926, 7e-05, 0.01325, 0.00031}},
		metric{Name: "Datastore/statement/MySQL/City866/insert", Data: [6]float64{31, 0.005130000000000003, 0.005130000000000003, 7e-05, 0.00136, 0}},
		metric{Name: "Datastore/statement/MySQL/City867/insert", Data: [6]float64{31, 0.0053500000000000015, 0.0053500000000000015, 7e-05, 0.00124, 0}},
		metric{Name: "Datastore/statement/MySQL/City868/insert", Data: [6]float64{31, 0.004810000000000001, 0.004810000000000001, 7e-05, 0.00129, 0}},
		metric{Name: "Datastore/statement/MySQL/City869/insert", Data: [6]float64{31, 0.01648, 0.01648, 7e-05, 0.00619, 7.000000000000001e-05}},
		metric{Name: "Datastore/statement/MySQL/City870/insert", Data: [6]float64{31, 0.004640000000000003, 0.004640000000000003, 8e-05, 0.00101, 0}},
		metric{Name: "Datastore/statement/MySQL/City871/insert", Data: [6]float64{31, 0.010310000000000003, 0.010310000000000003, 7e-05, 0.00558, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City872/insert", Data: [6]float64{31, 0.004910000000000003, 0.004910000000000003, 8e-05, 0.00106, 0}},
		metric{Name: "Datastore/statement/MySQL/City873/insert", Data: [6]float64{31, 0.004550000000000003, 0.004550000000000003, 7e-05, 0.00094, 0}},
		metric{Name: "Datastore/statement/MySQL/City874/insert", Data: [6]float64{31, 0.005250000000000003, 0.005250000000000003, 8e-05, 0.00118, 0}},
		metric{Name: "Datastore/statement/MySQL/City875/insert", Data: [6]float64{31, 0.004320000000000002, 0.004320000000000002, 8e-05, 0.00086, 0}},
		metric{Name: "Datastore/statement/MySQL/City876/insert", Data: [6]float64{31, 0.010860000000000003, 0.010860000000000003, 7e-05, 0.00572, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City877/insert", Data: [6]float64{31, 0.029560000000000003, 0.029560000000000003, 8e-05, 0.0254, 0.00065}},
		metric{Name: "Datastore/statement/MySQL/City878/insert", Data: [6]float64{31, 0.004730000000000002, 0.004730000000000002, 7e-05, 0.00097, 0}},
		metric{Name: "Datastore/statement/MySQL/City879/insert", Data: [6]float64{31, 0.01406, 0.01406, 7e-05, 0.00866, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City880/insert", Data: [6]float64{31, 0.008610000000000001, 0.008610000000000001, 7e-05, 0.00422, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City881/insert", Data: [6]float64{31, 0.007870000000000004, 0.007870000000000004, 8e-05, 0.00386, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City882/insert", Data: [6]float64{31, 0.017570000000000002, 0.017570000000000002, 8e-05, 0.01336, 0.00018}},
		metric{Name: "Datastore/statement/MySQL/City883/insert", Data: [6]float64{31, 0.004200000000000002, 0.004200000000000002, 8e-05, 0.00079, 0}},
		metric{Name: "Datastore/statement/MySQL/City884/insert", Data: [6]float64{31, 0.007370000000000003, 0.007370000000000003, 8e-05, 0.00342, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City885/insert", Data: [6]float64{31, 0.004220000000000002, 0.004220000000000002, 8e-05, 0.00067, 0}},
		metric{Name: "Datastore/statement/MySQL/City886/insert", Data: [6]float64{31, 0.031200000000000002, 0.031200000000000002, 7e-05, 0.02565, 0.00066}},
		metric{Name: "Datastore/statement/MySQL/City887/insert", Data: [6]float64{31, 0.006030000000000002, 0.006030000000000002, 8e-05, 0.00229, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City888/insert", Data: [6]float64{31, 0.02821, 0.02821, 7e-05, 0.01146, 0.00020999999999999998}},
		metric{Name: "Datastore/statement/MySQL/City889/insert", Data: [6]float64{31, 0.02128, 0.02128, 7e-05, 0.00812, 0.00014}},
		metric{Name: "Datastore/statement/MySQL/City890/insert", Data: [6]float64{31, 0.004720000000000001, 0.004720000000000001, 8e-05, 0.00078, 0}},
		metric{Name: "Datastore/statement/MySQL/City891/insert", Data: [6]float64{31, 0.014320000000000003, 0.014320000000000003, 7e-05, 0.00864, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City892/insert", Data: [6]float64{31, 0.00957, 0.00957, 8e-05, 0.0019, 0}},
		metric{Name: "Datastore/statement/MySQL/City893/insert", Data: [6]float64{31, 0.0047300000000000016, 0.0047300000000000016, 8e-05, 0.0008, 0}},
		metric{Name: "Datastore/statement/MySQL/City894/insert", Data: [6]float64{31, 0.005120000000000003, 0.005120000000000003, 7e-05, 0.00091, 0}},
		metric{Name: "Datastore/statement/MySQL/City895/insert", Data: [6]float64{31, 0.025190000000000004, 0.025190000000000004, 8e-05, 0.01536, 0.00026000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City896/insert", Data: [6]float64{31, 0.023620000000000006, 0.023620000000000006, 7e-05, 0.01459, 0.00022}},
		metric{Name: "Datastore/statement/MySQL/City897/insert", Data: [6]float64{31, 0.005460000000000001, 0.005460000000000001, 7e-05, 0.00057, 0}},
		metric{Name: "Datastore/statement/MySQL/City898/insert", Data: [6]float64{31, 0.007390000000000002, 0.007390000000000002, 8e-05, 0.00136, 0}},
		metric{Name: "Datastore/statement/MySQL/City899/insert", Data: [6]float64{31, 0.06611999999999998, 0.06611999999999998, 8e-05, 0.03633, 0.0019199999999999998}},
		metric{Name: "Datastore/statement/MySQL/City900/insert", Data: [6]float64{31, 0.02688, 0.02688, 8e-05, 0.01704, 0.00030000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City901/insert", Data: [6]float64{31, 0.010920000000000003, 0.010920000000000003, 8e-05, 0.00536, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City902/insert", Data: [6]float64{31, 0.01159, 0.01159, 7e-05, 0.00579, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City903/insert", Data: [6]float64{31, 0.016210000000000002, 0.016210000000000002, 7e-05, 0.01061, 0.00011}},
		metric{Name: "Datastore/statement/MySQL/City904/insert", Data: [6]float64{31, 0.04205999999999997, 0.04205999999999997, 7e-05, 0.02804, 0.00086}},
		metric{Name: "Datastore/statement/MySQL/City905/insert", Data: [6]float64{31, 0.01838, 0.01838, 7e-05, 0.01319, 0.00017}},
		metric{Name: "Datastore/statement/MySQL/City906/insert", Data: [6]float64{31, 0.03813999999999998, 0.03813999999999998, 8e-05, 0.03109, 0.00097}},
		metric{Name: "Datastore/statement/MySQL/City907/insert", Data: [6]float64{31, 0.014760000000000002, 0.014760000000000002, 7e-05, 0.00618, 5e-05}},
		metric{Name: "Datastore/statement/MySQL/City908/insert", Data: [6]float64{31, 0.01818, 0.01818, 7e-05, 0.0112, 0.00013}},
		metric{Name: "Datastore/statement/MySQL/City909/insert", Data: [6]float64{31, 0.025740000000000002, 0.025740000000000002, 8e-05, 0.01382, 0.00021}},
		metric{Name: "Datastore/statement/MySQL/City910/insert", Data: [6]float64{31, 0.009440000000000002, 0.009440000000000002, 8e-05, 0.00456, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City911/insert", Data: [6]float64{31, 0.006880000000000003, 0.006880000000000003, 8e-05, 0.00286, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City912/insert", Data: [6]float64{31, 0.03289999999999999, 0.03289999999999999, 8e-05, 0.02902, 0.00084}},
		metric{Name: "Datastore/statement/MySQL/City913/insert", Data: [6]float64{31, 0.027620000000000002, 0.027620000000000002, 7e-05, 0.0235, 0.00055}},
		metric{Name: "Datastore/statement/MySQL/City914/insert", Data: [6]float64{31, 0.034719999999999994, 0.034719999999999994, 8e-05, 0.03086, 0.00095}},
		metric{Name: "Datastore/statement/MySQL/City915/insert", Data: [6]float64{31, 0.009680000000000001, 0.009680000000000001, 8e-05, 0.00586, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City916/insert", Data: [6]float64{31, 0.004050000000000002, 0.004050000000000002, 8e-05, 0.00032, 0}},
		metric{Name: "Datastore/statement/MySQL/City917/insert", Data: [6]float64{31, 0.0039200000000000025, 0.0039200000000000025, 7e-05, 0.00041, 0}},
		metric{Name: "Datastore/statement/MySQL/City918/insert", Data: [6]float64{31, 0.004330000000000002, 0.004330000000000002, 7e-05, 0.00054, 0}},
		metric{Name: "Datastore/statement/MySQL/City919/insert", Data: [6]float64{31, 0.005730000000000003, 0.005730000000000003, 7e-05, 0.00176, 0}},
		metric{Name: "Datastore/statement/MySQL/City920/insert", Data: [6]float64{31, 0.003980000000000003, 0.003980000000000003, 7e-05, 0.0003, 0}},
		metric{Name: "Datastore/statement/MySQL/City921/insert", Data: [6]float64{31, 0.004400000000000002, 0.004400000000000002, 8e-05, 0.00064, 0}},
		metric{Name: "Datastore/statement/MySQL/City922/insert", Data: [6]float64{31, 0.029499999999999995, 0.029499999999999995, 7e-05, 0.01438, 0.00029}},
		metric{Name: "Datastore/statement/MySQL/City923/insert", Data: [6]float64{31, 0.02653, 0.02653, 7e-05, 0.02233, 0.0005}},
		metric{Name: "Datastore/statement/MySQL/City924/insert", Data: [6]float64{31, 0.004420000000000002, 0.004420000000000002, 8e-05, 0.00042, 0}},
		metric{Name: "Datastore/statement/MySQL/City925/insert", Data: [6]float64{31, 0.004340000000000001, 0.004340000000000001, 7e-05, 0.00037, 0}},
		metric{Name: "Datastore/statement/MySQL/City926/insert", Data: [6]float64{31, 0.03166, 0.03166, 7e-05, 0.02735, 0.00075}},
		metric{Name: "Datastore/statement/MySQL/City927/insert", Data: [6]float64{31, 0.03383999999999999, 0.03383999999999999, 7e-05, 0.02877, 0.00083}},
		metric{Name: "Datastore/statement/MySQL/City928/insert", Data: [6]float64{31, 0.0040700000000000024, 0.0040700000000000024, 7e-05, 0.00036, 0}},
		metric{Name: "Datastore/statement/MySQL/City929/insert", Data: [6]float64{31, 0.005580000000000003, 0.005580000000000003, 8e-05, 0.00145, 0}},
		metric{Name: "Datastore/statement/MySQL/City930/insert", Data: [6]float64{31, 0.0039000000000000016, 0.0039000000000000016, 7e-05, 0.0003, 0}},
		metric{Name: "Datastore/statement/MySQL/City931/insert", Data: [6]float64{31, 0.010320000000000001, 0.010320000000000001, 8e-05, 0.00557, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City932/insert", Data: [6]float64{31, 0.03195999999999999, 0.03195999999999999, 7e-05, 0.02799, 0.00078}},
		metric{Name: "Datastore/statement/MySQL/City933/insert", Data: [6]float64{31, 0.009930000000000003, 0.009930000000000003, 7e-05, 0.00439, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City934/insert", Data: [6]float64{31, 0.003880000000000002, 0.003880000000000002, 7e-05, 0.00031, 0}},
		metric{Name: "Datastore/statement/MySQL/City935/insert", Data: [6]float64{31, 0.008980000000000002, 0.008980000000000002, 8e-05, 0.00518, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City936/insert", Data: [6]float64{31, 0.004010000000000002, 0.004010000000000002, 7e-05, 0.00049, 0}},
		metric{Name: "Datastore/statement/MySQL/City937/insert", Data: [6]float64{31, 0.003590000000000002, 0.003590000000000002, 7e-05, 0.00032, 0}},
		metric{Name: "Datastore/statement/MySQL/City938/insert", Data: [6]float64{31, 0.02961, 0.02961, 7e-05, 0.02431, 0.00059}},
		metric{Name: "Datastore/statement/MySQL/City939/insert", Data: [6]float64{31, 0.013930000000000001, 0.013930000000000001, 7e-05, 0.00837, 7e-05}},
		metric{Name: "Datastore/statement/MySQL/City940/insert", Data: [6]float64{31, 0.01889, 0.01889, 7e-05, 0.01523, 0.00023}},
		metric{Name: "Datastore/statement/MySQL/City941/insert", Data: [6]float64{31, 0.003910000000000002, 0.003910000000000002, 7e-05, 0.00033, 0}},
		metric{Name: "Datastore/statement/MySQL/City942/insert", Data: [6]float64{31, 0.029290000000000004, 0.029290000000000004, 8e-05, 0.01386, 0.00031}},
		metric{Name: "Datastore/statement/MySQL/City943/insert", Data: [6]float64{31, 0.04140999999999998, 0.04140999999999998, 7e-05, 0.03762, 0.00142}},
		metric{Name: "Datastore/statement/MySQL/City944/insert", Data: [6]float64{31, 0.003500000000000003, 0.003500000000000003, 7e-05, 0.00036, 0}},
		metric{Name: "Datastore/statement/MySQL/City945/insert", Data: [6]float64{31, 0.027170000000000003, 0.027170000000000003, 7e-05, 0.02331, 0.00054}},
		metric{Name: "Datastore/statement/MySQL/City946/insert", Data: [6]float64{31, 0.036250000000000004, 0.036250000000000004, 7e-05, 0.01716, 0.00041}},
		metric{Name: "Datastore/statement/MySQL/City947/insert", Data: [6]float64{31, 0.005320000000000002, 0.005320000000000002, 7e-05, 0.00168, 0}},
		metric{Name: "Datastore/statement/MySQL/City948/insert", Data: [6]float64{31, 0.015470000000000003, 0.015470000000000003, 7e-05, 0.01158, 0.00013}},
		metric{Name: "Datastore/statement/MySQL/City949/insert", Data: [6]float64{31, 0.01552, 0.01552, 8e-05, 0.01177, 0.00014}},
		metric{Name: "Datastore/statement/MySQL/City950/insert", Data: [6]float64{31, 0.003960000000000003, 0.003960000000000003, 7e-05, 0.00038, 0}},
		metric{Name: "Datastore/statement/MySQL/City951/insert", Data: [6]float64{31, 0.003900000000000003, 0.003900000000000003, 7e-05, 0.00037, 0}},
		metric{Name: "Datastore/statement/MySQL/City952/insert", Data: [6]float64{31, 0.02093, 0.02093, 7e-05, 0.01704, 0.00029}},
		metric{Name: "Datastore/statement/MySQL/City953/insert", Data: [6]float64{31, 0.043269999999999975, 0.043269999999999975, 8e-05, 0.02367, 0.0007}},
		metric{Name: "Datastore/statement/MySQL/City954/insert", Data: [6]float64{31, 0.004710000000000001, 0.004710000000000001, 7e-05, 0.00065, 0}},
		metric{Name: "Datastore/statement/MySQL/City955/insert", Data: [6]float64{31, 0.00474, 0.00474, 7e-05, 0.00051, 0}},
		metric{Name: "Datastore/statement/MySQL/City956/insert", Data: [6]float64{31, 0.003980000000000002, 0.003980000000000002, 7e-05, 0.00047, 0}},
		metric{Name: "Datastore/statement/MySQL/City957/insert", Data: [6]float64{31, 0.03293, 0.03293, 7e-05, 0.02124, 0.00051}},
		metric{Name: "Datastore/statement/MySQL/City958/insert", Data: [6]float64{31, 0.004340000000000003, 0.004340000000000003, 8e-05, 0.00037, 0}},
		metric{Name: "Datastore/statement/MySQL/City959/insert", Data: [6]float64{31, 0.016120000000000002, 0.016120000000000002, 7e-05, 0.00966, 0.0001}},
		metric{Name: "Datastore/statement/MySQL/City960/insert", Data: [6]float64{31, 0.005210000000000001, 0.005210000000000001, 7e-05, 0.00114, 0}},
		metric{Name: "Datastore/statement/MySQL/City961/insert", Data: [6]float64{31, 0.0039400000000000025, 0.0039400000000000025, 7e-05, 0.00038, 0}},
		metric{Name: "Datastore/statement/MySQL/City962/insert", Data: [6]float64{31, 0.004570000000000002, 0.004570000000000002, 7e-05, 0.00118, 0}},
		metric{Name: "Datastore/statement/MySQL/City963/insert", Data: [6]float64{31, 0.00442, 0.00442, 7e-05, 0.00076, 0}},
		metric{Name: "Datastore/statement/MySQL/City964/insert", Data: [6]float64{31, 0.02559, 0.02559, 7e-05, 0.02054, 0.00042}},
		metric{Name: "Datastore/statement/MySQL/City965/insert", Data: [6]float64{31, 0.004980000000000002, 0.004980000000000002, 7e-05, 0.00129, 0}},
		metric{Name: "Datastore/statement/MySQL/City966/insert", Data: [6]float64{31, 0.0038500000000000023, 0.0038500000000000023, 7e-05, 0.00036, 0}},
		metric{Name: "Datastore/statement/MySQL/City967/insert", Data: [6]float64{31, 0.0037200000000000024, 0.0037200000000000024, 8e-05, 0.00032, 0}},
		metric{Name: "Datastore/statement/MySQL/City968/insert", Data: [6]float64{31, 0.028239999999999998, 0.028239999999999998, 8e-05, 0.01275, 0.00030000000000000003}},
		metric{Name: "Datastore/statement/MySQL/City969/insert", Data: [6]float64{31, 0.003820000000000002, 0.003820000000000002, 7e-05, 0.00033, 0}},
		metric{Name: "Datastore/statement/MySQL/City970/insert", Data: [6]float64{31, 0.015620000000000002, 0.015620000000000002, 8e-05, 0.01155, 0.00013}},
		metric{Name: "Datastore/statement/MySQL/City971/insert", Data: [6]float64{31, 0.0042400000000000024, 0.0042400000000000024, 7e-05, 0.00059, 0}},
		metric{Name: "Datastore/statement/MySQL/City972/insert", Data: [6]float64{31, 0.028330000000000004, 0.028330000000000004, 7e-05, 0.01529, 0.00031}},
		metric{Name: "Datastore/statement/MySQL/City973/insert", Data: [6]float64{31, 0.029450000000000004, 0.029450000000000004, 7e-05, 0.02104, 0.00045000000000000004}},
		metric{Name: "Datastore/statement/MySQL/City974/insert", Data: [6]float64{31, 0.004590000000000003, 0.004590000000000003, 8e-05, 0.00063, 0}},
		metric{Name: "Datastore/statement/MySQL/City975/insert", Data: [6]float64{31, 0.01827, 0.01827, 8e-05, 0.01382, 0.00019}},
		metric{Name: "Datastore/statement/MySQL/City976/insert", Data: [6]float64{31, 0.01718, 0.01718, 8e-05, 0.01308, 0.00017}},
		metric{Name: "Datastore/statement/MySQL/City977/insert", Data: [6]float64{31, 0.03017, 0.03017, 7e-05, 0.0253, 0.00064}},
		metric{Name: "Datastore/statement/MySQL/City978/insert", Data: [6]float64{31, 0.017470000000000003, 0.017470000000000003, 8e-05, 0.0129, 0.00017}},
		metric{Name: "Datastore/statement/MySQL/City979/insert", Data: [6]float64{31, 0.004540000000000003, 0.004540000000000003, 8e-05, 0.00058, 0}},
		metric{Name: "Datastore/statement/MySQL/City980/insert", Data: [6]float64{31, 0.0070100000000000015, 0.0070100000000000015, 7e-05, 0.00295, 1e-05}},
		metric{Name: "Datastore/statement/MySQL/City981/insert", Data: [6]float64{31, 0.0037400000000000016, 0.0037400000000000016, 8e-05, 0.00033, 0}},
		metric{Name: "Datastore/statement/MySQL/City982/insert", Data: [6]float64{31, 0.0037100000000000015, 0.0037100000000000015, 7e-05, 0.0004, 0}},
		metric{Name: "Datastore/statement/MySQL/City983/insert", Data: [6]float64{31, 0.009300000000000004, 0.009300000000000004, 8e-05, 0.00547, 3e-05}},
		metric{Name: "Datastore/statement/MySQL/City984/insert", Data: [6]float64{31, 0.005080000000000004, 0.005080000000000004, 7e-05, 0.00136, 0}},
		metric{Name: "Datastore/statement/MySQL/City985/insert", Data: [6]float64{31, 0.027290000000000002, 0.027290000000000002, 8e-05, 0.02051, 0.00043000000000000004}},
		metric{Name: "Datastore/statement/MySQL/City986/insert", Data: [6]float64{31, 0.004360000000000001, 0.004360000000000001, 8e-05, 0.0005, 0}},
		metric{Name: "Datastore/statement/MySQL/City987/insert", Data: [6]float64{31, 0.010300000000000002, 0.010300000000000002, 8e-05, 0.00428, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City988/insert", Data: [6]float64{31, 0.03402, 0.03402, 7e-05, 0.02985, 0.00089}},
		metric{Name: "Datastore/statement/MySQL/City989/insert", Data: [6]float64{31, 0.004360000000000001, 0.004360000000000001, 8e-05, 0.00048, 0}},
		metric{Name: "Datastore/statement/MySQL/City990/insert", Data: [6]float64{31, 0.004380000000000001, 0.004380000000000001, 8e-05, 0.0004, 0}},
		metric{Name: "Datastore/statement/MySQL/City991/insert", Data: [6]float64{31, 0.011840000000000003, 0.011840000000000003, 8e-05, 0.00401, 2e-05}},
		metric{Name: "Datastore/statement/MySQL/City992/insert", Data: [6]float64{31, 0.020540000000000003, 0.020540000000000003, 8e-05, 0.01501, 0.00023}},
		metric{Name: "Datastore/statement/MySQL/City993/insert", Data: [6]float64{31, 0.004620000000000002, 0.004620000000000002, 8e-05, 0.00059, 0}},
		metric{Name: "Datastore/statement/MySQL/City994/insert", Data: [6]float64{31, 0.06958999999999997, 0.06958999999999997, 7e-05, 0.04748, 0.00244}},
		metric{Name: "Datastore/statement/MySQL/City995/insert", Data: [6]float64{31, 0.026510000000000002, 0.026510000000000002, 8e-05, 0.01233, 0.00026}},
		metric{Name: "Datastore/statement/MySQL/City996/insert", Data: [6]float64{31, 0.01237, 0.01237, 8e-05, 0.0088, 8e-05}},
		metric{Name: "Datastore/statement/MySQL/City997/insert", Data: [6]float64{31, 0.02997, 0.02997, 8e-05, 0.02659, 0.00071}},
		metric{Name: "Datastore/statement/MySQL/City998/insert", Data: [6]float64{31, 0.003670000000000002, 0.003670000000000002, 8e-05, 0.00027, 0}},
		metric{Name: "Datastore/statement/MySQL/City999/insert", Data: [6]float64{31, 0.004980000000000001, 0.004980000000000001, 7e-05, 0.00167, 0}},
		metric{Name: "Errors/WebTransaction/Uri/myblog2/mycity.php", Data: [6]float64{3, 0, 0, 0, 0, 0}},
		metric{Name: "Errors/all", Data: [6]float64{3, 0, 0, 0, 0, 0}},
		metric{Name: "Errors/allWeb", Data: [6]float64{3, 0, 0, 0, 0, 0}},
		metric{Name: "HttpDispatcher", Data: [6]float64{361, 33.18916000000001, 0, 0.00041, 0.90685, 18.556699999999992}},
		metric{Name: "Instance/Reporting", Data: [6]float64{21, 0, 0, 0, 0, 0}},
		metric{Name: "Memory/Physical", Data: [6]float64{361, 5061.906420000001, 0, 13.49219, 15.86719, 71101.64678999998}},
		metric{Name: "Supportability/AnalyticsEvents/TotalEventsSeen", Data: [6]float64{361, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/AnalyticsEvents/TotalEventsSent", Data: [6]float64{361, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/Events/Customer/Seen", Data: [6]float64{0, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/Events/Customer/Sent", Data: [6]float64{0, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysql_query", Data: [6]float64{94, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::multi_query", Data: [6]float64{31000, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::mysqli", Data: [6]float64{106, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::query", Data: [6]float64{51, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli::stmt_init", Data: [6]float64{24, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_connect", Data: [6]float64{41, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_prepare", Data: [6]float64{25, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_query", Data: [6]float64{75, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt::bind_param", Data: [6]float64{24, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt::execute", Data: [6]float64{24, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt::prepare", Data: [6]float64{24, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_bind_param", Data: [6]float64{41, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_execute", Data: [6]float64{25016, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_init", Data: [6]float64{16, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/mysqli_stmt_prepare", Data: [6]float64{16, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/InstrumentedFunction/sleepy_time", Data: [6]float64{35, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/MetricsDropped", Data: [6]float64{1310, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/api/add_custom_tracer", Data: [6]float64{35, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/api/get_browser_timing_footer", Data: [6]float64{152, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/api/get_browser_timing_header", Data: [6]float64{152, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/api/name_transaction", Data: [6]float64{32, 0, 0, 0, 0, 0}},
		metric{Name: "Supportability/execute/user/call_count", Data: [6]float64{361, 0.00035000000000000027, 0.00035000000000000027, 0, 1e-05, 0}},
		metric{Name: "WebTransaction", Data: [6]float64{361, 33.18916000000001, 6.48498, 0.00041, 0.90685, 18.556699999999992}},
		metric{Name: "WebTransaction/Custom/my display php page", Data: [6]float64{32, 0.5338700000000002, 0.5042800000000001, 0.0011, 0.03795, 0.01233}},
		metric{Name: "WebTransaction/Uri/myblog2/accentquery.php", Data: [6]float64{28, 0.66593, 0.40279, 0.00078, 0.09748, 0.038239999999999996}},
		metric{Name: "WebTransaction/Uri/myblog2/backslashquote.php", Data: [6]float64{23, 0.8756999999999999, 0.71145, 0.00394, 0.0558, 0.03807}},
		metric{Name: "WebTransaction/Uri/myblog2/displayfunct.php", Data: [6]float64{35, 1.2758399999999999, 0.82581, 0.00112, 0.16208, 0.08119000000000003}},
		metric{Name: "WebTransaction/Uri/myblog2/district.php", Data: [6]float64{16, 0.37147, 0.34234, 0.00213, 0.06418, 0.01435}},
		metric{Name: "WebTransaction/Uri/myblog2/districtoo.php", Data: [6]float64{24, 0.50529, 0.47634000000000004, 0.00141, 0.05316, 0.01607}},
		metric{Name: "WebTransaction/Uri/myblog2/mycity.php", Data: [6]float64{25, 9.297749999999999, 0.80901, 0.12335, 0.69294, 4.719230000000001}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_113.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_122.php", Data: [6]float64{1, 0.00057, 0.00057, 0.00057, 0.00057, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_125.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_129.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_150.php", Data: [6]float64{1, 0.00069, 0.00069, 0.00069, 0.00069, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_162.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_17.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_181.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_185.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_191.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_199.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_202.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_208.php", Data: [6]float64{1, 0.00069, 0.00069, 0.00069, 0.00069, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_210.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_22.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_222.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_226.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_227.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_243.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_25.php", Data: [6]float64{1, 0.00045, 0.00045, 0.00045, 0.00045, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_252.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_261.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_272.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_280.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_287.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_29.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_293.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_305.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_312.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_34.php", Data: [6]float64{1, 0.00054, 0.00054, 0.00054, 0.00054, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_346.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_352.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_353.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_357.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_360.php", Data: [6]float64{1, 0.00061, 0.00061, 0.00061, 0.00061, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_365.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_369.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_376.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_378.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_386.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_389.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_401.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_405.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_42.php", Data: [6]float64{1, 0.00057, 0.00057, 0.00057, 0.00057, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_437.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_44.php", Data: [6]float64{1, 0.00041, 0.00041, 0.00041, 0.00041, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_441.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_466.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_489.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_496.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_497.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_498.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_502.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_510.php", Data: [6]float64{1, 0.00064, 0.00064, 0.00064, 0.00064, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_513.php", Data: [6]float64{2, 0.00109, 0.00109, 0.00051, 0.00058, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_546.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_553.php", Data: [6]float64{1, 0.00045, 0.00045, 0.00045, 0.00045, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_559.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_568.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_576.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_578.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_582.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_591.php", Data: [6]float64{2, 0.00093, 0.00093, 0.00044, 0.00049, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_597.php", Data: [6]float64{2, 0.00095, 0.00095, 0.00047, 0.00048, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_600.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_602.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_610.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_62.php", Data: [6]float64{1, 0.00051, 0.00051, 0.00051, 0.00051, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_626.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_635.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_642.php", Data: [6]float64{1, 0.00043, 0.00043, 0.00043, 0.00043, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_651.php", Data: [6]float64{1, 0.00064, 0.00064, 0.00064, 0.00064, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_654.php", Data: [6]float64{1, 0.00061, 0.00061, 0.00061, 0.00061, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_655.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_666.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_668.php", Data: [6]float64{1, 0.00057, 0.00057, 0.00057, 0.00057, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_680.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_686.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_707.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_708.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_730.php", Data: [6]float64{1, 0.0006, 0.0006, 0.0006, 0.0006, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_736.php", Data: [6]float64{1, 0.00058, 0.00058, 0.00058, 0.00058, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_738.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_74.php", Data: [6]float64{1, 0.00049, 0.00049, 0.00049, 0.00049, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_744.php", Data: [6]float64{1, 0.00054, 0.00054, 0.00054, 0.00054, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_745.php", Data: [6]float64{1, 0.00042, 0.00042, 0.00042, 0.00042, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_747.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_779.php", Data: [6]float64{1, 0.00061, 0.00061, 0.00061, 0.00061, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_780.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_803.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_812.php", Data: [6]float64{1, 0.00059, 0.00059, 0.00059, 0.00059, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_828.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_829.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_839.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_85.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_857.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_865.php", Data: [6]float64{1, 0.00053, 0.00053, 0.00053, 0.00053, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_867.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_87.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_870.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_872.php", Data: [6]float64{1, 0.00046, 0.00046, 0.00046, 0.00046, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_882.php", Data: [6]float64{1, 0.00054, 0.00054, 0.00054, 0.00054, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_889.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_902.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_905.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_915.php", Data: [6]float64{1, 0.00044, 0.00044, 0.00044, 0.00044, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_918.php", Data: [6]float64{1, 0.00062, 0.00062, 0.00062, 0.00062, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_937.php", Data: [6]float64{1, 0.0005, 0.0005, 0.0005, 0.0005, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_939.php", Data: [6]float64{1, 0.00055, 0.00055, 0.00055, 0.00055, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_943.php", Data: [6]float64{2, 0.00107, 0.00107, 0.0005, 0.00057, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_944.php", Data: [6]float64{1, 0.00048, 0.00048, 0.00048, 0.00048, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_946.php", Data: [6]float64{1, 0.00047, 0.00047, 0.00047, 0.00047, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_957.php", Data: [6]float64{1, 0.00063, 0.00063, 0.00063, 0.00063, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_961.php", Data: [6]float64{1, 0.00056, 0.00056, 0.00056, 0.00056, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_978.php", Data: [6]float64{1, 0.0006, 0.0006, 0.0006, 0.0006, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/newPost_rum_995.php", Data: [6]float64{1, 0.00052, 0.00052, 0.00052, 0.00052, 0}},
		metric{Name: "WebTransaction/Uri/myblog2/poptables.php", Data: [6]float64{31, 18.863939999999996, 1.7271100000000001, 0.14208, 0.90685, 13.599230000000002}},
		metric{Name: "WebTransaction/Uri/myblog2/subquery.php", Data: [6]float64{27, 0.73692, 0.6234000000000001, 0.00147, 0.10433, 0.03799000000000001}},
	}
)
